@@ -19,7 +19,7 @@ import Darcs.Patch.Witnesses.Sealed
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Unsafe
 import Darcs.Patch.Witnesses.Ordered
-import Darcs.Patch.Prim.V1.Core ( FilePatchType( Hunk, TokReplace ), isIdentity )
+import Darcs.Patch.Prim.V1.Core ( FilePatchType( Hunk ), isIdentity )
 import qualified Darcs.Patch.Prim.V1.Core as Prim ( Prim( FP ) )
 import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import qualified Darcs.Patch.V2.Prim as V2 ( Prim(..) )
@@ -124,7 +124,7 @@ propFail n xs = sizeTree xs < n
 ----------------------------------------------------------------------
 -- ** FilePatchType generators
 
-aHunk :: forall wX wY . Content -> Gen (FilePatchType wX wY)
+aHunk :: Content -> Gen (Int, [BC.ByteString], [BC.ByteString])
 aHunk content
  = sized $ \n ->
      do pos <- choose (1, contentLen+1)
@@ -153,20 +153,20 @@ aHunk content
                       ]
         new <- vectorOf newLen aLine
         let old = take oldLen $ drop prefixLen $ content
-        return $ Hunk pos old new
+        return (pos, old, new)
   where
       contentLen = length content
 
-aTokReplace :: forall wX wY . Content -> Gen (FilePatchType wX wY)
+aTokReplace :: Content -> Gen (String, String, String)
 aTokReplace []
   = do w <- vectorOf 1 alpha
        w' <- vectorOf 1 alpha
-       return $ TokReplace defaultToks w w'
+       return (defaultToks, w, w')
 aTokReplace content
   = do let fileWords = concatMap BC.words content
        wB <- elements fileWords
        w' <- alphaBS `notIn` fileWords
-       return $ TokReplace defaultToks (BC.unpack wB) (BC.unpack w')
+       return (defaultToks, BC.unpack wB, BC.unpack w')
   where
       alphaBS = do x <- alpha; return $ BC.pack [x]
 
@@ -175,14 +175,14 @@ aTokReplace content
 
 aHunkP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,File) -> Gen (prim wX wY)
 aHunkP (path,file)
-  = do Hunk pos old new <- aHunk content
+  = do (pos, old, new) <- aHunk content
        return $ hunk (ap2fp path) pos old new
   where
       content = fileContent file
 
 aTokReplaceP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,File) -> Gen (prim wX wY)
 aTokReplaceP (path,file)
-  = do TokReplace tokchars old new <- aTokReplace content
+  = do (tokchars, old, new) <- aTokReplace content
        return $ tokreplace (ap2fp path) tokchars old new
   where
       content = fileContent file
@@ -289,9 +289,9 @@ aPrim repo
 -- Try to generate commutable pairs of hunks
 hunkPairP :: forall prim wX wY . PrimPatch prim => (AnchoredPath,File) -> Gen ((prim :> prim) wX wY)
 hunkPairP (path,file)
-  = do h1@(Hunk l1 old1 new1) <- aHunk content
-       (delta, content') <- selectChunk h1 content
-       Hunk l2' old2 new2 <- aHunk content'
+  = do (l1, old1, new1) <- aHunk content
+       (delta, content') <- selectChunk (Hunk l1 old1 new1) content
+       (l2', old2, new2) <- aHunk content'
        let l2 = l2'+delta
        return (hunk fpPath l1 old1 new1 :> hunk fpPath l2 old2 new2)
   where
