@@ -15,6 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 module Darcs.Test.Patch.Check ( PatchCheck(), doCheck, fileExists, dirExists,
                                 removeFile, removeDir, createFile, createDir,
                                 insertLine, deleteLine, isValid, doVerboseCheck,
@@ -29,7 +30,8 @@ import System.IO.Unsafe ( unsafePerformIO )
 import qualified Data.ByteString as B (ByteString)
 import Data.List ( isPrefixOf, inits )
 import Control.Monad.State ( State, evalState, runState )
-import Control.Monad.State.Class ( get, put, modify )
+import Control.Monad.State.Class ( get, put, modify, MonadState )
+import Control.Monad.Fail ( MonadFail(fail) )
 -- use Map, not IntMap, because Map has mapKeys and IntMap hasn't
 import Data.Map ( Map )
 import qualified Data.Map as M ( mapKeys, delete, insert, empty, lookup, null )
@@ -68,7 +70,11 @@ instance  Show Prop  where
 -- error handling mechanism.
 
 -- | PatchCheck is a state monad with a simulated repository state
-type PatchCheck = State KnownState
+newtype PatchCheck a = PatchCheck { runPatchCheck :: State KnownState a }
+  deriving (Functor, Applicative, Monad, MonadState KnownState)
+
+instance MonadFail PatchCheck where
+  fail = error
 
 -- | The @FileContents@ structure for an empty file
 emptyFilecontents :: FileContents
@@ -85,12 +91,12 @@ handleInconsistent v a =  do state <- get
                                _            -> a
 
 doCheck :: PatchCheck a -> a
-doCheck p = evalState p (P [] [])
+doCheck p = evalState (runPatchCheck p) (P [] [])
 
 -- | Run a check, and print the final repository state
 doVerboseCheck :: PatchCheck a -> a
 doVerboseCheck p =
-    case runState p (P [] []) of
+    case runState (runPatchCheck p) (P [] []) of
     (b, pc) -> unsafePerformIO $ do print pc
                                     return b
 
