@@ -1,7 +1,10 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE StandaloneDeriving, GeneralizedNewtypeDeriving #-}
 module Darcs.Test.Patch.Properties.Check ( Check(..), checkAPatch ) where
 
+import Prelude ()
+import Darcs.Prelude
 import Control.Monad ( liftM )
+import Data.Maybe ( isNothing )
 
 import Darcs.Test.Patch.Check ( PatchCheck,
                                 checkMove, removeDir, createDir,
@@ -16,17 +19,20 @@ import qualified Data.ByteString.Char8 as BC ( break, pack )
 import Darcs.Util.Path ( fn2fp )
 import qualified Data.Map as M ( mapMaybe )
 
-import Darcs.Patch ( invert,
-                     effect )
+import Darcs.Patch ( invert, effect, PrimPatch )
 import Darcs.Patch.Invert ( Invert )
-import Darcs.Patch.V1 ()
-import qualified Darcs.Patch.V1.Core as V1 ( RepoPatchV1(..) )
+import Darcs.Patch.V1 ( )
+import Darcs.Patch.V1.Core ( RepoPatchV1(..) )
+import Darcs.Patch.V2.RepoPatch ( RepoPatchV2, isConsistent )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
+import qualified Darcs.Patch.V2.Prim as V2 ( Prim(..) )
 import Darcs.Patch.V1.Core ( isMerger )
-import Darcs.Patch.Prim.V1 ()
+import qualified Darcs.Patch.Prim.FileUUID as FileUUID ( Prim )
 import Darcs.Patch.Prim.V1.Core ( Prim(..), DirPatchType(..), FilePatchType(..) )
 import Darcs.Patch.Witnesses.Ordered
 
-#include "impossible.h"
+type Prim1 = V1.Prim
+type Prim2 = V2.Prim
 
 class Check p where
    checkPatch :: p wX wY -> PatchCheck Bool
@@ -35,16 +41,27 @@ instance Check p => Check (FL p) where
    checkPatch NilFL = isValid
    checkPatch (p :>: ps) = checkPatch p >> checkPatch ps
 
+instance Check p => Check (p:>p) where
+   checkPatch (p1 :> p2) = checkPatch p1 >> checkPatch p2
+
 checkAPatch :: (Invert p, Check p) => p wX wY -> Bool
 checkAPatch p = doCheck $ do _ <- checkPatch p
                              checkPatch $ invert p
 
-instance Check (V1.RepoPatchV1 Prim) where
-   checkPatch p | isMerger p = do
-     checkPatch $ effect p
-   checkPatch (V1.Merger _ _ _ _) = impossible
-   checkPatch (V1.Regrem _ _ _ _) = impossible
-   checkPatch (V1.PP p) = checkPatch p
+instance PrimPatch prim => Check (RepoPatchV2 prim) where
+  checkPatch p = return $ isNothing $ isConsistent p
+
+instance Check (RepoPatchV1 Prim1) where
+   checkPatch p | isMerger p = checkPatch $ effect p
+   checkPatch (Merger _ _ _ _) = impossible
+   checkPatch (Regrem _ _ _ _) = impossible
+   checkPatch (PP p) = checkPatch p
+
+deriving instance Check Prim1
+deriving instance Check Prim2
+
+instance Check FileUUID.Prim where
+  checkPatch _ = return True -- XXX
 
 instance Check Prim where
 
