@@ -15,9 +15,6 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
-
 module Darcs.Test.Patch.Properties.V1Set2
     ( propCommuteInverse, propPatchAndInverseIsIdentity
     , propSimpleSmartMergeGoodEnough, propCommuteEquivalency
@@ -36,7 +33,9 @@ module Darcs.Test.Patch.Properties.V1Set2
     , propUnravelOrderIndependent, propResolveConflictsValid
     ) where
 
-import Prelude hiding ( pi )
+import Prelude ()
+import Darcs.Prelude
+
 import Test.QuickCheck
 import Test.Framework.Providers.QuickCheck2 ( testProperty )
 import Test.Framework ( Test )
@@ -46,24 +45,29 @@ import Darcs.Test.Patch.Properties.Check ( Check, checkAPatch )
 
 import Darcs.Patch ( invert, commute, merge,
                      readPatch, resolveConflicts,
-                     fromPrim, showPatch )
+                     fromPrim, showPatch, ShowPatchFor(..) )
 import Darcs.Patch.Commute ( Commute )
 import Darcs.Patch.Invert ( Invert )
 import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import Darcs.Patch.V1.Commute ( unravel, merger )
-import Darcs.Patch.Prim.V1 ()
-import Darcs.Patch.Prim.V1.Core ( Prim(..) )
-import Darcs.Patch.Prim.V1.Commute ( WrappedCommuteFunction(..), Perhaps(..),
-                                     subcommutes )
-import Darcs.Util.Printer ( renderPS, RenderMode(..) )
+import Darcs.Patch.Prim.V1 ( Prim )
+import Darcs.Patch.Prim.V1.Commute
+    ( Perhaps(..)
+    , toPerhaps
+    , speedyCommute
+    , cleverCommute
+    , commuteFiledir
+    , commuteFilepatches
+    )
+import Darcs.Util.Printer ( renderPS )
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), unsafeUnseal, unseal, mapSeal, Sealed2(..) )
 import Darcs.Patch.Witnesses.Unsafe
 
-#include "impossible.h"
-
-type Patch = V1.RepoPatchV1 Prim
+type Prim1 = V1.Prim
+type Patch = V1.RepoPatchV1 Prim1
 
 
 -- | Groups a set of tests by giving them the same prefix in their description.
@@ -84,7 +88,7 @@ propOtherInverseValid (Sealed2 p1) = checkAPatch (p1:>:invert p1:>:NilFL)
 propCommuteTwice :: Sealed2 (FL Patch :> FL Patch) -> Property
 propCommuteTwice (Sealed2 (p1:>p2)) =
     (doesCommute p1 p2) ==> (Just (p1:>p2) == (commute (p1:>p2) >>= commute))
-doesCommute :: (MyEq p, Invert p, Commute p, Check p) => p wX wY -> p wY wZ -> Bool
+doesCommute :: (Eq2 p, Invert p, Commute p, Check p) => p wX wY -> p wY wZ -> Bool
 doesCommute p1 p2 =
     commute (p1:>p2) /= Nothing && checkAPatch (p1:>:p2:>:NilFL)
 propCommuteEquivalency :: Sealed2 (FL Patch :> FL Patch) -> Property
@@ -208,9 +212,9 @@ propResolveConflictsValid p1 p2 =
     and $ map (\l -> (\ml -> checkAPatch (p+>+ml)) `unseal` mergeList l)
             $ resolveConflicts p
 
-mergeList :: [Sealed (FL Prim wX)] -> Sealed (FL Patch wX)
+mergeList :: [Sealed (FL Prim1 wX)] -> Sealed (FL Patch wX)
 mergeList patches = mapFL_FL fromPrim `mapSeal` doml NilFL patches
-    where doml :: FL Prim wX wY -> [Sealed (FL Prim wX)] -> Sealed (FL Prim wX)
+    where doml :: FL Prim1 wX wY -> [Sealed (FL Prim1 wX)] -> Sealed (FL Prim1 wX)
           doml mp (Sealed p:ps) =
               case commute (invert p :> mp) of
               Just (mp' :> _) -> doml (p +>+ mp') ps
@@ -218,7 +222,7 @@ mergeList patches = mapFL_FL fromPrim `mapSeal` doml NilFL patches
           doml mp [] = Sealed mp
 
 propReadShow :: FL Patch wX wY -> Bool
-propReadShow p = case readPatch $ renderPS Standard $ showPatch p of
+propReadShow p = case readPatch $ renderPS $ showPatch ForStorage p of
                    Just (Sealed p') -> isIsEq (p' =\/= p)
                    Nothing -> False
 
@@ -234,7 +238,28 @@ propCommuteInverse (Sealed2 (p1 :> p2)) =
                                Nothing -> False
                                Just (_ :> p1'') -> isIsEq (p1'' =/\= p1)
 
-type CommuteProperty = Sealed2 (Prim :> Prim) -> Property
+type CommuteProperty = Sealed2 (Prim1 :> Prim1) -> Property
+
+type CommuteFunction =
+    forall wX wY . (Prim1 :> Prim1) wX wY -> Perhaps ((Prim1 :> Prim1) wX wY)
+
+newtype WrappedCommuteFunction = WrappedCommuteFunction
+    { runWrappedCommuteFunction :: CommuteFunction }
+
+wrapCommuteFunction :: (forall wX wY . (Prim :> Prim) wX wY -> Perhaps ((Prim :> Prim) wX wY)) -> WrappedCommuteFunction
+wrapCommuteFunction f = WrappedCommuteFunction $
+  \(p :> q) -> do
+    q' :> p' <- f (V1.unPrim p :> V1.unPrim q)
+    return (V1.Prim q' :> V1.Prim p')
+
+subcommutes :: [(String, WrappedCommuteFunction)]
+subcommutes =
+    [("speedyCommute", wrapCommuteFunction speedyCommute),
+     ("commuteFiledir", wrapCommuteFunction (cleverCommute commuteFiledir)),
+     ("commuteFilepatches", wrapCommuteFunction (cleverCommute commuteFilepatches)),
+     ("commutex", wrapCommuteFunction (toPerhaps . commute))
+    ]
+
 
 subcommutesInverse :: [(String, CommuteProperty)]
 subcommutesInverse = zip names (map prop_subcommute cs)
@@ -297,19 +322,19 @@ subcommutesFailure = zip names (map prop cs)
                  Failed -> True
                  _ -> False
 
-doesFail :: WrappedCommuteFunction -> Prim wX wY -> Prim wY wZ -> Bool
+doesFail :: WrappedCommuteFunction -> Prim1 wX wY -> Prim1 wY wZ -> Bool
 doesFail c p1 p2 =
     fails (runWrappedCommuteFunction c (p1 :> p2)) && checkAPatch (p1 :>: p2 :>: NilFL)
         where fails Failed = True
               fails _ = False
 
-does :: WrappedCommuteFunction -> Prim wX wY -> Prim wY wZ -> Bool
+does :: WrappedCommuteFunction -> Prim1 wX wY -> Prim1 wY wZ -> Bool
 does c p1 p2 =
     succeeds (runWrappedCommuteFunction c (p1 :> p2)) && checkAPatch (p1 :>: p2 :>: NilFL)
         where succeeds (Succeeded _) = True
               succeeds _ = False
 
-nontrivial :: WrappedCommuteFunction -> Prim wX wY -> Prim wY wZ -> Bool
+nontrivial :: WrappedCommuteFunction -> Prim1 wX wY -> Prim1 wY wZ -> Bool
 nontrivial c p1 p2 =
     succeeds (runWrappedCommuteFunction c (p1 :> p2)) && checkAPatch (p1 :>: p2 :>: NilFL)
         where succeeds (Succeeded (p2' :> p1' )) = not (p1' `unsafeCompare` p1 && p2' `unsafeCompare` p2)
