@@ -1,6 +1,3 @@
-{-# LANGUAGE CPP #-}
-
-
 -- | Repository model
 module Darcs.Test.Patch.V1Model
   ( V1Model, repoTree
@@ -44,14 +41,12 @@ import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.ByteString.Lazy.Char8 as BLC
-import Data.List ( intercalate )
 import qualified Data.Map as M
 import Test.QuickCheck
   ( Arbitrary(..)
   , Gen, choose, vectorOf, frequency )
 
-#include "impossible.h"
-
+-- import Text.Show.Pretty ( ppShow )
 
 ----------------------------------------------------------------------
 -- * Model definition
@@ -76,17 +71,18 @@ type Content = [B.ByteString]
 ----------------------------------------
 -- Instances
 
+data FlatItem = Dir FilePath | File FilePath [String]
+  deriving Show
+
+flattenTree :: Tree Fail -> [FlatItem]
+flattenTree = map flattenEntry . T.list where
+  flattenEntry (fn, T.SubTree _) = Dir (BC.unpack (flatten fn))
+  flattenEntry (fn, T.File blob) = File (BC.unpack (flatten fn))
+    (map BLC.unpack $ BLC.lines $ unFail $ T.readBlob blob)
+  flattenEntry (_, _) = impossible
+
 instance Show (V1Model wX) where
-  show repo = "V1Model{ "
-            ++ intercalate " " (map showEntry $ list repo)
-            ++ " }"
-    where
-        showPath = show . flatten
-        showContent content = "[" ++ intercalate " " (map show content) ++ "]"
-        showEntry (path,item)
-          | isDir item  = showPath path
-          | isFile item = showPath path ++ showContent (fileContent item)
-        showEntry _ = impossible
+  show repo = "V1Model " ++ show (flattenTree (repoTree repo))
 
 instance Show1 V1Model where
   showDict1 = ShowDictClass
@@ -276,7 +272,7 @@ aRepo maxFiles maxDirs
 -- Small repositories help generating (potentially) conflicting patches.
 instance RepoModel V1Model where
   type RepoState V1Model = Tree
-  showModel m = show m
+  showModel m = show {- ppShow -} m
   aSmallRepo = do filesNo <- frequency [(3, return 1), (1, return 2)]
                   dirsNo <- frequency [(3, return 1), (1, return 0)]
                   aRepo filesNo dirsNo
