@@ -30,9 +30,12 @@ import Darcs.Patch.Witnesses.Show
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePStart, unsafeCoercePEnd )
 
 import Darcs.Patch.Merge ( Merge )
-import Darcs.Patch.Prim.V1 ( Prim )
 import Darcs.Patch.V2 ( RepoPatchV2 )
-import Darcs.Patch.Patchy ( Commute, Invert(..) )
+import qualified Darcs.Patch.V2.Prim as V2
+import Darcs.Patch.Commute ( Commute )
+import Darcs.Patch.Invert ( Invert(..) )
+
+type Prim2 = V2.Prim
 
 
 infixr 5 :>:
@@ -85,7 +88,7 @@ instance WSub prim prim => WSub (RepoPatchV2 prim) (RepoPatchV2 prim) where
    fromW = id
    toW = id
 
-instance WSub Prim Prim where
+instance WSub Prim2 Prim2 where
    fromW = id
    toW = id
 
@@ -107,7 +110,7 @@ instance (WSub wp p, Show2 wp) => Show (FL p wX wY) where
 instance (WSub wp p, Show2 wp) => Show2 (FL p) where
    showDict2 = ShowDictClass
 
-instance (WSub wp p, Commute wp, MyEq wp) => MyEq (FL p) where
+instance (WSub wp p, Commute wp, Eq2 wp) => Eq2 (FL p) where
    unsafeCompare x y = unsafeCompare (toW x) (toW y)
 
 instance (WSub wp p, Commute wp, Invert wp) => Invert (FL p) where
@@ -126,12 +129,12 @@ merge = fromW . W.merge . toW
 commute :: (WSub wp p, Commute wp) => (p :> p) wX wY -> Maybe ((p :> p) wX wY)
 commute = fmap fromW . W.commute . toW
 
-getPairs :: FL (RepoPatchV2 Prim) wX wY -> [Sealed2 (RepoPatchV2 Prim :> RepoPatchV2 Prim)]
+getPairs :: FL (RepoPatchV2 Prim2) wX wY -> [Sealed2 (RepoPatchV2 Prim2 :> RepoPatchV2 Prim2)]
 getPairs = map (mapSeal2 fromW) . W.getPairs . toW
 
-getTriples :: FL (RepoPatchV2 Prim) wX wY -> [Sealed2 (RepoPatchV2 Prim :> RepoPatchV2 Prim :> RepoPatchV2 Prim)]
+getTriples :: FL (RepoPatchV2 Prim2) wX wY -> [Sealed2 (RepoPatchV2 Prim2 :> RepoPatchV2 Prim2 :> RepoPatchV2 Prim2)]
 getTriples = map (mapSeal2 fromW) . W.getTriples . toW
 
-coalesce :: (Prim :> Prim) wX wY -> Maybe (FL Prim wX wY)
+coalesce :: (Prim2 :> Prim2) wX wY -> Maybe (FL Prim2 wX wY)
 coalesce = fmap fromW . W.coalesce . toW
 
