@@ -18,13 +18,11 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Darcs.Patch
-    ( RepoPatch
-    , RepoType
+    ( RepoType
     , IsRepoType
     , PrimOf
     , Named
     , WrappedNamed
-    , Patchy
     , fromPrim
     , fromPrims
     , rmfile
@@ -39,7 +37,9 @@ module Darcs.Patch
     , binary
     , description
     , showContextPatch
+    , ShowPatchFor(..)
     , showPatch
+    , displayPatch
     , showNicely
     , infopatch
     , changepref
@@ -88,15 +88,18 @@ module Darcs.Patch
     , getdeps
     , listConflictedFiles
     , isInconsistent
+    , module Darcs.Patch.RepoPatch
     ) where
 
 
-import Darcs.Patch.Apply ( applyToFilePaths, effectOnFilePaths, applyToTree,
-                           maybeApplyToTree, ApplyState )
-import Darcs.Patch.Commute ( commuteFL, commuteFLorComplain, commuteRL )
+import Darcs.Patch.Apply ( apply,applyToFilePaths, effectOnFilePaths, applyToTree,
+                           maybeApplyToTree )
+import Darcs.Patch.Commute ( commute, commuteFL, commuteFLorComplain, commuteRL )
 import Darcs.Patch.Conflict ( listConflictedFiles, resolveConflicts )
 import Darcs.Patch.Effect ( Effect(effect) )
-import Darcs.Patch.Invert ( invertRL, invertFL )
+import Darcs.Patch.Invert ( invert, invertRL, invertFL )
+import Darcs.Patch.Inspect ( listTouchedFiles, hunkMatches )
+import Darcs.Patch.Merge ( merge )
 import Darcs.Patch.Named ( Named,
                            adddeps, namepatch,
                            anonymous,
@@ -104,14 +107,6 @@ import Darcs.Patch.Named ( Named,
                            infopatch,
                            patch2patchinfo, patchname, patchcontents )
 import Darcs.Patch.Named.Wrapped ( WrappedNamed )
-import Darcs.Patch.Patchy ( Patchy,
-                            showPatch, showNicely, showContextPatch,
-                            invert,
-                            thing, things,
-                            apply,
-                            description, summary, summaryFL,
-                            commute, listTouchedFiles, hunkMatches
-                          )
 import Darcs.Patch.Prim ( fromPrims, fromPrim,
                           canonize,
                           sortCoalesceFL,
@@ -125,12 +120,7 @@ import Darcs.Patch.Read ( readPatch, readPatchPartial )
 import Darcs.Patch.Repair ( isInconsistent )
 import Darcs.Patch.RepoPatch ( RepoPatch )
 import Darcs.Patch.RepoType ( RepoType, IsRepoType )
+import Darcs.Patch.Show ( description, showPatch, showNicely, displayPatch
+                        , summary, summaryFL, thing, things, ShowPatchFor(..), ShowContextPatch(..) )
 import Darcs.Patch.Summary ( xmlSummary, plainSummary, plainSummaryPrims )
 import Darcs.Patch.TokenReplace ( forceTokReplace )
-import Darcs.Patch.V1.Commute ( merge )
-
-import Darcs.Util.Tree( Tree )
-
-
-instance (Patchy p, ApplyState p ~ Tree) => Patchy (Named p)
-instance (Patchy p, ApplyState p ~ Tree) => Patchy (WrappedNamed rt p)
