@@ -15,19 +15,13 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.Patch.Bundle
-    ( hashBundle
-    , makeBundle2
-    , makeBundleN
+    ( makeBundleN
     , scanBundle
     , contextPatches
     , scanContextFile
     , patchFilename
-    , getContext
     , minContext
-    , parseBundle
     ) where
 
 import Prelude ()
@@ -50,7 +44,7 @@ import Darcs.Patch.Commute( commute )
 import Darcs.Patch.Depends ( slightlyOptimizePatchset )
 import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Info ( PatchInfo, readPatchInfo, showPatchInfo,
-                          showPatchInfoUI, isTag )
+                          displayPatchInfo, isTag )
 import Darcs.Patch.Named.Wrapped ( WrappedNamed )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, piap, fmapFLPIAP, info,
                                   patchInfoAndPatch, unavailable, hopefully,
@@ -59,17 +53,18 @@ import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, piap, fmapFLPIAP, info,
 import Darcs.Patch.ReadMonads ( parseStrictly )
 import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, Origin )
-import Darcs.Patch.Show ( ShowPatchBasic )
+import Darcs.Patch.Show ( ShowPatchBasic, ShowPatchFor(ForStorage) )
 import Darcs.Patch.Witnesses.Ordered
     ( RL(..), FL(..), (:>)(..), reverseFL, (+<+),
     mapFL, mapFL_FL, mapRL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 
-import Darcs.Util.ByteString ( mmapFilePS, linesPS, unlinesPS, dropSpace, substrPS)
-import Darcs.Util.Crypt.SHA1 ( sha1PS )
+import Darcs.Util.ByteString
+    ( mmapFilePS, linesPS, unlinesPS, dropSpace, substrPS, decodeLocale )
+import Darcs.Util.Hash ( sha1PS )
 import Darcs.Util.Printer ( Doc, renderPS, newline, text, ($$),
-                 (<>), vcat, vsep, renderString, RenderMode(..) )
+                 (<>), vcat, vsep, renderString )
 
 -- |hashBundle creates a SHA1 string of a given a FL of named patches. This
 -- allows us to ensure that the patches in a received patchBundle have not been
@@ -77,7 +72,7 @@ import Darcs.Util.Printer ( Doc, renderPS, newline, text, ($$),
 hashBundle :: (PatchListFormat p, ShowPatchBasic p) => FL (WrappedNamed rt p) wX wY
            -> String
 hashBundle to_be_sent =
-    show $ sha1PS $ renderPS Standard $ vcat (mapFL showPatch to_be_sent) <> newline
+    show $ sha1PS $ renderPS $ vcat (mapFL (showPatch ForStorage) to_be_sent) <> newline
 
 makeBundleN :: (ApplyState p ~ Tree, RepoPatch p) => Maybe (Tree IO)
             -> PatchSet rt p wStart wX -> FL (WrappedNamed rt p) wX wY -> IO Doc
@@ -96,8 +91,8 @@ makeBundle2 :: (ApplyState p ~ Tree, RepoPatch p) => Maybe (Tree IO)
             -> FL (WrappedNamed rt p) wX wY -> IO Doc
 makeBundle2 the_s common' to_be_sent to_be_sent2 = do
     patches <- case the_s of
-                   Just tree -> fst `fmap` virtualTreeIO (showContextPatch to_be_sent) tree
-                   Nothing -> return (vsep $ mapFL showPatch to_be_sent)
+                   Just tree -> fst `fmap` virtualTreeIO (showContextPatch ForStorage to_be_sent) tree
+                   Nothing -> return (vsep $ mapFL (showPatch ForStorage) to_be_sent)
     return $ format patches
   where
     format the_new = text ""
@@ -107,7 +102,7 @@ makeBundle2 the_s common' to_be_sent to_be_sent2 = do
                      $$ text ""
                      $$ text "Context:"
                      $$ text ""
-                     $$ vcat (map showPatchInfo common)
+                     $$ vcat (map (showPatchInfo ForStorage) common)
                      $$ text "Patch bundle hash:"
                      $$ text (hashBundle to_be_sent2)
                      $$ text ""
@@ -153,7 +148,7 @@ parseBundle input = case sillyLex input of
 
     sealCtxAndPs ctx ps = Right $ sealContextWithPatches ctx ps
 
-    sealContextWithPatches :: RepoPatch p => [PatchInfo]
+    sealContextWithPatches :: [PatchInfo]
                            -> FL (PatchInfoAnd ('RepoType 'NoRebase) (Bracketed p)) wX wY
                            -> Sealed
                                   ((PatchSet rt p :> FL (PatchInfoAnd rt p)) Origin)
@@ -201,14 +196,14 @@ filterGpgDashes ps =
 
 -- |unavailablePatches converts a list of PatchInfos into a RL of PatchInfoAnd
 -- Unavailable patches. This is used to represent the Context of a patchBundle.
-unavailablePatches :: RepoPatch p => [PatchInfo] -> RL (PatchInfoAnd rt p) wX wY
+unavailablePatches :: [PatchInfo] -> RL (PatchInfoAnd rt p) wX wY
 unavailablePatches = foldr (flip (:<:) . piUnavailable) (unsafeCoerceP NilRL)
 
 -- |piUnavailable returns an Unavailable within a PatchInfoAnd given a
 -- PatchInfo.
-piUnavailable :: RepoPatch p => PatchInfo -> PatchInfoAnd rt p wX wY
+piUnavailable :: PatchInfo -> PatchInfoAnd rt p wX wY
 piUnavailable i = patchInfoAndPatch i . unavailable $
-    "Patch not stored in patch bundle:\n" ++ renderString Encode (showPatchInfoUI i)
+    "Patch not stored in patch bundle:\n" ++ renderString (displayPatchInfo i)
 
 -- |getContext parses a context list, returning a tuple containing the list,
 -- and remaining ByteString input.
@@ -238,11 +233,11 @@ getPatches ps = case parseStrictly readPatchInfo ps of
 -- removed any leading spaces. The before-newline part is unpacked to a String,
 -- and tupled up with the remaining ByteString.
 sillyLex :: B.ByteString -> (String, B.ByteString)
-sillyLex ps = (BC.unpack a, b)
+sillyLex ps = (decodeLocale a, b)
   where
     (a, b) = BC.break (== '\n') (dropSpace ps)
 
-contextPatches :: RepoPatch p => PatchSet rt p Origin wX
+contextPatches :: PatchSet rt p Origin wX
                -> (PatchSet rt p :> RL (PatchInfoAnd rt p)) Origin wX
 contextPatches set = case slightlyOptimizePatchset set of
     PatchSet (ts :<: Tagged t _ ps') ps ->
@@ -250,11 +245,11 @@ contextPatches set = case slightlyOptimizePatchset set of
     PatchSet NilRL ps -> PatchSet NilRL NilRL :> ps
 
 -- |'scanContextFile' scans the context in the file of the given name.
-scanContextFile :: RepoPatch p => FilePath -> IO (PatchSet rt p Origin wX)
+scanContextFile :: FilePath -> IO (PatchSet rt p Origin wX)
 scanContextFile filename = scanContext `fmap` mmapFilePS filename
   where
     -- are the type witnesses sensible?
-    scanContext :: RepoPatch p => B.ByteString -> PatchSet rt p Origin wX
+    scanContext :: B.ByteString -> PatchSet rt p Origin wX
     scanContext input
         | B.null input = error "Bad context!"
         | otherwise = case sillyLex input of
