@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP , ScopedTypeVariables #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 
 module Darcs.Patch.Depends
     ( getUncovered
@@ -28,8 +28,8 @@ module Darcs.Patch.Depends
     , slightlyOptimizePatchset
     , getPatchesBeyondTag
     , splitOnTag
-    , newsetUnion
-    , newsetIntersection
+    , patchSetUnion
+    , patchSetIntersection
     , findUncommon
     , merge2FL
     , getDeps
@@ -39,41 +39,35 @@ module Darcs.Patch.Depends
 import Prelude ()
 import Darcs.Prelude
 
-#include "impossible.h"
-
 import Prelude hiding ( pi )
 import Data.List ( delete, intersect, (\\) )
 import Data.Maybe ( fromMaybe )
-import qualified Data.ByteString.Char8 as BC ( unpack )
 import Control.Arrow ( (&&&) )
 
 import Darcs.Patch ( RepoPatch )
-import Darcs.Patch.Apply( ApplyState )
-import Darcs.Patch.Patchy ( Patchy )
 import Darcs.Patch.Named ( Named (..), patch2patchinfo )
 import Darcs.Patch.Named.Wrapped ( getdeps )
 import Darcs.Patch.Choices ( Label, patchChoices, forceFirst
-                           , PatchChoices, lpPatch, getChoices
+                           , PatchChoices, unLabel, getChoices
                            , LabelledPatch, label )
-import Darcs.Patch.Commute ( commute, commuteFL, commuteRL )
-import Darcs.Patch.Info ( PatchInfo, isTag, showPatchInfoUI, _piName )
+import Darcs.Patch.Commute ( Commute, commute, commuteFL, commuteRL )
+import Darcs.Patch.Info ( PatchInfo, isTag, displayPatchInfo, piName )
 import Darcs.Patch.Merge ( Merge, mergeFL )
 import Darcs.Patch.Permutations ( partitionFL, partitionRL )
 import Darcs.Patch.PatchInfoAnd( PatchInfoAnd, hopefully, hopefullyM, info )
-import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, newset2RL,
+import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, patchSet2RL,
                          appendPSFL )
 import Darcs.Patch.Progress ( progressRL )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..), (=\/=), (=/\=) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePStart )
 import Darcs.Patch.Witnesses.Ordered
     ( (:\/:)(..), (:/\:)(..), (:>)(..), Fork(..),
-    (+>+), mapFL, RL(..), FL(..), isShorterThanRL,
+    (+>>+), (+<<+), mapFL, RL(..), FL(..), isShorterThanRL,
     (+<+), reverseFL, reverseRL, mapRL, lengthFL, splitAtFL )
 import Darcs.Patch.Witnesses.Sealed
     ( Sealed(..), FlippedSeal(..), flipSeal, seal, Sealed2(..), seal2 )
 
-import Darcs.Util.Printer ( renderString, vcat, RenderMode(..) )
-import Darcs.Util.Tree ( Tree )
+import Darcs.Util.Printer ( renderString, vcat )
 
 {-
  - This module uses the following definitions:
@@ -100,17 +94,17 @@ type SPatchAndDeps p = ( Sealed2 (LabelledPatch (Named p))
                        )
 
 -- | Searchs dependencies in @repoFL@ of the patches in @getDepsFL@.
-getDeps :: (RepoPatch p, ApplyState p ~ Tree) =>
+getDeps :: (RepoPatch p) =>
             FL (Named p) wA wR -> FL (PatchInfoAnd rt p) wX wY -> [SPatchAndDeps p]
 getDeps repoFL getDepsFL =
         let repoChoices   = patchChoices repoFL
-            getDepsFL'    = mapFL (BC.unpack . _piName . info) getDepsFL
+            getDepsFL'    = mapFL (piName . info) getDepsFL
             labelledDeps  = getLabelledDeps getDepsFL' repoChoices
         in
             map (deps repoChoices) labelledDeps
     where
         -- Search dependencies for the patch with label @l@ in @repoChoices@.
-        deps :: (Patchy (Named p)) => PatchChoices (Named p) wX wY ->
+        deps :: (Commute p) => PatchChoices (Named p) wX wY ->
                 (String,Label) -> SPatchAndDeps p
         deps repoChoices (_,l) =
             case getChoices $ forceFirst l repoChoices of
@@ -122,7 +116,7 @@ getDeps repoFL getDepsFL =
                                                     -- has r, which is the patch
                                                     -- that we are looking at
                                                     -- dependencies.
-        getLabelledDeps :: Patchy (Named p) => [String] ->
+        getLabelledDeps :: (Commute p) => [String] ->
                            PatchChoices (Named p) x y -> [(String, Label)]
         getLabelledDeps patchnames repoChoices =
             case getChoices repoChoices of
@@ -138,7 +132,7 @@ getDeps repoFL getDepsFL =
                                     else filterDepsFL patchnames lps
             where
                 lpTostring :: LabelledPatch (Named p) wA wB -> String
-                lpTostring = BC.unpack . _piName . patch2patchinfo . lpPatch
+                lpTostring = piName . patch2patchinfo . unLabel
                 dep :: (String, Label)
                 dep = lpTostring &&& label $ lp
 
@@ -155,13 +149,13 @@ the repository that is more cheaply accessed (i.e. local), as
 taggedIntersection does its best to reduce the number of
 inventories that are accessed from its rightmost argument.
 -}
-taggedIntersection :: forall rt p wStart wX wY . Patchy p =>
-                      PatchSet rt p wStart wX -> PatchSet rt p wStart wY ->
+taggedIntersection :: forall rt p wStart wX wY . Commute p
+                   => PatchSet rt p wStart wX -> PatchSet rt p wStart wY ->
                       Fork (RL (Tagged rt p))
                            (RL (PatchInfoAnd rt p))
                            (RL (PatchInfoAnd rt p)) wStart wX wY
-taggedIntersection (PatchSet NilRL ps1) s2 = Fork NilRL ps1 (newset2RL s2)
-taggedIntersection s1 (PatchSet NilRL ps2) = Fork NilRL (newset2RL s1) ps2
+taggedIntersection (PatchSet NilRL ps1) s2 = Fork NilRL ps1 (patchSet2RL s2)
+taggedIntersection s1 (PatchSet NilRL ps2) = Fork NilRL (patchSet2RL s1) ps2
 taggedIntersection s1 (PatchSet (_ :<: Tagged t _ _) ps2)
     | Just (PatchSet ts1 ps1) <- maybeSplitSetOnTag (info t) s1 =
     Fork ts1 ps1 (unsafeCoercePStart ps2)
@@ -172,7 +166,7 @@ taggedIntersection s1 s2@(PatchSet (ts2 :<: Tagged t _ p) ps2) =
                        Just (PatchSet com NilRL :> us) ->
                            Fork com us (unsafeCoercePStart ps2)
                        Just _ -> impossible
-                       Nothing -> Fork NilRL (newset2RL s1) (newset2RL s2)
+                       Nothing -> Fork NilRL (patchSet2RL s1) (patchSet2RL s2)
 
 -- |'maybeSplitSetOnTag' takes a tag's 'PatchInfo', @t0@, and a 'PatchSet' and
 -- attempts to find @t0@ in one of the 'Tagged's in the PatchSet. If the tag is
@@ -188,7 +182,7 @@ maybeSplitSetOnTag t0 origSet@(PatchSet (ts :<: Tagged t _ pst) ps)
         Just $ PatchSet ts' (ps' +<+ ps)
 maybeSplitSetOnTag _ _ = Nothing
 
-getPatchesBeyondTag :: Patchy p => PatchInfo -> PatchSet rt p wStart wX
+getPatchesBeyondTag :: Commute p => PatchInfo -> PatchSet rt p wStart wX
                     -> FlippedSeal (RL (PatchInfoAnd rt p)) wX
 getPatchesBeyondTag t (PatchSet (_ :<: Tagged hp _ _) ps) | info hp == t =
     flipSeal ps
@@ -203,7 +197,7 @@ getPatchesBeyondTag t patchset@(PatchSet ts (ps :<: hp)) =
         else case getPatchesBeyondTag t (PatchSet ts ps) of
                  FlippedSeal xxs -> FlippedSeal (xxs :<: hp)
 getPatchesBeyondTag t (PatchSet NilRL NilRL) =
-    bug $ "tag\n" ++ renderString Encode (showPatchInfoUI t)
+    bug $ "tag\n" ++ renderString (displayPatchInfo t)
           ++ "\nis not in the patchset in getPatchesBeyondTag."
 getPatchesBeyondTag t0 (PatchSet (ts :<: Tagged t _ ps) NilRL) =
     getPatchesBeyondTag t0 (PatchSet ts (ps :<: t))
@@ -212,7 +206,7 @@ getPatchesBeyondTag t0 (PatchSet (ts :<: Tagged t _ ps) NilRL) =
 -- find the tag in the PatchSet, returning a pair: the clean PatchSet "up to"
 -- the tag, and a RL of patches after the tag; If the tag is not in the
 -- PatchSet, we return Nothing.
-splitOnTag :: Patchy p => PatchInfo -> PatchSet rt p wStart wX
+splitOnTag :: Commute p => PatchInfo -> PatchSet rt p wStart wX
            -> Maybe ((PatchSet rt p :> RL (PatchInfoAnd rt p)) wStart wX)
 -- If the tag we are looking for is the first Tagged tag of the patchset, just
 -- separate out the patchset's patches.
@@ -323,7 +317,7 @@ slightlyOptimizePatchset (PatchSet ps0 ts0) =
                      appendPSFL ps' (hp :>: NilFL)
         | otherwise = appendPSFL (sops $ PatchSet ts ps) (hp :>: NilFL)
 
-removeFromPatchSet :: Patchy p => FL (PatchInfoAnd rt p) wX wY
+removeFromPatchSet :: Commute p => FL (PatchInfoAnd rt p) wX wY
                    -> PatchSet rt p wStart wY -> Maybe (PatchSet rt p wStart wX)
 removeFromPatchSet bad (PatchSet ts ps) | all (`elem` mapRL info ps) (mapFL info bad) = do
     ps' <- fastRemoveSubsequenceRL (reverseFL bad) ps
@@ -332,14 +326,14 @@ removeFromPatchSet _ (PatchSet NilRL _) = Nothing
 removeFromPatchSet bad (PatchSet (ts :<: Tagged t _ tps) ps) =
     removeFromPatchSet bad (PatchSet ts (tps :<: t +<+ ps))
 
-fastRemoveSubsequenceRL :: Patchy p
+fastRemoveSubsequenceRL :: Commute p
                         => RL (PatchInfoAnd rt p) wY wZ
                         -> RL (PatchInfoAnd rt p) wX wZ
                         -> Maybe (RL (PatchInfoAnd rt p) wX wY)
 fastRemoveSubsequenceRL NilRL ys = Just ys
 fastRemoveSubsequenceRL (xs:<:x) ys = fastRemoveRL x ys >>= fastRemoveSubsequenceRL xs
 
-findCommonAndUncommon :: forall rt p wStart wX wY . Patchy p
+findCommonAndUncommon :: forall rt p wStart wX wY . Commute p
                       => PatchSet rt p wStart wX -> PatchSet rt p wStart wY
                       -> Fork (PatchSet rt p)
                               (FL (PatchInfoAnd rt p))
@@ -349,21 +343,21 @@ findCommonAndUncommon us them = case taggedIntersection us them of
         case partitionFL (infoIn them') $ reverseRL us' of
             _ :> bad@(_ :>: _) :> _ ->
                 bug $ "Failed to commute common patches:\n"
-                      ++ renderString Encode
-                          (vcat $ mapRL (showPatchInfoUI . info) $ reverseFL bad)
+                      ++ renderString
+                          (vcat $ mapRL (displayPatchInfo . info) $ reverseFL bad)
             (common2 :> NilFL :> only_ours) ->
                 case partitionFL (infoIn us') $ reverseRL them' of
                     _ :> bad@(_ :>: _) :> _ ->
                         bug $ "Failed to commute common patches:\n"
-                            ++ renderString Encode (vcat $
-                                mapRL (showPatchInfoUI . info) $ reverseFL bad)
+                            ++ renderString (vcat $
+                                mapRL (displayPatchInfo . info) $ reverseFL bad)
                     _ :> NilFL :> only_theirs ->
                         Fork (PatchSet common (reverseFL common2))
                             only_ours (unsafeCoercePStart only_theirs)
   where
     infoIn inWhat = (`elem` mapRL info inWhat) . info
 
-findCommonWithThem :: Patchy p
+findCommonWithThem :: Commute p
                    => PatchSet rt p wStart wX
                    -> PatchSet rt p wStart wY
                    -> (PatchSet rt p :> FL (PatchInfoAnd rt p)) wStart wX
@@ -372,12 +366,12 @@ findCommonWithThem us them = case taggedIntersection us them of
         case partitionFL ((`elem` mapRL info them') . info) $ reverseRL us' of
             _ :> bad@(_ :>: _) :> _ ->
                 bug $ "Failed to commute common patches:\n"
-                      ++ renderString Encode
-                          (vcat $ mapRL (showPatchInfoUI . info) $ reverseFL bad)
+                      ++ renderString
+                          (vcat $ mapRL (displayPatchInfo . info) $ reverseFL bad)
             common2 :> _nilfl :> only_ours ->
                 PatchSet common (reverseFL common2) :> unsafeCoerceP only_ours
 
-findUncommon :: Patchy p
+findUncommon :: Commute p
              => PatchSet rt p wStart wX -> PatchSet rt p wStart wY
              -> (FL (PatchInfoAnd rt p) :\/: FL (PatchInfoAnd rt p)) wX wY
 findUncommon us them =
@@ -385,7 +379,7 @@ findUncommon us them =
         _common :> us' -> case findCommonWithThem them us of
             _ :> them' -> unsafeCoercePStart us' :\/: them'
 
-countUsThem :: Patchy p
+countUsThem :: Commute p
             => PatchSet rt p wStart wX
             -> PatchSet rt p wStart wY
             -> (Int, Int)
@@ -395,7 +389,7 @@ countUsThem us them =
                                 tt = mapRL info them' in
                             (length $ uu \\ tt, length $ tt \\ uu)
 
-mergeThem :: (Patchy p, Merge p)
+mergeThem :: (Merge p)
           => PatchSet rt p wStart wX -> PatchSet rt p wStart wY
           -> Sealed (FL (PatchInfoAnd rt p) wX)
 mergeThem us them =
@@ -404,28 +398,28 @@ mergeThem us them =
             case merge2FL (reverseRL us') (reverseRL them') of
                them'' :/\: _ -> Sealed them''
 
-newsetIntersection :: Patchy p
+patchSetIntersection :: Commute p
                    => [SealedPatchSet rt p wStart]
                    -> SealedPatchSet rt p wStart
-newsetIntersection [] = seal $ PatchSet NilRL NilRL
-newsetIntersection [x] = x
-newsetIntersection (Sealed y : ys) =
-    case newsetIntersection ys of
+patchSetIntersection [] = seal $ PatchSet NilRL NilRL
+patchSetIntersection [x] = x
+patchSetIntersection (Sealed y : ys) =
+    case patchSetIntersection ys of
         Sealed z -> case taggedIntersection y z of
             Fork common a b -> case mapRL info a `intersect` mapRL info b of
                 morecommon ->
                     case partitionRL (\e -> info e `notElem` morecommon) a of
                         commonps :> _ -> seal $ PatchSet common commonps
 
-newsetUnion :: (Patchy p, Merge p)
+patchSetUnion :: (Merge p)
             => [SealedPatchSet rt p wStart]
             -> SealedPatchSet rt p wStart
-newsetUnion [] = seal $ PatchSet NilRL NilRL
-newsetUnion [x] = x
-newsetUnion (Sealed y@(PatchSet tsy psy) : Sealed y2 : ys) =
+patchSetUnion [] = seal $ PatchSet NilRL NilRL
+patchSetUnion [x] = x
+patchSetUnion (Sealed y@(PatchSet tsy psy) : Sealed y2 : ys) =
     case mergeThem y y2 of
         Sealed p2 ->
-            newsetUnion $ seal (PatchSet tsy (psy +<+ reverseFL p2)) : ys
+            patchSetUnion $ seal (PatchSet tsy (psy +<+ reverseFL p2)) : ys
 
 -- | Merge two FLs (say L and R), starting in a common context. The result is a
 -- FL starting in the original end context of L, going to a new context that is
@@ -440,7 +434,7 @@ newsetUnion (Sealed y@(PatchSet tsy psy) : Sealed y2 : ys) =
 --   (bf: I guess what was meant here is that 'merge2FL' works in the
 --    the way it does because it considers patch meta data whereas
 --    'mergeFL' cannot since it must work for primitive patches, too.
-merge2FL :: (Patchy p, Merge p)
+merge2FL :: (Merge p)
          => FL (PatchInfoAnd rt p) wX wY
          -> FL (PatchInfoAnd rt p) wX wZ
          -> (FL (PatchInfoAnd rt p) :/\: FL (PatchInfoAnd rt p)) wY wZ
@@ -454,7 +448,7 @@ merge2FL (x :>: xs) ys | Just ys' <- fastRemoveFL x ys = merge2FL xs ys'
                                                 ys'' :/\: xs' ->
                                                    ys'' :/\: (x' :>: xs')
 
-areUnrelatedRepos :: Patchy p
+areUnrelatedRepos :: Commute p
                   => PatchSet rt p wStart wX
                   -> PatchSet rt p wStart wY -> Bool
 areUnrelatedRepos us them =
@@ -477,12 +471,7 @@ areUnrelatedRepos us them =
 -- The types fit together (internally, in this module) because we commute the
 -- patch to the front before removing it and commutation inside a sequence does
 -- not change the sequence's contexts.
---
--- However, the use sites outside this module are something different. We
--- usually need coercions to get the patch(es) to be removed in shape. This is
--- not very nice but probably unavoidable given the approximative nature of
--- context witnesses.
-fastRemoveFL :: Patchy p
+fastRemoveFL :: Commute p
              => PatchInfoAnd rt p wX wY -- this type assumes element is at the front
              -> FL (PatchInfoAnd rt p) wX wZ
              -> Maybe (FL (PatchInfoAnd rt p) wY wZ)
@@ -496,18 +485,18 @@ fastRemoveFL a (b :>: bs) = do
     Just (b' :>: bs')
   where
     i = info a
-    pullout :: Patchy p
+    pullout :: Commute p
             => RL (PatchInfoAnd rt p) wA wB
             -> FL (PatchInfoAnd rt p) wB wC
             -> Maybe ((PatchInfoAnd rt p :> FL (PatchInfoAnd rt p)) wA wC)
     pullout _ NilFL = Nothing
     pullout acc (x :>: xs)
         | info x == i = do x' :> acc' <- commuteRL (acc :> x)
-                           Just (x' :> reverseRL acc' +>+ xs)
+                           Just (x' :> acc' +>>+ xs)
         | otherwise = pullout (acc :<: x) xs
 
 -- | Same as 'fastRemoveFL' only for 'RL'.
-fastRemoveRL :: Patchy p
+fastRemoveRL :: Commute p
              => PatchInfoAnd rt p wY wZ -- this type assumes element is at the back
              -> RL (PatchInfoAnd rt p) wX wZ
              -> Maybe (RL (PatchInfoAnd rt p) wX wY)
@@ -521,12 +510,12 @@ fastRemoveRL a (bs :<: b) = do
     Just (bs' :<: b')
   where
     i = info a
-    pullout :: Patchy p
+    pullout :: Commute p
             => RL (PatchInfoAnd rt p) wA wB
             -> FL (PatchInfoAnd rt p) wB wC
             -> Maybe ((RL (PatchInfoAnd rt p) :> PatchInfoAnd rt p) wA wC)
     pullout NilRL _ = Nothing
     pullout (xs :<: x) acc
         | info x == i = do acc' :> x' <- commuteFL (x :> acc)
-                           Just (xs +<+ reverseFL acc' :> x')
+                           Just (xs +<<+ acc' :> x')
         | otherwise = pullout xs (x :>: acc)
