@@ -1,5 +1,9 @@
+{-# OPTIONS_GHC -fno-warn-unrecognised-pragmas #-}
 module Darcs.Patch.Effect ( Effect(..) ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Prim.Class ( PrimOf )
 
 import Darcs.Patch.Witnesses.Ordered
@@ -17,6 +21,7 @@ class Effect p where
     effect = reverseRL . effectRL
     effectRL :: p wX wY -> RL (PrimOf p) wX wY
     effectRL = reverseFL . effect
+    {-# MINIMAL effect | effectRL #-}
 
 instance Effect p => Effect (FL p) where
     effect p = concatFL $ mapFL_FL effect p
