@@ -21,9 +21,8 @@ module Darcs.Patch.Index.Types where
 import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Util.Crypt.SHA1( SHA1(..) )
-import Darcs.Util.Path ( fp2fn, fn2fp, FileName )
-import Darcs.Patch.Info ( makePatchname, PatchInfo )
+import Darcs.Util.Hash( SHA1, sha1short, sha1zero )
+import Darcs.Util.Path ( fn2fp, FileName )
 import Data.Binary ( Binary(..) )
 import Data.Word ( Word32 )
 
@@ -40,10 +39,6 @@ instance Binary FileId where
    (rfp,cnt) <- get
    return $ FileId rfp cnt
 
--- | Parse FileId from a string
-parseFileId :: String -> FileId
-parseFileId s = let (f,'.':i) = break (=='.') s in FileId (fp2fn f) (read i)
-
 -- | Convert FileId to string
 showFileId :: FileId -> String
 showFileId (FileId fn i) = show i++"#"++fn2fp fn
@@ -59,12 +54,6 @@ instance Binary PatchId where
 pid2string :: PatchId -> String
 pid2string = show . patchId
 
--- | describes a filepath that is interpreted relative to a certain
---   point in the history of the repository. The point is given by
---   Just pid which denotes the history up to (including) pid or
---   Nothing which denotes the history including the last patch
-data DatedFilePath = DatedFilePath FilePath (Maybe PatchId)
-
 -- | This is used to track changes to files
 data PatchMod a = PTouch a
                 | PCreateFile a
@@ -79,9 +68,9 @@ data PatchMod a = PTouch a
                                      --   track of them
  deriving (Show, Eq, Functor)
 
-makePatchID :: PatchInfo -> PatchId
-makePatchID = PID . makePatchname
-
 short :: PatchId -> Word32
-short (PID (SHA1 a _ _ _ _)) = a
+short (PID sha1) = sha1short sha1
+
+zero :: PatchId
+zero = PID sha1zero
 
