@@ -6,7 +6,8 @@ module Darcs.Patch.Invert
 import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Patch.Witnesses.Ordered ( FL(..), RL(..), reverseFL, reverseRL )
+import Darcs.Patch.Witnesses.Ordered
+    ( FL(..), RL(..), reverseFL, reverseRL, (:>)(..) )
 
 
 class Invert p where
@@ -25,3 +26,6 @@ instance Invert p => Invert (FL p) where
 
 instance Invert p => Invert (RL p) where
     invert = reverseFL . invertRL
+
+instance Invert p => Invert (p :> p) where
+  invert (a :> b) = invert b :> invert a
