@@ -15,7 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, ScopedTypeVariables, MultiParamTypeClasses, FlexibleInstances, Rank2Types #-}
+{-# LANGUAGE ScopedTypeVariables, MultiParamTypeClasses, FlexibleInstances #-}
 
 -- | /First matcher, Second matcher and Nonrange matcher/
 --
@@ -47,11 +47,11 @@ module Darcs.Patch.Match
     , splitSecondFL
     , matchPatch
     , matchAPatch
-    , matchAPatchread
     , getNonrangeMatchS
     , firstMatch
     , secondMatch
     , haveNonrangeMatch
+    , haveNonrangeExplicitMatch
     , havePatchsetMatch
     , checkMatchSyntax
     , applyInvToMatcher
@@ -102,8 +102,7 @@ import Data.Char ( toLower )
 
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.Patch
-    ( Patchy
-    , IsRepoType
+    ( IsRepoType
     , hunkMatches
     , listTouchedFiles
     , invert
@@ -125,11 +124,12 @@ import Darcs.Patch.Matchable ( Matchable )
 import Darcs.Patch.MonadProgress ( MonadProgress )
 import Darcs.Patch.Named.Wrapped ( runInternalChecker, namedIsInternal, namedInternalChecker )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, conscientiously, hopefully )
-import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, newset2RL, Origin )
+import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, patchSet2RL, Origin )
 import Darcs.Patch.Type ( PatchType(..) )
-import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Apply( Apply, ApplyState )
 import Darcs.Patch.ApplyPatches( applyPatches )
 import Darcs.Patch.Depends ( getPatchesBeyondTag, splitOnTag )
+import Darcs.Patch.Invert( Invert )
 
 import Darcs.Patch.Witnesses.Eq ( isIsEq )
 import Darcs.Patch.Witnesses.Ordered ( RL(..), snocRLSealed, FL(..), (:>)(..) )
@@ -142,7 +142,6 @@ import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
 import Darcs.Util.DateMatcher ( parseDateMatcher )
 
 import Darcs.Util.Tree ( Tree )
-#include "impossible.h"
 
 -- | A type for predicates over patches which do not care about
 -- contexts
@@ -177,7 +176,7 @@ data MatchFlag =
                  deriving ( Show )
 
 
-makeMatcher :: String -> (Sealed2 (PatchInfoAnd rt p) -> Bool) -> Matcher rt p
+makeMatcher :: String -> MatchFun rt p -> Matcher rt p
 makeMatcher = MATCH
 
 -- | @applyMatcher@ applies a matcher to a patch.
@@ -197,7 +196,7 @@ matchPattern pattern =
     Left err -> error err
     Right m -> makeMatcher pattern m
 
-addInternalMatcher :: (IsRepoType rt, Matchable p) => Maybe (Matcher rt p) -> Maybe (Matcher rt p)
+addInternalMatcher :: (IsRepoType rt) => Maybe (Matcher rt p) -> Maybe (Matcher rt p)
 addInternalMatcher om =
   case namedInternalChecker of
     Nothing -> om
@@ -218,16 +217,16 @@ matchParser = submatcher <?> helpfulErrorMsg
     -- When using <?>, Parsec prepends "expecting " to the given error message,
     -- so the phrasing below makes sense.
     helpfulErrorMsg = "valid expressions over: "
-                      ++ intercalate ", " (map (\(name, _, _, _) -> name) ps)
+                      ++ intercalate ", " (map (\(name, _, _, _, _) -> name) ps)
                       ++ "\nfor more help, see `darcs help patterns`."
 
     -- This type signature is just to bind an ambiguous type var.
-    ps :: [(String, String, [String], String -> MatchFun rt DummyPatch)]
+    ps :: [(String, String, String, [String], String -> MatchFun rt DummyPatch)]
     ps = primitiveMatchers
 
     -- matchAnyPatch is returned if submatch fails without consuming any
     -- input, i.e. if we pass --match '', we want to match anything.
-    matchAnyPatch :: Matchable p => MatchFun rt p
+    matchAnyPatch :: MatchFun rt p
     matchAnyPatch = const True
 
 submatch :: Matchable p => CharParser st (MatchFun rt p)
@@ -258,16 +257,15 @@ match = between spaces spaces (parens submatch <|> choice matchers_)
   where
     matchers_ = map createMatchHelper primitiveMatchers
 
-createMatchHelper :: (String, String, [String], String -> MatchFun rt p)
+createMatchHelper :: (String, String, String, [String], String -> MatchFun rt p)
                   -> CharParser st (MatchFun rt p)
-createMatchHelper (key,_,_,matcher) =
+createMatchHelper (key,_,_,_,matcher) =
   do _ <- trystring key
      spaces
      q <- quoted
      return $ matcher q
 
--- FIXME: would this be better defined in Darcs.Commands.Help?
--- | The string that is emitted when the user runs @darcs help --match@.
+-- | The string that is emitted when the user runs @darcs help patterns@.
 helpOnMatchers :: [String]
 helpOnMatchers =
   ["Selecting Patches:",
@@ -291,45 +289,42 @@ helpOnMatchers =
   ++ ["", "Here are some examples:", ""]
   ++ examples
   where -- This type signature exists to appease GHC.
-        ps :: [(String, String, [String], String -> MatchFun rt DummyPatch)]
+        ps :: [(String, String, String, [String], String -> MatchFun rt DummyPatch)]
         ps = primitiveMatchers
-        keywords = [showKeyword k d | (k,d,_,_) <- ps]
-        examples = [showExample k e | (k,_,es,_) <- ps, e <- es]
+        keywords = [showKeyword (unwords [k,a]) d | (k,a,d,_,_) <- ps]
+        examples = [showExample k e | (k,_,_,es,_) <- ps, e <- es]
         showKeyword keyword description =
-            -- FIXME: it would be nice to have a variable name here:
-            -- "author REGEX - match against author (email address)"
-            -- or "exact STRING - match against exact patch name".
-            "- " ++ keyword ++ " - " ++ description ++ "."
+            "  " ++ keyword ++ " - " ++ description ++ "."
         showExample keyword example =
-            "    darcs log --match "
+            "  darcs log --match "
             ++ "'" ++ keyword ++ " " ++ example ++ "'"
 
-primitiveMatchers :: Matchable p => [(String, String, [String], String -> MatchFun rt p)]
-                     -- ^ keyword (operator), help description, list
+primitiveMatchers :: Matchable p => [(String, String, String, [String], String -> MatchFun rt p)]
+                     -- ^ keyword (operator), argument name, help description, list
                      -- of examples, matcher function
 primitiveMatchers =
- [ ("exact", "check a literal string against the patch name"
+ [ ("exact", "STRING", "check literal STRING is equal to patch name"
            , ["\"Resolve issue17: use dynamic memory allocation.\""]
            , exactmatch )
- , ("name", "check a regular expression against the patch name"
+ , ("name", "REGEX", "match REGEX against patch name"
           , ["issue17", "\"^[Rr]esolve issue17\\>\""]
           , namematch )
- , ("author", "check a regular expression against the author name"
+ , ("author", "REGEX", "match REGEX against patch author"
             , ["\"David Roundy\"", "droundy", "droundy@darcs.net"]
             , authormatch )
- , ("hunk", "check a regular expression against the contents of a hunk patch"
+ , ("hunk", "REGEX", "match REGEX against contents of a hunk patch"
             , ["\"foo = 2\"", "\"^instance .* Foo where$\""]
             , hunkmatch )
- , ("comment", "check a regular expression against the log message"
+ , ("comment", "REGEX", "match REGEX against the full log message"
          , ["\"prevent deadlocks\""]
          , logmatch )
- , ("hash",  "match a full hash or a prefix for a patch"
+ , ("hash", "HASH", "match HASH against (a prefix of) the hash of a patch"
           ,  ["c719567e92c3b0ab9eddd5290b705712b8b918ef","c7195"]
           ,  hashmatch )
- , ("date", "match the patch date"
+ , ("date", "DATE", "match DATE against the patch date"
           , ["\"2006-04-02 22:41\"", "\"tea time yesterday\""]
           , datematch )
- , ("touch", "match file paths for a patch"
+ , ("touch", "REGEX", "match file paths for a patch"
           , ["src/foo.c", "src/", "\"src/*.(c|h)\""]
           , touchmatch ) ]
 
@@ -346,7 +341,10 @@ quoted = between (char '"') (char '"')
          <|> between spaces spaces (many $ noneOf " ()")
          <?> "string"
 
-namematch, exactmatch, authormatch, hunkmatch, hashmatch, datematch, touchmatch
+datematch, hashmatch, authormatch, exactmatch, namematch, logmatch
+  :: String -> MatchFun rt p
+
+hunkmatch, touchmatch
   :: Matchable p => String -> MatchFun rt p
 
 namematch r (Sealed2 hp) = isJust $ matchRegex (mkRegex r) $ justName (info hp)
@@ -355,7 +353,6 @@ exactmatch r (Sealed2 hp) = r == justName (info hp)
 
 authormatch a (Sealed2 hp) = isJust $ matchRegex (mkRegex a) $ justAuthor (info hp)
 
-logmatch :: Matchable p => String -> MatchFun rt p
 logmatch l (Sealed2 hp) = isJust $ matchRegex (mkRegex l) $ justLog (info hp)
 
 hunkmatch r (Sealed2 hp) = let regexMatcher = isJust . matchRegex (mkRegex r) . BC.unpack
@@ -373,14 +370,31 @@ touchmatch r (Sealed2 hp) = let files = listTouchedFiles hp
 
 data InclusiveOrExclusive = Inclusive | Exclusive deriving Eq
 
+data IncludeInternalPatches = IncludeInternalPatches | ExcludeInternalPatches
+                              deriving Eq
+
 -- | @haveNonrangeMatch flags@ tells whether there is a flag in
 -- @flags@ which corresponds to a match that is "non-range". Thus,
 -- @--match@, @--patch@, @--hash@ and @--index@ make @haveNonrangeMatch@
 -- true, but not @--from-patch@ or @--to-patch@.
-haveNonrangeMatch :: forall rt p . (IsRepoType rt, Matchable p) => PatchType rt p -> [MatchFlag] -> Bool
-haveNonrangeMatch _ fs =
+haveNonrangeMatch :: forall rt p . (IsRepoType rt, Matchable p)
+                  => PatchType rt p -> [MatchFlag] -> Bool
+haveNonrangeMatch pt fs = haveNonrangeMatch' IncludeInternalPatches pt fs
+
+-- | @haveNonrangeExplicitMatch flags@ is just like @haveNonrangeMatch flags@,
+-- but ignores "internal matchers" used to mask "internal patches"
+haveNonrangeExplicitMatch :: forall rt p . (IsRepoType rt, Matchable p)
+                          => PatchType rt p -> [MatchFlag] -> Bool
+haveNonrangeExplicitMatch pt fs = haveNonrangeMatch' ExcludeInternalPatches pt fs
+
+haveNonrangeMatch' :: forall rt p . (IsRepoType rt, Matchable p)
+                   => IncludeInternalPatches -> PatchType rt p -> [MatchFlag] -> Bool
+haveNonrangeMatch' i _ fs =
      case hasIndexRange fs of Just (m,n) | m == n -> True; _ -> False
-  || isJust (nonrangeMatcher fs::Maybe (Matcher rt p))
+  || isJust (nonrangeMatch::Maybe (Matcher rt p))
+    where
+     nonrangeMatch | i == IncludeInternalPatches = nonrangeMatcher fs
+                   | otherwise = nonrangeMatcherArgs fs
 
 -- | @havePatchsetMatch flags@ tells whether there is a "patchset
 -- match" in the flag list. A patchset match is @--match@ or
@@ -438,11 +452,11 @@ getFirstMatchS fs repo =
 secondMatch :: [MatchFlag] -> Bool
 secondMatch fs = isJust (secondMatcher fs::Maybe (Matcher rt DummyPatch)) || isJust (hasIndexRange fs)
 
-unpullLastN :: (ApplyMonad (ApplyState p) m, MonadProgress m, Patchy p, IsRepoType rt)
+unpullLastN :: (Apply p, Invert p, ApplyMonad (ApplyState p) m, MonadProgress m, IsRepoType rt)
             => PatchSet rt p wX wY
             -> Int
             -> m ()
-unpullLastN repo n = applyInvRL `unsealFlipped` safetake n (newset2RL repo)
+unpullLastN repo n = applyInvRL `unsealFlipped` safetake n (patchSet2RL repo)
 
 checkMatchSyntax :: [MatchFlag] -> IO ()
 checkMatchSyntax opts =
@@ -456,16 +470,16 @@ getMatchPattern (OnePattern m:_) = Just m
 getMatchPattern (SeveralPattern m:_) = Just m
 getMatchPattern (_:fs) = getMatchPattern fs
 
-tagmatch :: Matchable p => String -> Matcher rt p
+tagmatch :: String -> Matcher rt p
 tagmatch r = makeMatcher ("tag-name "++r) tm
     where tm (Sealed2 p) =
               let n = justName (info p) in
               "TAG " `isPrefixOf` n && isJust (matchRegex (mkRegex r) $ drop 4 n)
 
-patchmatch :: Matchable p => String -> Matcher rt p
+patchmatch :: String -> Matcher rt p
 patchmatch r = makeMatcher ("patch-name "++r) (namematch r)
 
-hashmatch' :: Matchable p => String -> Matcher rt p
+hashmatch' :: String -> Matcher rt p
 hashmatch' r = makeMatcher ("hash "++r) (hashmatch r)
 
 
@@ -535,18 +549,11 @@ secondMatcherIsTag [] = False
 secondMatcherIsTag (UpToTag _:_) = True
 secondMatcherIsTag (_:fs) = secondMatcherIsTag fs
 
--- | @matchAPatchread fs p@ tells whether @p@ matches the matchers in
--- the flags listed in @fs@.
-matchAPatchread :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchInfoAnd rt p wX wY -> Bool
-matchAPatchread fs = case nonrangeMatcher fs of
-                       Nothing -> const True
-                       Just m -> applyMatcher m
-
 -- | @matchAPatch fs p@ tells whether @p@ matches the matchers in
 -- the flags @fs@
 matchAPatch :: (IsRepoType rt, Matchable p) => [MatchFlag] -> PatchInfoAnd rt p wX wY -> Bool
 matchAPatch fs p =
-    case nonrangeMatcher fs of
+  case nonrangeMatcher fs of
     Nothing -> True
     Just m -> applyMatcher m p
 
@@ -647,8 +654,8 @@ findAPatch m (PatchSet (ts :<: Tagged t _ ps) NilRL) = findAPatch m (PatchSet ts
 findAPatch m (PatchSet ts (ps:<:p)) | applyMatcher m p = seal2 $ hopefully p
                                     | otherwise = findAPatch m (PatchSet ts ps)
 
--- | @matchAPatchset m ps@ returns a (the largest?) subset of @ps@
--- ending in patch which matches @m@. Calls 'error' if there is none.
+-- | @matchAPatchset m ps@ returns a prefix of @ps@
+-- ending in a patch matching @m@, and calls 'error' if there is none.
 matchAPatchset :: Matchable p => Matcher rt p -> PatchSet rt p wStart wX
                -> SealedPatchSet rt p wStart
 matchAPatchset m (PatchSet NilRL NilRL) = error $ "Couldn't find patch matching " ++ show m
@@ -723,7 +730,7 @@ getTagS matcher repo = do
 -- | @applyInvp@ tries to get the patch that's in a 'PatchInfoAnd
 -- patch', and to apply its inverse. If we fail to fetch the patch
 -- then we share our sorrow with the user.
-applyInvp :: (Patchy p, ApplyMonad (ApplyState p) m) => PatchInfoAnd rt p wX wY -> m ()
+applyInvp :: (Apply p, Invert p, ApplyMonad (ApplyState p) m) => PatchInfoAnd rt p wX wY -> m ()
 applyInvp hp = apply (invert $ fromHopefully hp)
     where fromHopefully = conscientiously $ \e ->
                      text "Sorry, patch not available:"
@@ -739,5 +746,5 @@ safetake _ NilRL = error "There aren't that many patches..."
 safetake i (as:<:a) | isIsEq (namedIsInternal (hopefully a)) = safetake i as `snocRLSealed` a
 safetake i (as:<:a) = safetake (i-1) as `snocRLSealed` a
 
-applyInvRL :: (ApplyMonad (ApplyState p) m, MonadProgress m, Patchy p) => RL (PatchInfoAnd rt p) wX wR -> m ()
+applyInvRL :: (Apply p, Invert p, ApplyMonad (ApplyState p) m, MonadProgress m) => RL (PatchInfoAnd rt p) wX wR -> m ()
 applyInvRL = applyPatches . invertRL -- this gives nicer feedback
