@@ -55,5 +55,5 @@ class Monad m => MonadProgress m where
 silentlyRunProgressActions :: Monad m => String -> [ProgressAction m ()] -> m ()
 silentlyRunProgressActions _ = mapM_ paAction
 
-instance (Functor m, Monad m) => MonadProgress (TM.TreeMonad m) where
+instance (Monad m) => MonadProgress (TM.TreeMonad m) where
   runProgressActions = silentlyRunProgressActions
