@@ -1,9 +1,6 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.Prim.FileUUID ( Prim ) where
 
-import Prelude ()
-import Darcs.Prelude
-
 import Darcs.Patch.Prim.FileUUID.Apply ()
 import Darcs.Patch.Prim.FileUUID.Coalesce ()
 import Darcs.Patch.Prim.FileUUID.Commute ()
@@ -12,11 +9,10 @@ import Darcs.Patch.Prim.FileUUID.Details ()
 import Darcs.Patch.Prim.FileUUID.Read ()
 import Darcs.Patch.Prim.FileUUID.Show ()
 
-import Darcs.Patch.Prim.Class ( PrimPatch, PrimPatchBase(..), FromPrim(..) )
-import Darcs.Patch.Patchy ( Patchy )
+import Darcs.Patch.Prim.Class ( PrimPatchCommon, PrimPatch, PrimPatchBase(..), FromPrim(..) )
 
+instance PrimPatchCommon Prim
 instance PrimPatch Prim
-instance Patchy Prim
 
 instance PrimPatchBase Prim where
   type PrimOf Prim = Prim
