@@ -3,7 +3,9 @@ module Darcs.Patch.Prim.V1.Details
     ()
     where
 
-import Prelude hiding ( pi )
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Prim.Class ( PrimDetails(..) )
 import Darcs.Patch.Prim.V1.Core
     ( Prim(..), FilePatchType(..), DirPatchType(..) )
