@@ -20,13 +20,13 @@ import Darcs.Patch.Prim ( FromPrim(..) )
 import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.Invert ( Invert(..) )
 import Darcs.Patch.Named ( Named(..), commuterNamedId, commuterIdNamed )
-import Darcs.Patch.Prim ( PrimPatchBase, PrimOf )
+import Darcs.Patch.Prim ( PrimPatchBase(..), PrimPatch )
 import Darcs.Patch.Rebase.Name
     ( RebaseName(..)
     , commuteNamedName, commuteNameNamed
     , commutePrimName, commuteNamePrim
     )
-import Darcs.Patch.Witnesses.Eq ( MyEq(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), mapFL_FL, (:>)(..), (+>+) )
 import Darcs.Patch.Witnesses.Show
@@ -37,10 +37,10 @@ import Darcs.Patch.Witnesses.Show
 -- |A single rebase fixup, needed to ensure that the actual patches
 -- being stored in the rebase state have the correct context.
 data RebaseFixup p wX wY where
-  PrimFixup :: PrimOf p wX wY -> RebaseFixup p wX wY
+  PrimFixup :: PrimPatch (PrimOf p) => PrimOf p wX wY -> RebaseFixup p wX wY
   NameFixup :: RebaseName p wX wY -> RebaseFixup p wX wY
 
-namedToFixups :: Effect p => Named p wX wY -> FL (RebaseFixup p) wX wY
+namedToFixups :: (PrimPatch (PrimOf p), Effect p) => Named p wX wY -> FL (RebaseFixup p) wX wY
 namedToFixups (NamedP p _ contents) = NameFixup (AddName p) :>: mapFL_FL PrimFixup (effect contents)
 
 instance Show2 (PrimOf p) => Show (RebaseFixup p wX wY) where
@@ -58,7 +58,7 @@ instance Show2 (PrimOf p) => Show2 (RebaseFixup p) where
 instance PrimPatchBase p => PrimPatchBase (RebaseFixup p) where
     type PrimOf (RebaseFixup p) = PrimOf p
 
-instance (PrimPatchBase p, Apply p, ApplyState p ~ ApplyState (PrimOf p)) => Apply (RebaseFixup p) where
+instance (PrimPatchBase p, ApplyState p ~ ApplyState (PrimOf p)) => Apply (RebaseFixup p) where
     type ApplyState (RebaseFixup p) = ApplyState p
     apply (PrimFixup p) = apply p
     apply (NameFixup p) = apply p
@@ -67,7 +67,7 @@ instance Effect (RebaseFixup p) where
     effect (PrimFixup p) = p :>: NilFL
     effect (NameFixup p) = effect p
 
-instance MyEq (PrimOf p) => MyEq (RebaseFixup p) where
+instance Eq2 (PrimOf p) => Eq2 (RebaseFixup p) where
     PrimFixup p1 `unsafeCompare` PrimFixup p2 = p1 `unsafeCompare` p2
     PrimFixup _ `unsafeCompare` _ = False
     _ `unsafeCompare` PrimFixup _ = False
@@ -164,4 +164,4 @@ commuteFixupNamed (PrimFixup p :> q) = do
     return (q' :> mapFL_FL PrimFixup ps')
 commuteFixupNamed (NameFixup n :> q) = do
     q' :> n' <- commuteNameNamed (n :> q)
-    return (q' :> (NameFixup n' :>: NilFL))
\ No newline at end of file
+    return (q' :> (NameFixup n' :>: NilFL))
