@@ -15,59 +15,59 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.Patch.Show
      ( ShowPatchBasic(..)
+     , displayPatch
+     , ShowPatchFor(..)
      , ShowPatch(..)
-     , showNamedPrefix
+     , ShowContextPatch(..)
      , formatFileName
      ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Prelude hiding ( pi )
+import qualified Data.ByteString.Char8 as BC ( unpack )
 
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.ApplyMonad ( ApplyMonad )
 import Darcs.Patch.Format ( FileNameFormat(..) )
-import Darcs.Patch.Info ( PatchInfo, showPatchInfo )
-import Darcs.Patch.Witnesses.Ordered ( FL )
+import Darcs.Patch.Witnesses.Ordered ( FL, mapFL )
 
+import Darcs.Util.ByteString ( packStringToUTF8, encodeLocale )
 import Darcs.Util.English ( plural, Noun(Noun) )
-import Darcs.Util.Path ( FileName, fn2ps, encodeWhite, fn2fp )
-import Darcs.Util.Printer ( Doc, vcat, blueText, ($$), text, packedString )
+import Darcs.Util.Path ( FileName, encodeWhite, fn2fp )
+import Darcs.Util.Printer ( Doc, vcat, text, packedString )
 
-showNamedPrefix :: PatchInfo -> [PatchInfo] -> Doc
-showNamedPrefix n d = showPatchInfo n
-                      $$ blueText "<"
-                      $$ vcat (map showPatchInfo d)
-                      $$ blueText ">"
+data ShowPatchFor = ForDisplay | ForStorage
+
+displayPatch :: ShowPatchBasic p => p wX wY -> Doc
+displayPatch p = showPatch ForDisplay p
 
 class ShowPatchBasic p where
-    showPatch :: p wX wY -> Doc
-
-class ShowPatchBasic p => ShowPatch p where
-    showNicely :: p wX wY -> Doc
-    showNicely = showPatch
+    showPatch :: ShowPatchFor -> p wX wY -> Doc
 
+class ShowPatchBasic p => ShowContextPatch p where
     -- | showContextPatch is used to add context to a patch, as diff
     -- -u does. Thus, it differs from showPatch only for hunks. It is
     -- used for instance before putting it into a bundle. As this
     -- unified context is not included in patch representation, this
     -- requires access to the tree.
-    showContextPatch :: (Monad m,
-                         ApplyMonad (ApplyState p) m)
-                     => p wX wY -> m Doc
-    showContextPatch p = return $ showPatch p
+    showContextPatch :: (ApplyMonad (ApplyState p) m)
+                     => ShowPatchFor -> p wX wY -> m Doc
+
+-- This class is used only for user interaction, not for storage
+class ShowPatchBasic p => ShowPatch p where
+    showNicely :: p wX wY -> Doc
+    showNicely = showPatch ForDisplay
 
     description :: p wX wY -> Doc
-    description = showPatch
+    description = showPatch ForDisplay
 
     summary :: p wX wY -> Doc
 
     summaryFL :: FL p wX wY -> Doc
+    summaryFL = vcat . mapFL summary
 
     thing :: p wX wY -> String
     thing _ = "patch"
@@ -75,6 +75,19 @@ class ShowPatchBasic p => ShowPatch p where
     things :: p wX wY -> String
     things x = plural (Noun $ thing x) ""
 
+-- | Format a 'FileName' to a 'Doc' according to the given 'FileNameFormat'.
+--
+-- NOTE: This is not only used for display but also to format patch files. This is
+--       why we have to do the white space encoding here.
+--       See 'Darcs.Repository.Hashed.writePatchIfNecessary'.
+--
+-- Besides white space encoding, for 'NewFormat' we just pack it into a 'Doc'. For
+-- 'OldFormat' we must emulate the non-standard darcs-1 encoding of file paths: it
+-- is an UTF8 encoding of the raw byte stream, interpreted as code points.
+--
+-- See also 'Darcs.Patch.Show.readFileName'.
 formatFileName :: FileNameFormat -> FileName -> Doc
-formatFileName OldFormat = packedString . fn2ps
+formatFileName OldFormat = packedString . packStringToUTF8 . BC.unpack .
+                            encodeLocale . encodeWhite . fn2fp
 formatFileName NewFormat = text . encodeWhite . fn2fp
+formatFileName UserFormat = text . fn2fp
