@@ -1,13 +1,16 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.V1.Read () where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Patch.Invert ( invert )
 import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.Read ( ReadPatch(..) )
 import Darcs.Patch.ReadMonads ( ParserM, choice, string,
                                 lexChar, myLex', skipSpace )
 
-import Darcs.Patch.V1.Core ( Patch(..) )
+import Darcs.Patch.V1.Core ( RepoPatchV1(..) )
 import Darcs.Patch.V1.Commute ( merger )
 
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), seal, mapSeal )
@@ -18,13 +21,13 @@ import qualified Data.ByteString.Char8 as BC ( unpack, pack )
 import qualified Data.ByteString       as B  (ByteString )
 
 
-instance PrimPatch prim => ReadPatch (Patch prim) where
+instance PrimPatch prim => ReadPatch (RepoPatchV1 prim) where
  readPatch'
    = choice [ liftM seal $ skipSpace >> readMerger True
             , liftM seal $ skipSpace >> readMerger False
             , liftM (mapSeal PP) readPatch'
             ]
-readMerger :: (ParserM m, PrimPatch prim) => Bool -> m (Patch prim wX wY)
+readMerger :: (ParserM m, PrimPatch prim) => Bool -> m (RepoPatchV1 prim wX wY)
 readMerger b = do string s
                   g <- myLex'
                   lexChar '('
