@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, FlexibleInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
 {-# OPTIONS_HADDOCK ignore-exports #-}
 
 module Darcs.Patch.Witnesses.Sealed
@@ -46,7 +46,7 @@ module Darcs.Patch.Witnesses.Sealed
 import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Patch.Witnesses.Eq ( MyEq, EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2, EqCheck(..) )
 import Darcs.Patch.Witnesses.Show
 import Darcs.Patch.Witnesses.Eq ( (=\/=) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP1, unsafeCoerceP )
@@ -60,7 +60,7 @@ data Sealed a where
 seal :: a wX -> Sealed a
 seal = Sealed
 
-instance MyEq a => Eq (Sealed (a wX)) where
+instance Eq2 a => Eq (Sealed (a wX)) where
     Sealed x == Sealed y | IsEq <- x =\/= y = True
                          | otherwise = False
 
@@ -139,7 +139,7 @@ newtype Stepped (f :: (* -> *) -> *) a wX = Stepped { unStepped :: f (a wX) }
 -- of the type witnesses.
 newtype FreeLeft p = FLInternal (Poly (Stepped Sealed p))
 
--- |'FreeLeft' p is @ \forall y . \exists x . p x y @
+-- |'FreeRight' p is @ \forall y . \exists x . p x y @
 -- In other words the caller is free to specify the right witness,
 -- and then the left witness is an existential.
 -- Note that the order of the type constructors is important for ensuring
