@@ -28,7 +28,6 @@
 -- Stability   : experimental
 -- Portability : portable
 
-{-# LANGUAGE CPP #-}
 module Darcs.Repository.Diff
     (
       treeDiff
@@ -38,7 +37,7 @@ import Prelude ()
 import Darcs.Prelude
 
 import qualified Data.ByteString.Lazy.Char8 as BLC
-import qualified Data.ByteString as BS
+import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 
 import Data.List ( sortBy )
@@ -69,9 +68,6 @@ import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+) )
 import Darcs.Patch.Witnesses.Sealed ( Gap(..) )
 import Darcs.Repository.Flags ( DiffAlgorithm(..) )
 
-#include "impossible.h"
-
-
 data Diff m = Added (TreeItem m)
             | Removed (TreeItem m)
             | Changed (TreeItem m) (TreeItem m)
@@ -87,7 +83,7 @@ getDiff p (Just t) Nothing = (p, Removed t)
 getDiff _ Nothing Nothing = impossible -- zipTrees should never return this
 
 
-treeDiff :: forall m w prim . (Functor m, Monad m, Gap w, PrimPatch prim)
+treeDiff :: forall m w prim . (Monad m, Gap w, PrimPatch prim)
          => DiffAlgorithm
          -> (FilePath -> FileType)
          -> Tree m
@@ -148,12 +144,40 @@ treeDiff da ft t1 t2 = do
         | BL.null a && BL.null b = emptyGap NilFL
         | BL.null a = freeGap (diff_from_empty p b)
         | BL.null b = freeGap (diff_to_empty p a)
+
+        -- What is 'a line'? One view is that a line is something that is
+        -- /terminated/ by either a newline or end of file. Another view is
+        -- that lines are /separated/ by newline symbols.
+        --
+        -- The first view is the more "intuitive" one. The second is more
+        -- "technical", it has the simpler definition and the highly desirable
+        -- property that splitting a text into lines and joining them with
+        -- newline symbols are inverse operations. The last point is the reason
+        -- we never use the standard versions of 'unlines' for ByteString
+        -- anywhere in darcs.
+        --
+        -- The two views differ mostly when enumerating the lines of a file
+        -- that ends with a newline symbol: here, the technical view counts one
+        -- more (empty) line. This leads to un-intuitive (though technically
+        -- not incorrect) results when calculating the diff for a change that
+        -- appends an empty line to a file that already has a newline at the
+        -- end. For instance, for a file with a single, newline-terminated line
+        -- of text, the LCS algorithm would tell us that a *third* (empty) line
+        -- is being added.
+        --
+        -- To avoid this, we add a special case here: we strip off common
+        -- newline symbols at the end. When we later split the result into
+        -- lines for the diff algorithm, it never gets to see the empty
+        -- last lines in both files and thus gives us the more intuitive result.
+
+        | BLC.last a == '\n' && BLC.last b == '\n'
+                    = freeGap (line_diff p (linesB $ BLC.init a) (linesB $ BLC.init b))
         | otherwise = freeGap (line_diff p (linesB a) (linesB b))
 
     line_diff p a b = canonize da (hunk p 1 a b)
 
     diff_to_empty p x | BLC.last x == '\n' = line_diff p (init $ linesB x) []
-                      | otherwise = line_diff p (linesB x) [BS.empty]
+                      | otherwise = line_diff p (linesB x) [B.empty]
 
     diff_from_empty p x = invert (diff_to_empty p x)
 
@@ -161,4 +185,4 @@ treeDiff da ft t1 t2 = do
 
     linesB = map strict . BLC.split '\n'
 
-    strict = BS.concat . BL.toChunks
+    strict = B.concat . BL.toChunks
