@@ -2,8 +2,6 @@
 --
 -- This file is licensed under the GPL, version two or later.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.Repository.Format
     ( RepoFormat(..)
     , RepoProperty(..)
@@ -22,10 +20,8 @@ module Darcs.Repository.Format
 import Prelude ()
 import Darcs.Prelude
 
-#include "impossible.h"
-
 import Control.Monad ( mplus, (<=<) )
-import qualified Data.ByteString.Char8 as BC ( split, unpack, elemIndex )
+import qualified Data.ByteString.Char8 as BC ( split, pack, unpack, elemIndex )
 import qualified Data.ByteString  as B ( null, empty )
 import Data.List ( partition, intercalate, (\\) )
 import Data.Maybe ( isJust, mapMaybe )
@@ -149,7 +145,8 @@ tryIdentifyRepoFormat repo = do
 
 -- | Write the repo format to the given file.
 writeRepoFormat :: RepoFormat -> FilePath -> IO ()
-writeRepoFormat rf loc = writeBinFile loc $ show rf
+writeRepoFormat rf loc = writeBinFile loc $ BC.pack $ show rf
+-- note: this assumes show returns ascii
 
 -- | Create a repo format. The first argument is whether to use the old (darcs-1)
 -- format; the second says whether the repo has a working tree.
