@@ -14,22 +14,31 @@
 -- along with this program; if not, write to the Free Software Foundation,
 -- Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-{-# LANGUAGE CPP #-}
-
-
-module Darcs.Repository.InternalTypes ( Repository(..), Pristine(..)
-                                      , extractCache, modifyCache
+module Darcs.Repository.InternalTypes ( Repository, PristineType(..)
+                                      , repoCache, modifyCache
+                                      , repoPatchType
+                                      , repoFormat
+                                      , repoLocation
+                                      , withRepoLocation
+                                      , repoPristineType
+                                      , coerceR
+                                      , coerceU
+                                      , coerceT
+                                      , mkRepo
                                       ) where
 
 import Prelude ()
 import Darcs.Prelude
 
+import Data.Coerce ( coerce )
 import Data.List ( nub, sortBy )
 import Darcs.Repository.Cache ( Cache (..) , compareByLocality )
 import Darcs.Repository.Format ( RepoFormat )
-import Darcs.Patch ( RepoPatch, RepoType )
+import Darcs.Patch ( RepoType )
+import Darcs.Patch.Type ( PatchType(..) )
+import Darcs.Util.File ( withCurrentDirectory )
 
-data Pristine
+data PristineType
   = NoPristine
   | PlainPristine
   | HashedPristine
@@ -42,14 +51,41 @@ data Pristine
 -- and the tentative state, which represents work in progress that will
 -- eventually become the new recorded state unless something goes wrong.
 data Repository (rt :: RepoType) (p :: * -> * -> *) wRecordedstate wUnrecordedstate wTentativestate =
-  Repo !String !RepoFormat !Pristine Cache deriving ( Show )
+  Repo !String !RepoFormat !PristineType Cache deriving ( Show )
 
-extractCache :: Repository rt p wR wU wT -> Cache
-extractCache (Repo _ _ _ c) = c
+repoLocation :: Repository rt p wR wU wT -> String
+repoLocation (Repo loc _ _ _) = loc
+
+withRepoLocation :: Repository rt p wR wU wT -> IO a -> IO a
+withRepoLocation repo = withCurrentDirectory (repoLocation repo)
+
+repoFormat :: Repository rt p wR wU wT -> RepoFormat
+repoFormat (Repo _ fmt _ _) = fmt
+
+repoPristineType :: Repository rt p wR wU wT -> PristineType
+repoPristineType (Repo _ _ pr _) = pr
+
+repoCache :: Repository rt p wR wU wT -> Cache
+repoCache (Repo _ _ _ c) = c
 
 -- | 'modifyCache' @repository function@ modifies the cache of
 --   @repository@ with @function@, remove duplicates and sort the results with 'compareByLocality'.
-modifyCache :: forall rt p wR wU wT . (RepoPatch p)  => Repository rt p wR wU wT -> (Cache -> Cache) -> Repository rt p wR wU wT
+modifyCache :: forall rt p wR wU wT . Repository rt p wR wU wT -> (Cache -> Cache) -> Repository rt p wR wU wT
 modifyCache (Repo dir rf pristine cache) f
    = Repo dir rf pristine $ cmap ( sortBy compareByLocality . nub ) $ f cache
   where cmap g (Ca c) = Ca (g c)
+
+repoPatchType :: Repository rt p wR wU wT -> PatchType rt p
+repoPatchType _ = PatchType
+
+coerceR :: Repository rt p wR wU wT -> Repository rt p wR' wU wT
+coerceR = coerce
+
+coerceU :: Repository rt p wR wU wT -> Repository rt p wR wU' wT
+coerceU = coerce
+
+coerceT :: Repository rt p wR wU wT -> Repository rt p wR wU wT'
+coerceT = coerce
+
+mkRepo :: String -> RepoFormat -> PristineType -> Cache -> Repository rt p wR wU wT
+mkRepo = Repo
