@@ -1,41 +1,42 @@
-{-# LANGUAGE CPP, NamedFieldPuns #-}
+{-# LANGUAGE NamedFieldPuns #-}
 
--- Copyright (C) 2009-2010 Benedikt Schmidt
---
--- This program is free software; you can redistribute it and/or modify
--- it under the terms of the GNU General Public License as published by
--- the Free Software Foundation; either version 2, or (at your option)
--- any later version.
---
--- This program is distributed in the hope that it will be useful,
--- but WITHOUT ANY WARRANTY; without even the implied warranty of
--- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--- GNU General Public License for more details.
---
--- You should have received a copy of the GNU General Public License
--- along with this program; see the file COPYING.  If not, write to
--- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
--- Boston, MA 02110-1301, USA.
+{-|
+License : GPL-2
 
+The patch-index stores additional information that is extracted from
+the PatchSet for the repository to speed up certain commands (namely
+@log@ and @annotate@). More precisely, for every file tracked by the
+repository, it stores the list of patches that touch it.
+
+When created, patch-index lives in @_darcs\/patch_index\/@, and it
+should be automatically maintained each time the set of patches of
+the repository is updated.
+
+Patch-index can also be explicitely disabled by creating a file
+@_darcs\/no_patch_index@. "Explicitely disabed" means that no command
+should attempt to automatically create the patch-index.
+
+See <http://darcs.net/Internals/PatchIndex> for more information.
+-}
 module Darcs.Repository.PatchIndex (
   doesPatchIndexExist,
   isPatchIndexDisabled,
   isPatchIndexInSync,
   canUsePatchIndex,
-  canCreatePI,
   createPIWithInterrupt,
   createOrUpdatePatchIndexDisk,
   deletePatchIndex,
-  dumpPatchIndex,
-  filterPatches,
+  attemptCreatePatchIndex,
   PatchFilter,
   maybeFilterPatches,
   getRelevantSubsequence,
-  piTest,
-  attemptCreatePatchIndex
+  dumpPatchIndex,
+  piTest
 ) where
 
-import Prelude hiding ( pi, (<$>) )
+import Prelude ()
+import Darcs.Prelude
+
 import Data.Binary ( encodeFile, decodeFile )
 import Data.Word ( Word32 )
 import Data.Int ( Int8 )
@@ -51,43 +52,27 @@ import Control.Monad.State.Strict ( evalState, execState, State, gets, modify )
 import Control.Applicative ( (<$>) )
 import System.Directory ( createDirectory, renameDirectory, doesFileExist, doesDirectoryExist )
 import Darcs.Repository.Format ( formatHas, RepoProperty( HashedInventory ) )
-import Darcs.Repository.InternalTypes ( Repository(..) )
-import Darcs.Repository.HashedRepo ( readRepo )
+import Darcs.Repository.InternalTypes ( Repository, repoLocation, repoFormat )
 import Darcs.Patch.Witnesses.Ordered ( mapFL, RL(..), FL(..), reverseRL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed2(..), Sealed(..), seal, seal2, unsafeUnseal )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd(..), info )
 import Darcs.Util.Lock ( withPermDir, rmRecursive )
-import Darcs.Patch ( RepoPatch, IsRepoType, listTouchedFiles )
+import Darcs.Patch ( RepoPatch, listTouchedFiles )
 import Darcs.Util.Path ( FileName, fp2fn, fn2fp, toFilePath )
-import Darcs.Patch.Apply ( applyToFileMods, ApplyState(..) )
-import Darcs.Patch.Set ( newset2FL, Origin, newset2FL )
-import Darcs.Patch.Patchy ( Commute )
+import Darcs.Patch.Apply ( ApplyState(..) )
+import Darcs.Patch.Set ( PatchSet(..), patchSet2FL, Origin, patchSet2FL )
 import Darcs.Patch.Inspect ( PatchInspect )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.Progress ( debugMessage )
 import Darcs.Patch.Index.Types
+import Darcs.Patch.Index.Monad ( applyToFileMods, makePatchID )
 import System.FilePath( (</>) )
 import System.IO (openFile, IOMode(WriteMode), hClose)
 import qualified Data.ByteString as B
-import Darcs.Util.Crypt.SHA256 (sha256sum )
-import Darcs.Util.Crypt.SHA1 ( SHA1(..), showAsHex )
+import Darcs.Util.Hash ( sha256sum, showAsHex )
 import Darcs.Util.Tree ( Tree(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Util.SignalHandler ( catchInterrupt )
-#include "impossible.h"
-
-{- -----------------------------------------------------------------------------
-   The patch index stores additional information that is extracted from
-   the PatchSet for the repository to speed up certain commands.
-
-  createPatchIndexDisk:
-     Create the on-disk patch-index index from scratch.
-   updatePatchIndexDisk:
-     Update the on-disk patch-index index.
-  ----------------------------------------------------------------------------- -}
-
--- ---------------------------------------------------------------------
--- Data structures for the patch-index
 
 data FileIdSpan = FidSpan
                     !FileId           -- the fileid has some fixed name in the
@@ -128,26 +113,13 @@ data PatchIndex =
         infom::InfoMap
     }
 
--- | an empty patch-index
-emptyPatchIndex :: PatchIndex
-emptyPatchIndex = PatchIndex [] M.empty M.empty M.empty
-
 -- | On-disk version of patch index
 --   version 1 is the one introduced in darcs 2.10
 --           2 changes the pids order to newer-to-older
 version :: Int8
 version = 2
 
--- ---------------------------------------------------------------------
--- Query the patch-index
-
-getInventoryHash :: FilePath -> IO String
-getInventoryHash repodir = do
-  inv <- B.readFile (repodir </> darcsdir </> "hashed_inventory")
-  return $ sha256sum inv
-
--- ---------------------------------------------------------------------
--- create patch-index
+type PIM a = State PatchIndex a
 
 -- | 'applyPatchMods pmods pindex' applies a list of PatchMods to the given
 --   patch index pindex
@@ -155,12 +127,12 @@ applyPatchMods :: [(PatchId, [PatchMod FileName])] -> PatchIndex -> PatchIndex
 applyPatchMods pmods pindex =
   flip execState pindex $ mapM_ goList pmods
  where goList :: (PatchId, [PatchMod FileName]) -> PIM ()
-       -- nubSeq handles invalid patch in darcs repo:
-       --   move with identical target name "rename darcs_patcher to darcs-patcher."
        goList (pid, mods) = do
            modify (\pind -> pind{pids = pid:pids pind})
            mapM_ (curry go pid) (nubSeq mods)
-
+       -- nubSeq handles invalid patch in darcs repo:
+       --   move with identical target name "rename darcs_patcher to darcs-patcher."
+       nubSeq = map head . group
        go :: (PatchId, PatchMod FileName) -> PIM ()
        go (pid, PCreateFile fn) = do
          fid <- createFidStartSpan fn pid
@@ -196,11 +168,6 @@ applyPatchMods pmods pindex =
            Just [] -> error $ "applyPatchMods: impossible, no entry for "++show fn
                               ++" in FileIdSpans in duplicate, empty list"
 
--- ---------------------------------------------------------------------
--- Update and query patch index
-
-type PIM a = State PatchIndex a
-
 -- | create new filespan for created file
 createFidStartSpan :: FileName -> PatchId -> PIM FileId
 createFidStartSpan fn pstart = do
@@ -309,24 +276,18 @@ lookupFids' fn = do
                   Nothing -> error "lookupFids' : could not find file"
               else return []
 
--- | remove sequential duplicates
-nubSeq :: Eq a => [a] -> [a]
-nubSeq = map head . group
-
--- ---------------------------------------------------------------------
--- Create/Update patch-index on disk
-
--- | create patch index that corresponds to all patches in repo
+-- | Creates patch index that corresponds to all patches in repo.
 createPatchIndexDisk
-  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-  => Repository rt p wR wU wT -> IO ()
-createPatchIndexDisk repository@(Repo r _ _ _) = do
-  rawpatches <- newset2FL `fmap` readRepo repository r
-  let patches = mapFL Sealed2 rawpatches
+  :: (RepoPatch p, ApplyState p ~ Tree)
+  => Repository rt p wR wU wT
+  -> PatchSet rt p Origin wR
+  -> IO ()
+createPatchIndexDisk repository ps = do
+  let patches = mapFL Sealed2 $ patchSet2FL ps
   createPatchIndexFrom repository $ patches2patchMods patches S.empty
 
 -- | convert patches to patchmods
-patches2patchMods :: (Apply p, Commute p, PatchInspect p, ApplyState p ~ Tree)
+patches2patchMods :: (Apply p, PatchInspect p, ApplyState p ~ Tree)
                   => [Sealed2 (PatchInfoAnd rt p)] -> Set FileName -> [(PatchId, [PatchMod FileName])]
 patches2patchMods patches fns = snd $ mapAccumL go fns patches
   where
@@ -387,20 +348,22 @@ removePidSuffix pid2idx oldpids@(oldpid:_) (PatchIndex pids fidspans fpspans inf
 
 -- | update the patch index to the current state of the repository
 updatePatchIndexDisk
-    :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-    => Repository rt p wR wU wT -> IO ()
-updatePatchIndexDisk repo@(Repo repodir _ _ _) = do
+    :: (RepoPatch p, ApplyState p ~ Tree)
+    => Repository rt p wR wU wT
+    -> PatchSet rt p Origin wR
+    -> IO ()
+updatePatchIndexDisk repo patches = do
+    let repodir = repoLocation repo
     (_,_,pid2idx,pindex) <- loadPatchIndex repodir
     -- check that patch index is up to date
-    patches <- newset2FL `fmap` readRepo repo repodir
-    let pidsrepo = mapFL (makePatchID . info) patches
+    let flpatches = patchSet2FL patches
+    let pidsrepo = mapFL (makePatchID . info) flpatches
         (oldpids,_,len_common) = uncommon (reverse $ pids pindex) pidsrepo
         pindex' = removePidSuffix pid2idx oldpids pindex
         filenames = fpSpans2fileNames (fpspans pindex')
         cdir = repodir </> indexDir
     -- reread to prevent holding onto patches for too long
-    rawpatches <- newset2FL `fmap` readRepo repo repodir
-    let newpatches = drop len_common $ mapFL seal2 rawpatches
+    let newpatches = drop len_common $ mapFL seal2 flpatches
         newpmods = patches2patchMods newpatches filenames
     inv_hash <- getInventoryHash repodir
     storePatchIndex repodir cdir inv_hash (applyPatchMods newpmods pindex')
@@ -414,18 +377,21 @@ updatePatchIndexDisk repo@(Repo repodir _ _ _) = do
 
 -- | 'createPatchIndexFrom repo pmods' creates a patch index from the given
 --   patchmods.
-createPatchIndexFrom :: RepoPatch p => Repository rt p wR wU wT
+createPatchIndexFrom :: Repository rt p wR wU wT
                      -> [(PatchId, [PatchMod FileName])] -> IO ()
-createPatchIndexFrom (Repo repodir _ _ _) pmods = do
+createPatchIndexFrom repo pmods = do
     inv_hash <- getInventoryHash repodir
     storePatchIndex repodir cdir inv_hash (applyPatchMods pmods emptyPatchIndex)
-  where cdir = repodir </> indexDir
+  where repodir = repoLocation repo
+        cdir = repodir </> indexDir
+        emptyPatchIndex = PatchIndex [] M.empty M.empty M.empty
 
+getInventoryHash :: FilePath -> IO String
+getInventoryHash repodir = do
+  inv <- B.readFile (repodir </> darcsdir </> "hashed_inventory")
+  return $ sha256sum inv
 
--- ---------------------------------------------------------------------
--- Load/Store patch-Index
-
--- | load patch-index from disk
+-- | Load patch-index from disk along with some meta data.
 loadPatchIndex :: FilePath -> IO (Int8, String, Map PatchId Int, PatchIndex)
 loadPatchIndex repodir = do
   let pindex_dir = repodir </> indexDir
@@ -437,65 +403,66 @@ loadPatchIndex repodir = do
   fpspans <- loadFpMap (pindex_dir </> fpMapFile)
   return (v, inv_hash, pid2idx, PatchIndex pids fidspans fpspans infom)
 
--- | load patch-index,
--- | ensuring that whenever loaded, the patch-index
--- | can actually be read by the current version of darcs,
--- | and up to date.
-loadSafePatchIndex :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+-- | If patch-index is useful as it is now, read it. If not, create or update it, then read it.
+loadSafePatchIndex :: (RepoPatch p, ApplyState p ~ Tree)
                    => Repository rt p wR wU wT
-                   -> IO (Map PatchId Int, PatchIndex)
-loadSafePatchIndex repo@(Repo repodir _ _ _) = do
+                   -> PatchSet rt p Origin wR     -- ^ PatchSet of the repository, used if we need to create the patch-index.
+                   -> IO PatchIndex
+loadSafePatchIndex repo ps = do
+   let repodir = repoLocation repo
    can_use <- isPatchIndexInSync repo
-   (_,_,pid2idx,pi) <-
+   (_,_,_,pi) <-
      if can_use
        then loadPatchIndex repodir
-       else do createOrUpdatePatchIndexDisk repo
+       else do createOrUpdatePatchIndexDisk repo ps
                loadPatchIndex repodir
-   return (pid2idx, pi)
+   return pi
 
--- | check if patch-index exits for this repository
+-- | Read-only. Checks if patch-index exists for this repository
+--   it works by checking if:
+--
+--     1. @_darcs\/patch_index\/@ and its corresponding files are all present
+--     2. patch index version is the one handled by this version of Darcs
 doesPatchIndexExist :: FilePath -> IO Bool
 doesPatchIndexExist repodir = do
- filesArePresent <- fmap and $ mapM (doesFileExist . (pindex_dir </>))
+ filesArePresent <- and <$> mapM (doesFileExist . (pindex_dir </>))
                     [repoStateFile, pidsFile, touchMapFile, fidMapFile, fpMapFile]
  if filesArePresent
-  then do (v, _, _, _) <- loadPatchIndex repodir
+  then do v <- piVersion
           return (v == version)   -- consider PI only of on-disk format is the current one
   else return False
    where pindex_dir = repodir </> indexDir
+         piVersion = fst <$> loadRepoState (pindex_dir </> repoStateFile)
 
--- | check if noPatchIndex exists
+-- | Read-only. Checks if @_darcs\/noPatchIndex@ exists, that is, if patch-index is explicitely disabled.
 isPatchIndexDisabled :: FilePath -> IO Bool
 isPatchIndexDisabled repodir = doesFileExist (repodir </> darcsdir  </> noPatchIndex)
 
--- | create or update patch index
-createOrUpdatePatchIndexDisk :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                             => Repository rt p wR wU wT -> IO ()
-createOrUpdatePatchIndexDisk repo@(Repo repodir _ _ _)= do
+-- | Create or update patch index
+--
+--   1. if @_darcs\/no_patch_index@ exists, delete it
+--   2. if patch index exists, update it
+--   3. if not, create it from scratch
+createOrUpdatePatchIndexDisk :: (RepoPatch p, ApplyState p ~ Tree)
+                             => Repository rt p wR wU wT -> PatchSet rt p Origin wR -> IO ()
+createOrUpdatePatchIndexDisk repo ps = do
+   let repodir = repoLocation repo
    rmRecursive (repodir </> darcsdir </> noPatchIndex) `catch` \(_ :: IOError) -> return ()
    dpie <- doesPatchIndexExist repodir
    if dpie
-      then updatePatchIndexDisk repo
-      else createPatchIndexDisk repo
+      then updatePatchIndexDisk repo ps
+      else createPatchIndexDisk repo ps
 
--- | Checks whether a patch index can (and should) be created. If we are not in
--- an old-fashioned repo, and if we haven't been told not to, then we should
--- create a patch index if it doesn't already exist.
-canCreatePI :: (RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT
-            -> IO Bool
-canCreatePI (Repo repodir format _ _) =
-    (not . or) <$> sequence [ doesntHaveHashedInventory format
-                            , isPatchIndexDisabled repodir
-                            , doesPatchIndexExist repodir
-                            ]
-  where
-    doesntHaveHashedInventory = return . not . formatHas HashedInventory
-
--- | see if the default is to use patch index or not
--- | creates Patch index, if it does not exist, and noPatchIndex is not set
-canUsePatchIndex :: (RepoPatch p, ApplyState p ~ Tree)
-                 => Repository rt p wR wU wT -> IO Bool
-canUsePatchIndex (Repo repodir _ _ _) = do
+-- | Read-only. Checks the two following things:
+--
+--   1. 'doesPatchIndexExist'
+--   2. 'isPatchIndexDisabled'
+--
+-- Then only if it exists and it is not explicitely disabled, returns @True@, else returns @False@
+-- (or an error if it exists and is explicitely disabled at the same time).
+canUsePatchIndex :: Repository rt p wR wU wT -> IO Bool
+canUsePatchIndex repo = do
+     let repodir = repoLocation repo
      piExists <- doesPatchIndexExist repodir
      piDisabled <- isPatchIndexDisabled repodir
      case (piExists, piDisabled) of
@@ -504,18 +471,22 @@ canUsePatchIndex (Repo repodir _ _ _) = do
         (True, True) -> error "patch index exists, and patch index is disabled. run optimize enable-patch-index or disable-patch-index to rectify."
         (False, False) -> return False
 
-createPIWithInterrupt :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                      => Repository rt p wR wU wT -> IO ()
-createPIWithInterrupt repo@(Repo repodir _ _ _) = do
-            putStrLn "Creating a patch index, please wait. To stop press Ctrl-C"
-            (do
-              createPatchIndexDisk repo
-              putStrLn "Created patch index.") `catchInterrupt` (putStrLn "Patch Index Disabled" >> deletePatchIndex repodir)
+-- | Creates patch-index (ignoring whether it is explicitely disabled).
+--   If it is ctrl-c'ed, then aborts, delete patch-index and mark it as disabled.
+createPIWithInterrupt :: (RepoPatch p, ApplyState p ~ Tree)
+                      => Repository rt p wR wU wT -> PatchSet rt p Origin wR -> IO ()
+createPIWithInterrupt repo ps = do
+    let repodir = repoLocation repo
+    putStrLn "Creating a patch index, please wait. To stop press Ctrl-C"
+    (do
+      createPatchIndexDisk repo ps
+      putStrLn "Created patch index.") `catchInterrupt` (putStrLn "Patch Index Disabled" >> deletePatchIndex repodir)
 
--- | check if patch-index is in sync with repository
-isPatchIndexInSync :: (RepoPatch p, ApplyState p ~ Tree)
-                   => Repository rt p wR wU wT -> IO Bool
-isPatchIndexInSync (Repo repodir _ _ _) = do
+-- | Checks if patch-index exists and is in sync with repository (more precisely with @_darcs\/hashed_inventory@).
+--   That is, checks if patch-index can be used as it is now.
+isPatchIndexInSync :: Repository rt p wR wU wT -> IO Bool
+isPatchIndexInSync repo = do
+   let repodir = repoLocation repo
    dpie <- doesPatchIndexExist repodir
    if dpie
     then do
@@ -524,7 +495,7 @@ isPatchIndexInSync (Repo repodir _ _ _) = do
       return (inv_hash == inv_hash_pindex)
     else return False
 
--- | store patch-index on disk
+-- | Stores patch-index on disk.
 storePatchIndex :: FilePath -> FilePath -> String -> PatchIndex -> IO ()
 storePatchIndex repodir cdir inv_hash (PatchIndex pids fidspans fpspans infom) = do
   createDirectory cdir `catch` \(_ :: IOError) -> return ()
@@ -556,7 +527,7 @@ loadPatchIds = decodeFile
 storeFidMap :: FilePath -> FileIdSpans -> IO ()
 storeFidMap fp fidm =
   encodeFile fp $ M.map (map (\(FidSpan a b c) -> (a, b, toIdxM c))) fidm
- where toIdxM (Nothing) = zero
+ where toIdxM Nothing = zero
        toIdxM (Just pid) = pid
 
 loadFidMap :: FilePath -> IO FileIdSpans
@@ -567,7 +538,7 @@ loadFidMap fp = M.map (map (\(a,b,c) -> FidSpan a b (toPidM c))) <$> decodeFile
 storeFpMap :: FilePath -> FilePathSpans -> IO ()
 storeFpMap fp fidm =
   encodeFile fp $ M.map (map (\(FpSpan a b c) -> (a, b, toIdxM c))) fidm
- where toIdxM (Nothing) = zero
+ where toIdxM Nothing = zero
        toIdxM (Just pid) = pid
 
 loadFpMap :: FilePath -> IO FilePathSpans
@@ -575,9 +546,6 @@ loadFpMap fp = M.map (map (\(a,b,c) -> FpSpan a b (toPidM c))) <$> decodeFile fp
   where toPidM pid | pid == zero = Nothing
                    | otherwise   = Just pid
 
-zero :: PatchId
-zero = PID $ SHA1 0 0 0 0 0
-
 storeInfoMap :: FilePath -> InfoMap -> IO ()
 storeInfoMap fp infom =
   encodeFile fp $ M.map (\fi -> (isFile fi, touching fi)) infom
@@ -585,30 +553,17 @@ storeInfoMap fp infom =
 loadInfoMap :: FilePath -> IO InfoMap
 loadInfoMap fp = M.map (\(isF,pids) -> FileInfo isF pids) <$> decodeFile fp
 
--- | Base directory for the patch index
-indexDir :: String
+indexDir, repoStateFile, pidsFile, fidMapFile, fpMapFile,
+  touchMapFile, noPatchIndex :: String
 indexDir = darcsdir </> "patch_index"
-
-repoStateFile :: String
 repoStateFile = "repo_state"
-
-pidsFile :: String
 pidsFile = "patch_ids"
-
-fidMapFile :: String
 fidMapFile = "fid_map"
-
-fpMapFile :: String
 fpMapFile = "fp_map"
-
-touchMapFile :: String
 touchMapFile = "touch_map"
-
-noPatchIndex :: String
 noPatchIndex = "no_patch_index"
 
------------------------------------------------------------------------
--- Delete patch index
+-- | Deletes patch-index (@_darcs\/patch_index\/@ and its contents) and mark repository as disabled (creates @_darcs\/no_patch_index@).
 deletePatchIndex :: FilePath -> IO ()
 deletePatchIndex repodir = do
     exists <- doesDirectoryExist indexDir
@@ -618,9 +573,6 @@ deletePatchIndex repodir = do
     (openFile (repodir </> darcsdir </> noPatchIndex) WriteMode >>= hClose)
             `catch` \(e :: IOError) -> error $ "Error: Could not disable patch index\n" ++ show e
 
------------------------------------------------------------------------
--- Dump information in patch index
-
 dumpRepoState :: [PatchId] -> String
 dumpRepoState = unlines . map pid2string
 
@@ -648,34 +600,38 @@ fpSpans2filePaths fpSpans infom =
 fpSpans2filePaths' :: FileIdSpans -> [FilePath]
 fpSpans2filePaths' fidSpans = [fn2fp fp | (fp, _)  <- M.toList fidSpans]
 
-dumpPatchIndex :: FilePath -> IO ()
-dumpPatchIndex repodir = do
-  (_,inv_hash,_,PatchIndex pids fidspans fpspans infom) <- loadPatchIndex repodir
-  putStrLn $ "Inventory hash:" ++ inv_hash
-  putStrLn "================="
-  putStrLn "Repo state:"
-  putStrLn "==========="
-  putStrLn $ dumpRepoState pids
-  putStrLn "Fileid spans:"
-  putStrLn "============="
-  putStrLn $ dumpFileIdSpans fidspans
-  putStrLn "Filepath spans:"
-  putStrLn "=============="
-  putStrLn $ dumpFilePathSpans fpspans
-  putStrLn "Info Map:"
-  putStrLn "========="
-  putStrLn $ dumpTouchingMap infom
-  putStrLn "Files:"
-  putStrLn "=============="
-  putStrLn $ unlines $ fpSpans2filePaths fpspans infom
+-- | Checks if patch index can be created and build it with interrupt.
+attemptCreatePatchIndex
+  :: (RepoPatch p, ApplyState p ~ Tree)
+  => Repository rt p wR wU wT -> PatchSet rt p Origin wR -> IO ()
+attemptCreatePatchIndex repo ps = do
+  canCreate <- canCreatePI repo
+  when canCreate $ createPIWithInterrupt repo ps
 
------------------------------------------------------------------------
--- Filtering functions based on FilePaths
--- returns an RL in which the order of patches matters, for annotate to use
-getRelevantSubsequence :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd rt p)
-                       => Sealed ((RL a) wK) -> Repository rt p wR wU wR -> [FileName] -> IO (Sealed ((RL a) Origin))
-getRelevantSubsequence pxes repository fns = do
-   (_, pi@(PatchIndex _ _ _ infom)) <- loadSafePatchIndex repository
+-- | Checks whether a patch index can (and should) be created. If we are not in
+-- an old-fashioned repo, and if we haven't been told not to, then we should
+-- create a patch index if it doesn't already exist.
+canCreatePI :: Repository rt p wR wU wT -> IO Bool
+canCreatePI repo =
+    (not . or) <$> sequence [ doesntHaveHashedInventory (repoFormat repo)
+                            , isPatchIndexDisabled repodir
+                            , doesPatchIndexExist repodir
+                            ]
+  where
+    repodir = repoLocation repo
+    doesntHaveHashedInventory = return . not . formatHas HashedInventory
+
+-- | Returns an RL in which the order of patches matters. Useful for the @annotate@ command.
+--   If patch-index does not exist and is not explicitely disabled, silently create it.
+--   (Also, if it is out-of-sync, which should not happen, silently update it).
+getRelevantSubsequence :: (RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd rt p)
+                       => Sealed ((RL a) wK)          -- ^ Sequence of patches you want to filter
+                       -> Repository rt p wR wU wR    -- ^ The repository (to attempt loading patch-index from its path)
+                       -> PatchSet rt p Origin wR     -- ^ PatchSet of repository (in case we need to create patch-index)
+                       -> [FileName]                  -- ^ File(s) about which you want patches from given sequence
+                       -> IO (Sealed ((RL a) Origin)) -- ^ Filtered sequence of patches.
+getRelevantSubsequence pxes repository ps fns = do
+   pi@(PatchIndex _ _ _ infom) <- loadSafePatchIndex repository ps
    let fids = map (\fn -> evalState (lookupFid fn) pi) fns
        pidss = map ((\(FileInfo _ a) -> a).fromJust.(`M.lookup` infom)) fids
        pids = S.unions pidss
@@ -686,38 +642,58 @@ getRelevantSubsequence pxes repository fns = do
                   => FL a wX wY -> RL a wB wX -> S.Set Word32 -> RL a wP wQ
         keepElems NilFL acc _ = unsafeCoerceP acc
         keepElems (x:>:xs) acc pids
-          | (short $ makePatchID $ info x) `S.member` pids = keepElems xs (acc:<:x) pids
-          | otherwise                                      = keepElems (unsafeCoerceP xs) acc pids
-
--- | filter given patches so as to keep only the patches that modify the given files
-filterPatches :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, a ~ PatchInfoAnd rt p) => Repository rt p wR wU wT -> [FilePath] -> [Sealed2 a] -> IO [Sealed2 a]
-filterPatches repository fps ops = do
-   (_, pi@(PatchIndex _ _ _ infom)) <- loadSafePatchIndex repository
-   let fids = concatMap ((\fn -> evalState (lookupFids' fn) pi). fp2fn) fps
-       npids = S.unions $ map (touching.fromJust.(`M.lookup` infom)) fids
-   return $ filter (flip S.member npids . (\(Sealed2 (PIAP pin _)) -> short $ makePatchID pin)) ops
+          | short (makePatchID $ info x) `S.member` pids = keepElems xs (acc:<:x) pids
+          | otherwise                                    = keepElems (unsafeCoerceP xs) acc pids
 
 type PatchFilter rt p = [FilePath] -> [Sealed2 (PatchInfoAnd rt p)] -> IO [Sealed2 (PatchInfoAnd rt p)]
 
--- | If a patch index is available, filter given patches so as to keep only the patches that
--- modify the given files. If none is available, return the original input.
+-- | If a patch index is available, returns a filter that takes a list of files and returns
+--   a @PatchFilter@ that only keeps patches that modify the given list of files.
+--   If patch-index cannot be used, return the original input.
+--   If patch-index does not exist and is not explicitely disabled, silently create it.
+--   (Also, if it is out-of-sync, which should not happen, silently update it).
 maybeFilterPatches
-    :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-    => Repository rt p wR wU wT
-    -> PatchFilter rt p
-maybeFilterPatches repo fps ops = do
+    :: (RepoPatch p, ApplyState p ~ Tree)
+    => Repository rt p wR wU wT  -- ^ The repository
+    -> PatchSet rt p Origin wR   -- ^ PatchSet of patches of repository (in case patch-index needs to be created)
+    -> PatchFilter rt p          -- ^ PatchFilter ready to be used by SelectChanges.
+maybeFilterPatches repo ps fps ops = do
     usePI <- canUsePatchIndex repo
-    -- in theory we could change the type signature to make this function staged,
-    -- but it doesn't seem worth it.
-    if usePI then filterPatches repo fps ops else return ops
+    if usePI
+      then do
+        pi@(PatchIndex _ _ _ infom) <- loadSafePatchIndex repo ps
+        let fids = concatMap ((\fn -> evalState (lookupFids' fn) pi). fp2fn) fps
+            npids = S.unions $ map (touching.fromJust.(`M.lookup` infom)) fids
+        return $ filter (flip S.member npids . (\(Sealed2 (PIAP pin _)) -> short $ makePatchID pin)) ops
+      else return ops
 
+-- | Dump information in patch index. Patch-index should be checked to exist beforehand. Read-only.
+dumpPatchIndex :: FilePath -> IO ()
+dumpPatchIndex repodir = do
+  (_,inv_hash,_,PatchIndex pids fidspans fpspans infom) <- loadPatchIndex repodir
+  putStrLn $ unlines $
+    [ "Inventory hash:" ++ inv_hash
+    , "================="
+    , "Repo state:"
+    , "==========="
+    , dumpRepoState pids
+    , "Fileid spans:"
+    , "============="
+    , dumpFileIdSpans fidspans
+    , "Filepath spans:"
+    , "=============="
+    , dumpFilePathSpans fpspans
+    , "Info Map:"
+    , "========="
+    , dumpTouchingMap infom
+    , "Files:"
+    , "=============="
+    ] ++ fpSpans2filePaths fpspans infom
 
------------------------------------------------------------------------
--- Test patch index
-
-piTest :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT -> IO ()
-piTest repository = do
-   (_, PatchIndex rpids fidspans fpspans infom) <- loadSafePatchIndex repository
+-- | Read-only sanity check on patch-index. Patch-index should be checked to exist beforehand. It may not be in sync with repository.
+piTest :: FilePath -> IO ()
+piTest repodir = do
+   (_,_,_,PatchIndex rpids fidspans fpspans infom) <- loadPatchIndex repodir
    let pids = reverse rpids
 
    -- test fidspans
@@ -760,9 +736,4 @@ piTest repository = do
           isInOrder [] _ = True
           isInOrder _ [] = False
 
--- | Check if patch index can be created and build it with interrupt.
-attemptCreatePatchIndex
-  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wT -> IO ()
-attemptCreatePatchIndex repo = do
-  canCreate <- canCreatePI repo
-  when canCreate $ createPIWithInterrupt repo
+
