@@ -1,7 +1,6 @@
 --  Copyright (C) 2009-2012 Ganesh Sittampalam
 --
 --  BSD3
-{-# LANGUAGE CPP #-}
 module Darcs.Repository.Rebase
     ( RebaseJobFlags(..)
     , withManualRebaseUpdate
@@ -52,7 +51,7 @@ import Darcs.Repository.Format
     , removeFromFormat
     , writeRepoFormat
     )
-import Darcs.Repository.Internal
+import Darcs.Repository.Hashed
     ( tentativelyAddPatch
     , tentativelyAddPatch_
     , tentativelyAddPatches_
@@ -64,19 +63,17 @@ import Darcs.Repository.Internal
     , readRepo
     , UpdatePristine(..)
     )
-import Darcs.Repository.InternalTypes ( Repository(..) )
+import Darcs.Repository.InternalTypes ( Repository, repoFormat, repoLocation )
 
 import qualified Darcs.Util.Diff as D ( DiffAlgorithm(MyersDiff) )
+import Darcs.Util.Printer ( ePutDocLn, text )
 import Darcs.Util.Progress ( debugMessage )
-
 import Darcs.Util.Tree ( Tree )
 
 import Control.Exception ( finally )
 
 import System.FilePath.Posix ( (</>) )
 
-#include "impossible.h"
-
 -- | Some common flags that are needed to run rebase jobs.
 -- Normally flags are captured directly by the implementation of the specific
 -- job's function, but the rebase infrastructure needs to do work on the repository
@@ -98,7 +95,7 @@ withManualRebaseUpdate
    -> IO (Repository rt p wR wU wT2, x)
 withManualRebaseUpdate (RebaseJobFlags compr verb uw) r subFunc
  | SRepoType SIsRebase <- singletonRepoType :: SRepoType rt
- = do patches <- readTentativeRepo r
+ = do patches <- readTentativeRepo r (repoLocation r)
       let go :: PatchSet rt p wS wT1 -> IO (Repository rt p wR wU wT2, x)
           go (PatchSet _ NilRL) = bug "trying to recontext rebase without rebase patch at head (tag)"
           go (PatchSet _ (_ :<: q)) =
@@ -151,9 +148,8 @@ checkSuspendedStatus :: (RepoPatch p, ApplyState p ~ Tree)
                      => Repository ('RepoType 'IsRebase) p wR wU wR
                      -> RebaseJobFlags
                      -> IO ()
-checkSuspendedStatus repo@(Repo _ rf _ _) flags@(RebaseJobFlags compr _verb uw) = do
-    allpatches <- readRepo repo
-    (_, Sealed2 ps) <- return $ takeAnyRebase allpatches
+checkSuspendedStatus repo flags@(RebaseJobFlags compr _verb uw) = do
+    (_, Sealed2 ps) <- takeAnyRebase <$> readRepo repo
     case countToEdit ps of
          0 -> do
                debugMessage "Removing the rebase patch file..."
@@ -168,9 +164,11 @@ checkSuspendedStatus repo@(Repo _ rf _ _) flags@(RebaseJobFlags compr _verb uw)
                (rebase, _, _) <- takeHeadRebase <$> readRepo repo'
                repo'' <- tentativelyRemovePatches repo' compr uw (rebase :>: NilFL)
                finalizeRepositoryChanges repo'' uw compr
-               writeRepoFormat (removeFromFormat RebaseInProgress rf) (darcsdir </> "format")
+               writeRepoFormat
+                  (removeFromFormat RebaseInProgress (repoFormat repo))
+                  (darcsdir </> "format")
                putStrLn "Rebase finished!"
-         n -> putStrLn $ "Rebase in progress: " ++ show n ++ " suspended patches"
+         n -> ePutDocLn $ text $ "Rebase in progress: " ++ show n ++ " suspended patches"
 
 moveRebaseToEnd :: (RepoPatch p, ApplyState p ~ Tree)
                 => Repository ('RepoType 'IsRebase) p wR wU wR
@@ -191,14 +189,13 @@ moveRebaseToEnd repo (RebaseJobFlags compr verb uw) = do
             finalizeRepositoryChanges repo'''' uw compr
             return repo''''
 
-displaySuspendedStatus :: (RepoPatch p, ApplyState p ~ Tree) => Repository ('RepoType 'IsRebase) p wR wU wR -> IO ()
+displaySuspendedStatus :: RepoPatch p => Repository ('RepoType 'IsRebase) p wR wU wR -> IO ()
 displaySuspendedStatus repo = do
-    allpatches <- readRepo repo
-    (_, Sealed2 ps) <- return $ takeAnyRebase allpatches
-    putStrLn $ "Rebase in progress: " ++ show (countToEdit ps) ++ " suspended patches"
+    (_, Sealed2 ps) <- takeAnyRebase <$> readRepo repo
+    ePutDocLn $ text $ "Rebase in progress: " ++ show (countToEdit ps) ++ " suspended patches"
 
 maybeDisplaySuspendedStatus
-  :: (RepoPatch p, ApplyState p ~ Tree)
+  :: RepoPatch p
   => SRebaseType rebaseType -> Repository ('RepoType rebaseType) p wR wU wR -> IO ()
 maybeDisplaySuspendedStatus SIsRebase repo = displaySuspendedStatus repo
 maybeDisplaySuspendedStatus SNoRebase _    = return ()
@@ -207,7 +204,9 @@ startRebaseIfNecessary :: (RepoPatch p, ApplyState p ~ Tree)
                        => Repository ('RepoType 'IsRebase) p wR wU wT
                        -> RebaseJobFlags
                        -> IO (Repository ('RepoType 'IsRebase) p wR wU wT)
-startRebaseIfNecessary repo@(Repo _ rf _ _) (RebaseJobFlags compr verb uw) =
+startRebaseIfNecessary repo (RebaseJobFlags compr verb uw) =
+  let rf = repoFormat repo
+  in
     if formatHas RebaseInProgress rf
     then return repo
     else do -- TODO this isn't under the repo lock, and it should be
