@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP, PatternGuards #-}
-
 module Darcs.Repository.Repair ( replayRepository, checkIndex,
                                  replayRepositoryInTemp,
                                  RepositoryConsistency(..) )
@@ -26,8 +24,8 @@ import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed ( Sealed2(..), Sealed(..), unFreeLeft )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Repair ( Repair(applyAndTryToFix) )
-import Darcs.Patch.Info ( showPatchInfoUI )
-import Darcs.Patch.Set ( Origin, PatchSet(..), newset2FL, newset2RL )
+import Darcs.Patch.Info ( displayPatchInfo )
+import Darcs.Patch.Set ( Origin, PatchSet(..), patchSet2FL, patchSet2RL )
 import Darcs.Patch ( RepoPatch, IsRepoType, PrimOf, isInconsistent )
 
 import Darcs.Repository.Flags
@@ -36,10 +34,10 @@ import Darcs.Repository.Format ( identifyRepoFormat,
                                  RepoProperty ( HashedInventory ), formatHas )
 import Darcs.Repository.Cache ( HashedDir( HashedPristineDir ) )
 import Darcs.Repository.HashedIO ( cleanHashdir )
-import Darcs.Repository.HashedRepo ( readHashedPristineRoot, writeAndReadPatch )
-import Darcs.Repository.InternalTypes ( Repository(..), extractCache )
+import Darcs.Repository.Hashed ( readHashedPristineRoot, writeAndReadPatch )
+import Darcs.Repository.InternalTypes ( Repository, repoCache, repoLocation )
 import Darcs.Repository.Prefs ( filetypeFunction )
-import Darcs.Repository.Internal ( readRepo )
+import Darcs.Repository.Hashed ( readRepo )
 import Darcs.Repository.State
     ( readRecorded
     , readIndex
@@ -52,7 +50,7 @@ import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Exception ( catchall )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.Lock( rmRecursive, withTempDir )
-import Darcs.Util.Printer ( Doc, putDocLn, text, RenderMode(..) )
+import Darcs.Util.Printer ( Doc, putDocLn, text )
 import Darcs.Util.Printer.Color ( showDoc )
 
 import Darcs.Util.Hash( Hash(NoHash), encodeBase16 )
@@ -62,9 +60,7 @@ import Darcs.Util.Tree.Hashed( darcsUpdateHashes, hashedTreeIO )
 import Darcs.Util.Tree.Plain( readPlainTree )
 import Darcs.Util.Index( updateIndex )
 
-import qualified Data.ByteString.Char8 as BS
-
-#include "impossible.h"
+import qualified Data.ByteString.Char8 as BC
 
 replaceInFL :: FL (PatchInfoAnd rt a) wX wY
             -> [Sealed2 (WPatchInfo :||: PatchInfoAnd rt a)]
@@ -79,12 +75,12 @@ applyAndFix :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~
             => Repository rt p wR wU wT -> Compression -> FL (PatchInfoAnd rt p) Origin wR
             -> TreeIO (FL (PatchInfoAnd rt p) Origin wR, Bool)
 applyAndFix _ _ NilFL = return (NilFL, True)
-applyAndFix r@(Repo r' _ _ c) compr psin =
+applyAndFix r compr psin =
     do liftIO $ beginTedious k
        liftIO $ tediousSize k $ lengthFL psin
        (repaired, ok) <- aaf psin
        liftIO $ endTedious k
-       orig <- liftIO $ newset2FL `fmap` readRepo r
+       orig <- liftIO $ patchSet2FL `fmap` readRepo r
        return (replaceInFL orig repaired, ok)
     where k = "Replaying patch"
           aaf :: FL (PatchInfoAnd rt p) wW wZ -> TreeIO ([Sealed2 (WPatchInfo :||: PatchInfoAnd rt p)], Bool)
@@ -95,28 +91,30 @@ applyAndFix r@(Repo r' _ _ c) compr psin =
               Just err -> liftIO $ putDocLn err
               Nothing -> return ()
             let !winfp = winfo p -- assure that 'p' can be garbage collected.
-            liftIO $ finishedOneIO k $ showDoc Encode $ showPatchInfoUI $ unWPatchInfo winfp
+            liftIO $ finishedOneIO k $ showDoc $ displayPatchInfo $ unWPatchInfo winfp
             (ps', restok) <- aaf ps
             case mp' of
               Nothing -> return (ps', restok)
-              Just (e,pp) -> liftIO $ do putStrLn e
-                                         p' <- withCurrentDirectory r' $ writeAndReadPatch c compr pp
-                                         return (Sealed2 (winfp :||: p'):ps', False)
+              Just (e,pp) -> liftIO $ do
+                putStrLn e
+                p' <- withCurrentDirectory (repoLocation r) $
+                  writeAndReadPatch (repoCache r) compr pp
+                return (Sealed2 (winfp :||: p'):ps', False)
 
 data RepositoryConsistency rt p wX =
     RepositoryConsistent
   | BrokenPristine (Tree IO)
   | BrokenPatches (Tree IO) (PatchSet rt p Origin wX)
 
-checkUniqueness :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+checkUniqueness :: (IsRepoType rt, RepoPatch p)
                 => (Doc -> IO ()) -> (Doc -> IO ()) -> Repository rt p wR wU wT -> IO ()
 checkUniqueness putVerbose putInfo repository =
     do putVerbose $ text "Checking that patch names are unique..."
        r <- readRepo repository
-       case hasDuplicate $ mapRL info $ newset2RL r of
+       case hasDuplicate $ mapRL info $ patchSet2RL r of
          Nothing -> return ()
          Just pinf -> do putInfo $ text "Error! Duplicate patch name:"
-                         putInfo $ showPatchInfoUI pinf
+                         putInfo $ displayPatchInfo pinf
                          fail "Duplicate patches found."
 
 hasDuplicate :: Ord a => [a] -> Maybe a
@@ -139,7 +137,7 @@ replayRepository' dflag whereToReplay' repo compr verbosity = do
   putVerbose $ text "Applying patches..."
   patches <- readRepo repo
   debugMessage "Fixing any broken patches..."
-  let psin = newset2FL patches
+  let psin = patchSet2FL patches
       repair = applyAndFix repo compr psin
 
   ((ps, patches_ok), newpris) <- hashedTreeIO repair emptyTree whereToReplay
@@ -161,7 +159,7 @@ replayRepository' dflag whereToReplay' repo compr verbosity = do
 
 cleanupRepositoryReplay :: Repository rt p wR wU wT -> IO ()
 cleanupRepositoryReplay r = do
-  let c = extractCache r
+  let c = repoCache r
   rf <- identifyRepoFormat "."
   unless (formatHas HashedInventory rf) $
          rmRecursive $ darcsdir ++ "/pristine.hashed"
@@ -207,8 +205,8 @@ checkIndex repo quiet = do
 
       format paths = unlines $ map (("  " ++) . anchorPath "") paths
       mismatches_disp = unlines [ anchorPath "" p ++
-                                    "\n    index: " ++ BS.unpack (encodeBase16 h1) ++
-                                    "\n  working: " ++ BS.unpack (encodeBase16 h2)
+                                    "\n    index: " ++ BC.unpack (encodeBase16 h1) ++
+                                    "\n  working: " ++ BC.unpack (encodeBase16 h2)
                                   | (p, h1, h2) <- mismatches ]
   unless (quiet || null index_extra) $
          putStrLn $ "Extra items in index!\n" ++ format index_extra
