@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 -- Copyright (C) 2009 Petr Rockai
 --           (C) 2012 José Neder
 --
@@ -49,7 +50,7 @@ import Data.List ( sortBy, union, delete )
 import Text.Regex( matchRegex )
 
 import System.Directory( removeFile, doesFileExist, doesDirectoryExist, renameFile )
-import System.FilePath ( (</>) )
+import System.FilePath ( (</>), (<.>) )
 import qualified Data.ByteString as B
     ( ByteString, readFile, drop, writeFile, empty, concat )
 import qualified Data.ByteString.Char8 as BC
@@ -423,7 +424,14 @@ checkIndex = do
   formatValid <- if exists
                      then I.indexFormatValid index_file
                      else return True
-  when (exists && not formatValid) $ removeFile index_file
+  when (exists && not formatValid) $ do
+-- TODO this conditional logic (rename or delete) is mirrored in
+-- Darcs.Util.Index.updateIndexFrom and should be refactored
+#if mingw32_HOST_OS
+    renameFile index_file (index_file <.> "old")
+#else
+    removeFile index_file
+#endif
   return (invalid, exists, formatValid)
 
 -- |Remove any patches (+dependencies) from a sequence that
