@@ -15,12 +15,13 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Commands
     ( CommandControl ( CommandData, HiddenCommand, GroupName )
     , DarcsCommand ( .. )
     , WrappedCommand(..)
     , wrappedCommandName
+    , wrappedCommandDescription
     , commandAlias
     , commandStub
     , commandOptions
@@ -28,12 +29,7 @@ module Darcs.UI.Commands
     , withStdOpts
     , disambiguateCommands
     , CommandArgs(..)
-    , getCommandHelp
-    , getCommandMiniHelp
     , getSubcommands
-    , usage
-    , usageHelper
-    , subusage
     , extractCommands
     , extractAllCommands
     , normalCommand
@@ -46,7 +42,6 @@ module Darcs.UI.Commands
     , putWarning
     , putVerboseWarning
     , abortRun
-    , printDryRunMessageAndExit
     , setEnvDarcsPatches
     , setEnvDarcsFiles
     , defaultRepo
@@ -62,50 +57,41 @@ import Darcs.Prelude
 import Prelude hiding ( (^) )
 import Control.Monad ( when, unless )
 import Data.List ( sort, isPrefixOf )
-import Data.Maybe ( catMaybes )
 import Darcs.Util.Tree ( Tree )
 import System.Console.GetOpt ( OptDescr )
-import System.Exit ( exitSuccess )
 import System.IO ( stderr )
-#ifndef WIN32
-import System.Posix.Env ( setEnv )
-
+import System.IO.Error ( catchIOError )
+import System.Environment ( setEnv )
 import Darcs.Patch ( listTouchedFiles )
 import qualified Darcs.Patch ( summary )
-#endif
-import Darcs.Patch ( RepoPatch, xmlSummary, Patchy )
+import Darcs.Patch ( RepoPatch )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Info ( toXml )
 import Darcs.Patch.Inspect ( PatchInspect )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, hopefullyM )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info )
 import Darcs.Patch.Witnesses.Ordered ( FL, mapFL )
 
 import qualified Darcs.Repository as R ( amInHashedRepository, amInRepository
                                        , amNotInRepository, findRepository )
 import Darcs.Repository.Prefs ( defaultrepo )
 
-import Darcs.UI.Options ( DarcsOption, DarcsOptDescr, (^), optDescr, odesc, parseFlags )
+import Darcs.UI.Options ( DarcsOption, DarcsOptDescr, (^), optDescr, odesc, parseFlags, (?) )
 import Darcs.UI.Options.All
-    ( StdCmdAction, stdCmdActions, anyVerbosity, UseCache, useCache, hooks
-    , Verbosity(..), verbosity, Summary(..), DryRun(..), dryRun, XmlOutput(..) )
+    ( StdCmdAction, stdCmdActions, anyVerbosity, UseCache, useCache, HooksConfig, hooks
+    , Verbosity(..), DryRun(..), dryRun
+    )
 
-import Darcs.UI.Flags ( remoteRepos, workRepo, DarcsFlag )
-import Darcs.UI.PrintPatch ( showFriendly )
-import Darcs.UI.Usage ( usageInfo )
+import Darcs.UI.Flags ( DarcsFlag, remoteRepos, workRepo, quiet, verbose )
 
+import Darcs.Util.ByteString ( decodeLocale, packStringToUTF8 )
 import Darcs.Util.Path ( AbsolutePath )
-import Darcs.Util.Printer ( Doc, putDocLn, hPutDocLn, text, (<+>), errorDoc
-                          , vsep, insertBeforeLastline, prefix, ($$), vcat
-#ifndef WIN32
-                          , renderString
-#endif
-                          , RenderMode(..)
-                          )
-#ifndef WIN32
-import Darcs.Util.Progress ( beginTedious, endTedious, tediousSize
-                           , finishedOneIO )
-#endif
-import Darcs.Util.Text ( chompTrailingNewline )
+import Darcs.Util.Printer
+    ( Doc, text, (<+>), ($$), vcat
+    , putDocLnWith, hPutDocLn, errorDoc, renderString
+    )
+import Darcs.Util.Printer.Color ( fancyPrinters )
+import Darcs.Util.Progress
+    ( debugMessage, beginTedious, endTedious, tediousSize, finishedOneIO )
 
 extractCommands :: [CommandControl] -> [WrappedCommand]
 extractCommands ccl = [ cmd | CommandData cmd <- ccl ]
@@ -158,7 +144,8 @@ data DarcsCommand parsedFlags =
                               (AbsolutePath, AbsolutePath)
                            -> parsedFlags -> [String] -> IO ()
           , commandPrereq :: [DarcsFlag] -> IO (Either String ())
-          , commandGetArgPossibilities :: IO [String]
+          , commandCompleteArgs :: (AbsolutePath, AbsolutePath)
+                                -> [DarcsFlag] -> [String] -> IO [String]
           , commandArgdefaults :: [DarcsFlag] -> AbsolutePath -> [String]
                                -> IO [String]
           , commandBasicOptions :: [DarcsOptDescr DarcsFlag]
@@ -177,7 +164,7 @@ data DarcsCommand parsedFlags =
           }
 
 withStdOpts :: DarcsOption (Maybe StdCmdAction -> Bool -> Bool -> Verbosity -> Bool -> b) c
-            -> DarcsOption (UseCache -> Maybe String -> Bool -> Maybe String -> Bool -> a) b
+            -> DarcsOption (UseCache -> HooksConfig -> a) b
             -> DarcsOption a c
 withStdOpts basicOpts advancedOpts =
   basicOpts ^ stdCmdActions ^ anyVerbosity ^ advancedOpts ^ useCache ^ hooks
@@ -199,7 +186,7 @@ nodefaults :: [DarcsFlag] -> AbsolutePath -> [String] -> IO [String]
 nodefaults _ _ = return
 
 getSubcommands :: DarcsCommand pf -> [CommandControl]
-getSubcommands c@(SuperCommand {}) = commandSubCommands c
+getSubcommands c@(SuperCommand {}) = commandGroup "Subcommands:" : commandSubCommands c
 getSubcommands _ = []
 
 commandAlias :: String -> Maybe (DarcsCommand pf) -> DarcsCommand pf -> DarcsCommand pf
@@ -222,120 +209,10 @@ commandStub n h d c = c { commandName = n
                         , commandCommand = \_ _ _ -> putStr h
                         }
 
-usage :: [CommandControl] -> String
-usage cs = unlines
-    [ "Usage: darcs COMMAND ..."
-    , ""
-    , "Commands:"
-    , usageHelper cs
-    , "Use 'darcs COMMAND --help' for help on a single command."
-    , "Use 'darcs --version' to see the darcs version number."
-    , "Use 'darcs --exact-version' to see a detailed darcs version."
-    , "Use 'darcs help patterns' for help on patch matching."
-    , "Use 'darcs help environment' for help on environment variables."
-    , "Use 'darcs help manpage' to display help in the manpage format."
-    , "Use 'darcs help markdown' to display help in the markdown format."
-    , ""
-    , "Check bug reports at http://bugs.darcs.net/"
-    ]
-
-subusage :: DarcsCommand pf -> String
-subusage super = usageInfo header (odesc stdCmdActions) ++ superHelp
-  where
-    header = unlines [ unwords [ "Usage:"
-                               , commandProgramName super
-                               , commandName super
-                               , "SUBCOMMAND ..."
-                               ]
-                     , ""
-                     , commandDescription super
-                     , ""
-                     , "Subcommands:"
-                     , usageHelper (getSubcommands super)
-                     , "Options:"
-                     ]
-    superHelp = '\n' : commandHelp super
-
-usageHelper :: [CommandControl] -> String
-usageHelper xs = usageHelper' (maximum $ 15 : (catMaybes $ map f xs)) xs
-   where
-         -- returns length of necessary tabbing this command
-         f (CommandData c) = Just ((+2) . length . wrappedCommandName $ c)
-         f _ = Nothing
-
-usageHelper' :: Int -> [CommandControl] -> String
-usageHelper' _ [] = ""
-usageHelper' x (HiddenCommand _ : cs) = usageHelper' x cs
-usageHelper' x (CommandData c : cs) = "  " ++ padSpaces (wrappedCommandName c) x
-                                        ++ chompTrailingNewline (wrappedCommandDescription c)
-                                        ++ "\n" ++ usageHelper' x cs
-usageHelper' x (GroupName n : cs) = "\n" ++ n ++ "\n" ++ usageHelper' x cs
-
-padSpaces :: String -> Int -> String
-padSpaces s n = s ++ replicate (n - length s) ' '
-
 superName :: Maybe (DarcsCommand pf) -> String
 superName Nothing  = ""
 superName (Just x) = commandName x ++ " "
 
-getCommandMiniHelp :: Maybe (DarcsCommand pf1) -> DarcsCommand pf2 -> String
-getCommandMiniHelp msuper cmd = unlines
-    [ getCommandHelpCore msuper cmd
-    , ""
-    , unwords [ "See"
-              , commandProgramName cmd
-              , "help"
-              , maybe "" ((++ " ") . commandName) msuper ++ commandName cmd
-              , "for details."
-              ]
-    ]
-
-getCommandHelp :: Maybe (DarcsCommand pf1) -> DarcsCommand pf2 -> String
-getCommandHelp msuper cmd = basicHelp ++ advancedHelp ++ cmdHelp
-  where
-    basicHelp = unlines (reverse basicR)
-
-    advancedHelp = if null advanced
-                       then ""
-                       else '\n' :
-                            unlines ("Advanced options:" : reverse advancedR)
-
-    cmdHelp =  '\n' : commandHelp cmd
-
-    -- we could just call usageInfo twice, but then the advanced
-    -- options might not line up with the basic ones (no short switches)
-    (advancedR, basicR) = splitAt (length advanced) . reverse . lines $
-                              combinedUsage
-
-    combinedUsage = let header = getCommandHelpCore msuper cmd ++ subcommands
-                                     ++ "\n\nOptions:"
-                    in usageInfo header (basic ++ advanced)
-
-    (basic, advanced) = commandAlloptions cmd
-
-    subcommands = case msuper of
-                      Nothing -> case getSubcommands cmd of
-                                     [] -> []
-                                     s  -> "\n\nSubcommands:\n"
-                                           ++ usageHelper s
-                      -- we don't want to list subcommands if we're already
-                      -- specifying them
-                      Just _  -> ""
-
-getCommandHelpCore :: Maybe (DarcsCommand pf1) -> DarcsCommand pf2 -> String
-getCommandHelpCore msuper cmd =
-    unwords [ "Usage:"
-            , commandProgramName cmd
-            , superName msuper ++ commandName cmd
-            , "[OPTION]..."
-            , unwords args_help
-            ]
-    ++ "\n" ++ commandDescription cmd
-  where
-    args_help = case cmd of
-                    (DarcsCommand {}) -> commandExtraArgHelp cmd
-                    _ -> []
-
 data CommandArgs where
     CommandOnly :: DarcsCommand parsedFlags -> CommandArgs
     SuperCommandOnly :: DarcsCommand parsedFlags -> CommandArgs
@@ -366,89 +243,36 @@ extract cmd cs = case potentials of
     potentials = [c | c <- extractCommands cs, cmd `isPrefixOf` wrappedCommandName c]
                  ++ [h | h <- extractHiddenCommands cs, cmd == wrappedCommandName h]
 
-amVerbose :: [DarcsFlag] -> Bool
-amVerbose = (== Verbose) . parseFlags verbosity
-
-amQuiet :: [DarcsFlag] -> Bool
-amQuiet = (== Quiet) . parseFlags verbosity
-
 putVerbose :: [DarcsFlag] -> Doc -> IO ()
-putVerbose flags = when (amVerbose flags) . putDocLn
+putVerbose flags = when (verbose flags) . putDocLnWith fancyPrinters
 
 putInfo :: [DarcsFlag] -> Doc -> IO ()
-putInfo flags = unless (amQuiet flags) . putDocLn
+putInfo flags = unless (quiet flags) . putDocLnWith fancyPrinters
 
 putWarning :: [DarcsFlag] -> Doc -> IO ()
-putWarning flags = unless (amQuiet flags) . hPutDocLn Encode stderr
+putWarning flags = unless (quiet flags) . hPutDocLn stderr
 
 putVerboseWarning :: [DarcsFlag] -> Doc -> IO ()
-putVerboseWarning flags = when (amVerbose flags) . hPutDocLn Encode stderr
+putVerboseWarning flags = when (verbose flags) . hPutDocLn stderr
 
 abortRun :: [DarcsFlag] -> Doc -> IO ()
 abortRun flags msg = if parseFlags dryRun flags == YesDryRun
-                        then putInfo flags $ text "NOTE:" <+> msg
+                        then putInfo flags $ "NOTE:" <+> msg
                         else errorDoc msg
 
--- | @'printDryRunMessageAndExit' action flags patches@ prints a string
--- representing the action that would be taken if the @--dry-run@ option had
--- not been passed to darcs. Then darcs exits successfully.  @action@ is the
--- name of the action being taken, like @\"push\"@ @flags@ is the list of flags
--- which were sent to darcs @patches@ is the sequence of patches which would be
--- touched by @action@.
-printDryRunMessageAndExit :: (RepoPatch p, ApplyState p ~ Tree)
-                          => String
-                          -> Verbosity -> Summary -> DryRun -> XmlOutput
-                          -> Bool -- interactive
-                          -> FL (PatchInfoAnd rt p) wX wY
-                          -> IO ()
-printDryRunMessageAndExit action v s d x interactive patches = do
-    when (d == YesDryRun) $ do
-        putInfoX . text $ unwords [ "Would"
-                                  , action
-                                  , "the following changes:"
-                                  ]
-        putDocLn put_mode
-        putInfoX $ text ""
-        putInfoX $ text "Making no changes: this is a dry run."
-        exitSuccess
-    when (not interactive && s == YesSummary) $ do
-        putInfoX . text $ unwords [ "Will"
-                                  , action
-                                  , "the following changes:"
-                                  ]
-        putDocLn put_mode
-  where
-    put_mode = if x == YesXml
-                   then text "<patches>" $$
-                        vcat (mapFL (indent . xml_info s) patches) $$
-                        text "</patches>"
-                   else vsep $ mapFL (showFriendly v s) patches
-
-    putInfoX = if x == YesXml then const (return ()) else putDocLn
-
-    xml_info YesSummary = xml_with_summary
-    xml_info NoSummary  = toXml . info
-
-    xml_with_summary hp
-        | Just p <- hopefullyM hp = insertBeforeLastline (toXml $ info hp)
-                                        (indent $ xmlSummary p)
-    xml_with_summary hp = toXml (info hp)
-
-    indent = prefix "    "
-
 -- | Set the DARCS_PATCHES and DARCS_PATCHES_XML environment variables with
 -- info about the given patches, for use in post-hooks.
 setEnvDarcsPatches :: (RepoPatch p, ApplyState p ~ Tree)
                    => FL (PatchInfoAnd rt p) wX wY -> IO ()
-#ifndef WIN32
 setEnvDarcsPatches ps = do
     let k = "Defining set of chosen patches"
+    debugMessage $ unlines ("setEnvDarcsPatches:" : listTouchedFiles ps)
     beginTedious k
     tediousSize k 3
     finishedOneIO k "DARCS_PATCHES"
-    setEnvCautiously "DARCS_PATCHES" (renderString Encode $ Darcs.Patch.summary ps)
+    setEnvCautiously "DARCS_PATCHES" (renderString $ Darcs.Patch.summary ps)
     finishedOneIO k "DARCS_PATCHES_XML"
-    setEnvCautiously "DARCS_PATCHES_XML" . renderString Encode $
+    setEnvCautiously "DARCS_PATCHES_XML" . renderString $
         text "<patches>" $$
         vcat (mapFL (toXml . info) ps) $$
         text "</patches>"
@@ -456,43 +280,37 @@ setEnvDarcsPatches ps = do
     setEnvCautiously "DARCS_FILES" $ unlines (listTouchedFiles ps)
     endTedious k
 
+-- | Set the DARCS_FILES environment variable to the files touched by the
+-- given patch, one per line, for use in post-hooks.
+setEnvDarcsFiles :: (PatchInspect p) => p wX wY -> IO ()
+setEnvDarcsFiles ps =
+    setEnvCautiously "DARCS_FILES" $ unlines (listTouchedFiles ps)
+
 -- | Set some environment variable to the given value, unless said value is
 -- longer than 10K characters, in which case do nothing.
 setEnvCautiously :: String -> String -> IO ()
 setEnvCautiously e v
     | toobig (10 * 1024) v = return ()
-    | otherwise = setEnv e v True
+    | otherwise =
+        setEnv e v `catchIOError` (\_ -> setEnv e (decodeLocale (packStringToUTF8 v)))
   where
     -- note: not using (length v) because we want to be more lazy than that
     toobig :: Int -> [a] -> Bool
     toobig 0 _ = True
     toobig _ [] = False
     toobig n (_ : xs) = toobig (n - 1) xs
-#else
-setEnvDarcsPatches _ = return ()
-#endif
-
--- | Set the DARCS_FILES environment variable to the files touched by the
--- given patch, one per line, for use in post-hooks.
-setEnvDarcsFiles :: (PatchInspect p, Patchy p) => p wX wY -> IO ()
-#ifndef WIN32
-setEnvDarcsFiles ps =
-    setEnvCautiously "DARCS_FILES" $ unlines (listTouchedFiles ps)
-#else
-setEnvDarcsFiles _ = return ()
-#endif
 
 defaultRepo :: [DarcsFlag] -> AbsolutePath -> [String] -> IO [String]
-defaultRepo fs = defaultrepo (remoteRepos fs)
+defaultRepo fs = defaultrepo (remoteRepos ? fs)
 
 amInHashedRepository :: [DarcsFlag] -> IO (Either String ())
-amInHashedRepository fs = R.amInHashedRepository (workRepo fs)
+amInHashedRepository fs = R.amInHashedRepository (workRepo ? fs)
 
 amInRepository :: [DarcsFlag] -> IO (Either String ())
-amInRepository fs = R.amInRepository (workRepo fs)
+amInRepository fs = R.amInRepository (workRepo ? fs)
 
 amNotInRepository :: [DarcsFlag] -> IO (Either String ())
-amNotInRepository fs = R.amNotInRepository (workRepo fs)
+amNotInRepository fs = R.amNotInRepository (workRepo ? fs)
 
 findRepository :: [DarcsFlag] -> IO (Either String ())
-findRepository fs = R.findRepository (workRepo fs)
+findRepository fs = R.findRepository (workRepo ? fs)
