@@ -23,30 +23,23 @@
 -- Stability   : experimental
 -- Portability : portable
 
-{-# LANGUAGE CPP #-}
-module Darcs.UI.Commands.Add
-    (
-      add
-    , expandDirs
-    ) where
+module Darcs.UI.Commands.Add ( add ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-#include "impossible.h"
-
-import Prelude hiding ( (^) )
-
 import Control.Exception ( catch, IOException )
-import Control.Monad ( when, unless, liftM )
+import Control.Monad ( when, unless )
 import Data.List ( (\\), nub )
 import Data.List.Ordered ( nubSort )
 import Data.Maybe ( isNothing, maybeToList )
 import Darcs.Util.Printer ( text )
 import Darcs.Util.Tree ( Tree, findTree, expand )
-import Darcs.Util.Path ( floatPath, anchorPath, parents,
-                    SubPath, toFilePath, simpleSubPath, toPath, AbsolutePath )
-import System.FilePath.Posix ( takeDirectory, (</>) )
+import Darcs.Util.Path
+    ( floatPath, anchorPath, parents
+    , SubPath, toFilePath, AbsolutePath
+    )
+import System.FilePath.Posix ( takeDirectory )
 import System.Posix.Files ( isRegularFile, isDirectory, isSymbolicLink )
 import System.Directory ( getPermissions, readable )
 
@@ -55,32 +48,30 @@ import qualified System.FilePath.Windows as WindowsFilePath
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts, putInfo, putWarning, putVerboseWarning
     , nodefaults, amInHashedRepository)
+import Darcs.UI.Commands.Util.Tree ( treeHas, treeHasDir, treeHasAnycase )
+import Darcs.UI.Commands.Util ( expandDirs, doesDirectoryReallyExist )
+import Darcs.UI.Completion ( unknownFileArgs )
 import Darcs.UI.Flags
     ( DarcsFlag
-    , includeBoring, doAllowCaseOnly, doAllowWindowsReserved, useCache, dryRun, umask
-    , fixSubPaths, verbosity )
+    , includeBoring, allowCaseDifferingFilenames, allowWindowsReservedFilenames, useCache, dryRun, umask
+    , fixSubPaths, quiet )
 import Darcs.UI.Options
-    ( DarcsOption
-    , (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+    ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.UI.Commands.Util.Tree ( treeHas, treeHasDir, treeHasAnycase )
 import Darcs.Repository.Flags ( UpdateWorking(..) )
-import Darcs.Patch ( Patchy, PrimPatch, applyToTree, addfile, adddir )
+import Darcs.Patch ( PrimPatch, applyToTree, addfile, adddir )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Repository.State ( readRecordedAndPending, updateIndex )
 import Darcs.Repository
     ( withRepoLock
     , RepoJob(..)
     , addToPending
-    , listFiles
-    , listUnregisteredFiles
     )
 import Darcs.Repository.Prefs ( darcsdirFilter, boringFileFilter )
-import Darcs.Util.File ( getFileStatus, withCurrentDirectory )
+import Darcs.Util.File ( getFileStatus )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+), nullFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), Gap(..), FreeLeft, unFreeLeft )
 
-
 addDescription :: String
 addDescription = "Add new files to version control."
 
@@ -111,38 +102,6 @@ addHelp' =
     "`ReadMe` and `README`).  If `--case-ok` is used, the repository might be\n" ++
     "unusable on those systems!\n\n"
 
-addBasicOpts :: DarcsOption a
-                (O.IncludeBoring -> Bool -> Bool -> Bool -> Maybe String -> O.DryRun -> a)
-addBasicOpts = O.includeBoring
-             ^ O.allowProblematicFilenames
-             ^ O.recursive
-             ^ O.workingRepoDir
-             ^ O.dryRun
-
-addAdvancedOpts :: DarcsOption a (O.UMask -> a)
-addAdvancedOpts = O.umask
-
-addOpts :: DarcsOption a
-           (O.IncludeBoring
-            -> Bool
-            -> Bool
-            -> Bool
-            -> Maybe String
-            -> O.DryRun
-            -> Maybe O.StdCmdAction
-            -> Bool
-            -> Bool
-            -> O.Verbosity
-            -> Bool
-            -> O.UMask
-            -> O.UseCache
-            -> Maybe String
-            -> Bool
-            -> Maybe String
-            -> Bool
-            -> a)
-addOpts = withStdOpts addBasicOpts addAdvancedOpts
-
 add :: DarcsCommand [DarcsFlag]
 add = DarcsCommand
     { commandProgramName          = "darcs"
@@ -153,8 +112,7 @@ add = DarcsCommand
     , commandExtraArgHelp         = [ "<FILE or DIRECTORY> ..." ]
     , commandCommand              = addCmd
     , commandPrereq               = amInHashedRepository
-    , commandGetArgPossibilities  = listUnregisteredFiles False
-                                    -- bash completion should not offer boring files
+    , commandCompleteArgs  = unknownFileArgs
     , commandArgdefaults          = nodefaults
     , commandAdvancedOptions      = odesc addAdvancedOpts
     , commandBasicOptions         = odesc addBasicOpts
@@ -162,6 +120,15 @@ add = DarcsCommand
     , commandCheckOptions         = ocheck addOpts
     , commandParseOptions         = onormalise addOpts
     }
+  where
+    addBasicOpts
+      = O.includeBoring
+      ^ O.allowProblematicFilenames
+      ^ O.recursive
+      ^ O.repoDir
+      ^ O.dryRun
+    addAdvancedOpts = O.umask
+    addOpts = withStdOpts addBasicOpts addAdvancedOpts
 
 
 addCmd :: (AbsolutePath, AbsolutePath)
@@ -181,7 +148,9 @@ addCmd paths opts args
 addFiles :: [DarcsFlag]  -- ^ Command options
          -> [SubPath]
          -> IO ()
-addFiles opts origfiles = withRepoLock (dryRun opts) (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+addFiles opts origfiles =
+  withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $
+  RepoJob $ \repository -> do
     -- TODO do not expand here, and use findM/findIO or such later
     -- (needs adding to hashed-storage first though)
     cur <- expand =<< readRecordedAndPending repository
@@ -196,19 +165,19 @@ addFiles opts origfiles = withRepoLock (dryRun opts) (useCache opts) YesUpdateWo
     mapM_ (putWarning opts . text . ((msgSkipping msgs ++ " boring file ")++)) $
         flist \\ nboring flist
     Sealed ps <- fmap unFreeLeft $ addp msgs opts cur $ nboring flist
-    when (nullFL ps && not (null origfiles) && notQuiet) $
+    -- TODO whether we fail or not depends on verbosity BAD BAD BAD
+    when (nullFL ps && not (null origfiles) && not (quiet opts)) $
         fail "No files were added"
     unless gotDryRun $
       do addToPending repository YesUpdateWorking ps
          updateIndex repository
   where
-    gotDryRun = dryRun opts == O.YesDryRun
+    gotDryRun = dryRun ? opts == O.YesDryRun
     msgs | gotDryRun = dryRunMessages
          | otherwise = normalMessages
-    notQuiet = verbosity opts /= O.Quiet
 
 
-addp :: forall prim . (Patchy prim, PrimPatch prim, ApplyState prim ~ Tree)
+addp :: forall prim . (PrimPatch prim, ApplyState prim ~ Tree)
      => AddMessages
      -> [DarcsFlag]
      -> Tree IO
@@ -319,8 +288,8 @@ addp msgs opts cur0 files = do
                       msgSkipping msgs ++ " '" ++ f ++ "' ... " ++ show e
                   return (cur, Nothing, Nothing)
           parentdir = takeDirectory f
-    gotAllowCaseOnly = doAllowCaseOnly opts
-    gotAllowWindowsReserved = doAllowWindowsReserved opts
+    gotAllowCaseOnly = allowCaseDifferingFilenames ? opts
+    gotAllowWindowsReserved = allowWindowsReservedFilenames ? opts
 
 
 data AddMessages = AddMessages
@@ -352,34 +321,9 @@ dryRunMessages = AddMessages
     }
 
 
-doesDirectoryReallyExist :: FilePath -> IO Bool
-doesDirectoryReallyExist f = maybe False isDirectory `fmap` getFileStatus f
-
-
-expandDirs :: Bool -> [SubPath]
-           -> IO [SubPath]
-expandDirs doIncludeBoring fs =
-  do
-    liftM (map (fromJust . simpleSubPath)) $
-      concat `fmap` mapM (expandOne doIncludeBoring . toPath) fs
-
-
-expandOne :: Bool -> FilePath
-          -> IO [FilePath]
-expandOne doIncludeBoring "" = listFiles doIncludeBoring
-expandOne doIncludeBoring f = do
-    isdir <- doesDirectoryReallyExist f
-    if not isdir
-      then return [f]
-      else do
-        fs <- withCurrentDirectory f (listFiles doIncludeBoring)
-        return $ f: map (f </>) fs
-
-
 getParents :: Tree IO
            -> [FilePath]
            -> [FilePath]
 getParents cur = map (anchorPath "") . go . map floatPath
   where
     go fs = filter (isNothing . findTree cur) $ concatMap parents fs
-
