@@ -14,22 +14,21 @@
 --  along with this program; see the file COPYING.  If not, write to
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
-{-# LANGUAGE CPP, OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Darcs.UI.Commands.Annotate ( annotate ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Prelude hiding ( (^) )
-
 import Control.Arrow ( first )
-import Control.Monad ( unless )
+import Control.Monad ( when )
 
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
-import Darcs.UI.Flags ( DarcsFlag(NoPatchIndexFlag), useCache, fixSubPaths, umask )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise
-                        , defaultFlags, parseFlags )
+import Darcs.UI.Completion ( knownFileArgs )
+import Darcs.UI.Flags ( DarcsFlag, useCache, fixSubPaths, patchIndexYes )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise
+                        , defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.State ( readRecorded )
 import Darcs.Repository
@@ -38,11 +37,9 @@ import Darcs.Repository
     , RepoJob(..)
     , readRepo
     , repoPatchType
-    , listRegisteredFiles
     )
-import Darcs.Repository.Flags ( UpdateWorking(..) )
 import Darcs.Repository.PatchIndex ( attemptCreatePatchIndex )
-import Darcs.Patch.Set ( newset2RL )
+import Darcs.Patch.Set ( patchSet2RL )
 import Darcs.Patch ( invertRL )
 import qualified Data.ByteString.Char8 as BC ( pack, concat, intercalate )
 import Data.ByteString.Lazy ( toChunks )
@@ -57,10 +54,8 @@ import qualified Darcs.Patch.Annotate as A
 import Darcs.Util.Tree( TreeItem(..), readBlob, list, expand )
 import Darcs.Util.Tree.Monad( findM, virtualTreeIO )
 import Darcs.Util.Path( floatPath, anchorPath, fp2fn, toFilePath
-                      , AbsolutePath )
-import qualified Darcs.Util.Diff as D ( DiffAlgorithm(MyersDiff) )
-
-#include "impossible.h"
+                      , AbsolutePath, SubPath )
+import Darcs.Util.Exception ( die )
 
 annotateDescription :: String
 annotateDescription = "Annotate lines of a file with the last patch that modified it."
@@ -74,36 +69,6 @@ annotateHelp = unlines
  , "machine postprocessing."
  ]
 
-annotateBasicOpts :: DarcsOption a
-                     (Bool
-                      -> [O.MatchFlag]
-                      -> Maybe String
-                      -> a)
-annotateBasicOpts = O.machineReadable
-                  ^ O.matchUpToOne
-                  ^ O.workingRepoDir
-
-annotateAdvancedOpts :: DarcsOption a (O.WithPatchIndex -> a)
-annotateAdvancedOpts = O.patchIndexYes
-
-annotateOpts :: DarcsOption a
-                (   Bool
-                 -> [O.MatchFlag]
-                 -> Maybe String
-                 -> Maybe O.StdCmdAction
-                 -> Bool
-                 -> Bool
-                 -> O.Verbosity
-                 -> Bool
-                 -> O.WithPatchIndex
-                 -> O.UseCache
-                 -> Maybe String
-                 -> Bool
-                 -> Maybe String
-                 -> Bool
-                 -> a)
-annotateOpts = annotateBasicOpts `withStdOpts` annotateAdvancedOpts
-
 annotate :: DarcsCommand [DarcsFlag]
 annotate = DarcsCommand
     { commandProgramName = "darcs"
@@ -114,60 +79,69 @@ annotate = DarcsCommand
     , commandExtraArgHelp = ["[FILE or DIRECTORY]"]
     , commandCommand = annotateCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = listRegisteredFiles
+    , commandCompleteArgs = knownFileArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc annotateAdvancedOpts
     , commandBasicOptions = odesc annotateBasicOpts
     , commandDefaults = defaultFlags annotateOpts
     , commandCheckOptions = ocheck annotateOpts
     , commandParseOptions = onormalise annotateOpts
-}
+    }
+  where
+    annotateBasicOpts = O.machineReadable ^ O.matchUpToOne ^ O.repoDir
+    annotateAdvancedOpts = O.patchIndexYes
+    annotateOpts = annotateBasicOpts `withStdOpts` annotateAdvancedOpts
 
 annotateCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-annotateCmd _ _ [""] = fail "No filename argument given to annotate!"
 annotateCmd fps opts args = do
- let matchFlags = parseFlags O.matchUpToOne opts
- unless (NoPatchIndexFlag `elem` opts)
-   $ withRepoLockCanFail (useCache opts) YesUpdateWorking (umask opts) $ RepoJob attemptCreatePatchIndex
- withRepository (useCache opts) $ RepoJob $ \repository -> do
-  r <- readRepo repository
-  (origpath:_) <- fixSubPaths fps args
-  recorded <- readRecorded repository
+  fixed_paths <- fixSubPaths fps args
+  case fixed_paths of
+    [] -> die "Error: annotate needs a filename to work with"
+    (fixed_path:_) -> do
+      when (patchIndexYes ? opts == O.YesPatchIndex)
+        $ withRepoLockCanFail (useCache ? opts)
+        $ RepoJob (\repo -> readRepo repo >>= attemptCreatePatchIndex repo)
+      annotateCmd' opts fixed_path
 
+annotateCmd' :: [DarcsFlag] -> SubPath -> IO ()
+annotateCmd' opts fixed_path = withRepository (useCache ? opts) $ RepoJob $ \repository -> do
+  let matchFlags = parseFlags O.matchUpToOne opts
+  r <- readRepo repository
+  recorded <- readRecorded repository
   (patches, initial, path') <-
     if haveNonrangeMatch (repoPatchType repository) matchFlags
        then do Sealed x <- getOnePatchset repository matchFlags
-               let fn = [fp2fn $ toFilePath origpath]
+               let fn = [fp2fn $ toFilePath fixed_path]
                    nonRangeMatch = getNonrangeMatchS matchFlags r
                    (_, [path], _) = withFileNames Nothing fn nonRangeMatch
                initial <- snd `fmap` virtualTreeIO (getNonrangeMatchS matchFlags r) recorded
-               return (seal $ newset2RL x, initial, toFilePath path)
-       else return (seal $ newset2RL r, recorded, toFilePath origpath)
+               return (seal $ patchSet2RL x, initial, toFilePath path)
+       else return (seal $ patchSet2RL r, recorded, toFilePath fixed_path)
   let path = "./" ++ path'
 
   found <- findM initial (floatPath $ toFilePath path)
   -- TODO need to decide about the --machine flag
   let fmt = if parseFlags O.machineReadable opts then A.machineFormat else A.format
+  usePatchIndex <- (O.yes (O.patchIndexYes ? opts) &&) <$> canUsePatchIndex repository
   case found of
-    Nothing -> fail $ "No such file or directory: " ++ toFilePath path
+    Nothing -> die $ "Error: no such file or directory: " ++ toFilePath path
     Just (SubTree s) -> do
       s' <- expand s
       let subs = map (fp2fn . (path </>) . anchorPath "" . fst) $ list s'
           showPath (n, File _) = BC.pack (path </> n)
           showPath (n, _) = BC.concat [BC.pack (path </> n), "/"]
       (Sealed ans_patches) <- do
-         upi <- canUsePatchIndex repository
-         if not upi
+         if not usePatchIndex
             then return patches
-            else getRelevantSubsequence patches repository subs
+            else getRelevantSubsequence patches repository r subs
       putStrLn $ fmt (BC.intercalate "\n" $
                         map (showPath . first (anchorPath "")) $ list s') $
-        A.annotateDirectory D.MyersDiff (invertRL ans_patches) (fp2fn path) subs
+        A.annotateDirectory (invertRL ans_patches) (fp2fn path) subs
     Just (File b) -> do (Sealed ans_patches) <- do
-                           upi <- canUsePatchIndex repository
-                           if not upi
+                           if not usePatchIndex
                               then return patches
-                              else getRelevantSubsequence patches repository [fp2fn path]
+                              else getRelevantSubsequence patches repository r [fp2fn path]
                         con <- BC.concat `fmap` toChunks `fmap` readBlob b
-                        putStrLn $ fmt con $ A.annotate D.MyersDiff (invertRL ans_patches) (fp2fn path) con
+                        putStrLn $ fmt con $
+                          A.annotateFile (invertRL ans_patches) (fp2fn path) con
     Just (Stub _ _) -> impossible
