@@ -676,6 +676,7 @@ type Branch = B.ByteString
 type AuthorInfo = B.ByteString
 type Message = B.ByteString
 type Content = B.ByteString
+type Tag = B.ByteString
 
 data RefId = MarkId Int | HashId B.ByteString | Inline
            deriving Show
@@ -690,7 +691,7 @@ data CopyRenameNames = Quoted B.ByteString B.ByteString
 data Object = Blob (Maybe Int) Content
             | Reset Branch (Maybe RefId)
             | Commit Branch Marked AuthorInfo Message
-            | Tag Int AuthorInfo Message
+            | Tag Tag Int AuthorInfo Message
             | Modify (Either Int Content) B.ByteString -- (mark or content), filename
             | Gitlink B.ByteString
             | Copy CopyRenameNames
@@ -819,11 +820,11 @@ fastImport' repo marks = do
               BC.concat [BC.pack "pristine:", root]
           return Done
 
-        process (Toplevel n b) (Tag what author msg) = do
+        process (Toplevel n b) (Tag tag what author msg) = do
           if Just what == n
              then addtag author msg
-             else liftIO $ putStrLn $ "WARNING: Ignoring out-of-order tag " ++
-                             head (lines $ decodeLocale msg)
+             else liftIO $ putStrLn $
+                    "WARNING: Ignoring out-of-order tag " ++ decodeLocale tag
           return (Toplevel n b)
 
         process (Toplevel n _) (Reset branch from) =
@@ -1003,12 +1004,13 @@ parseObject = next' mbObject
                       message <- p_data
                       return $ Commit branch mark committer message
 
-        p_tag = do _ <- lexString "tag" >> line -- FIXME we ignore branch for now
+        p_tag = do _ <- lexString "tag"
+                   tag <- line
                    lexString "from"
                    mark <- p_marked
                    author <- p_author "tagger"
                    message <- p_data
-                   return $ Tag mark author message
+                   return $ Tag tag mark author message
 
         p_blob = do lexString "blob"
                     mark <- optional p_mark
