@@ -15,8 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.UI.Commands.Diff ( diffCommand, getDiffDoc ) where
 
 import Prelude ()
@@ -39,10 +37,9 @@ import Darcs.UI.External
     , execPipeIgnoreError
     )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
-import Darcs.UI.Flags
-    ( DarcsFlag ( AfterPatch, DiffCmd, LastN )
-    , wantGuiPause, useCache, fixSubPaths )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+import Darcs.UI.Completion ( knownFileArgs )
+import Darcs.UI.Flags ( DarcsFlag, wantGuiPause, useCache, fixSubPaths )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags ( WantGuiPause (..), DiffAlgorithm(MyersDiff) )
 import Darcs.Patch.PatchInfoAnd ( info, n2pia )
@@ -55,22 +52,21 @@ import Darcs.Patch.Match
     , matchSecondPatchset
     )
 import Darcs.Repository ( withRepository, RepoJob(..), readRepo )
-import Darcs.Repository.State ( readUnrecorded, restrictSubpaths
-                              , readRecorded, unrecordedChanges
-                              , UseIndex(..), ScanKnown(..), applyTreeFilter )
+import Darcs.Repository.State
+    ( readUnrecorded, restrictSubpaths
+    , readRecorded, unrecordedChanges
+    , UseIndex(..), ScanKnown(..), applyTreeFilter
+    )
 import Darcs.Patch.Witnesses.Ordered ( mapRL, (:>)(..), (+>+), RL(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoercePEnd )
 import Darcs.Patch.Witnesses.Sealed ( unseal, Sealed(..), seal )
 import Darcs.Patch ( RepoPatch, IsRepoType, apply, listTouchedFiles, invert, fromPrims )
 import Darcs.Patch.Depends ( findCommonWithThem )
 import Darcs.Patch.Named.Wrapped ( anonymous )
-import Darcs.Patch.Set ( PatchSet(..), newset2RL )
-import Darcs.Patch.Info ( PatchInfo, showPatchInfoUI )
+import Darcs.Patch.Set ( PatchSet(..), patchSet2RL )
+import Darcs.Patch.Info ( PatchInfo, displayPatchInfo )
 import Darcs.Util.Lock ( withTempDir )
-import Darcs.Util.Printer ( Doc, putDoc, vcat, empty, RenderMode(..), ($$) )
-
-#include "impossible.h"
-
+import Darcs.Util.Printer ( Doc, putDoc, vcat, empty, ($$) )
 
 diffDescription :: String
 diffDescription = "Create a diff between two versions of the repository."
@@ -110,43 +106,6 @@ diffHelp =
  "\n" ++
  "If this option is used, `--diff-opts` is ignored.\n"
 
-diffBasicOpts :: DarcsOption a
-                 ([O.MatchFlag]
-                  -> O.ExternalDiff
-                  -> Bool
-                  -> Maybe String
-                  -> Bool
-                  -> a)
-diffBasicOpts
-    = O.matchRange
-    ^ O.extDiff
-    ^ O.unidiff
-    ^ O.workingRepoDir
-    ^ O.storeInMemory
-
-diffAdvancedOpts :: DarcsOption a (WantGuiPause -> a)
-diffAdvancedOpts = O.pauseForGui
-
-diffOpts :: DarcsOption a
-            ([O.MatchFlag]
-             -> O.ExternalDiff
-             -> Bool
-             -> Maybe String
-             -> Bool
-             -> Maybe O.StdCmdAction
-             -> Bool
-             -> Bool
-             -> O.Verbosity
-             -> Bool
-             -> WantGuiPause
-             -> O.UseCache
-             -> Maybe String
-             -> Bool
-             -> Maybe String
-             -> Bool
-             -> a)
-diffOpts = diffBasicOpts `withStdOpts` diffAdvancedOpts
-
 diffCommand :: DarcsCommand [DarcsFlag]
 diffCommand = DarcsCommand
     { commandProgramName = "darcs"
@@ -157,7 +116,7 @@ diffCommand = DarcsCommand
     , commandExtraArgHelp = ["[FILE or DIRECTORY]..."]
     , commandCommand = diffCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = knownFileArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc diffAdvancedOpts
     , commandBasicOptions = odesc diffBasicOpts
@@ -165,11 +124,18 @@ diffCommand = DarcsCommand
     , commandCheckOptions = ocheck diffOpts
     , commandParseOptions = onormalise diffOpts
     }
+  where
+    diffBasicOpts
+      = O.matchRange
+      ^ O.extDiff
+      ^ O.repoDir
+      ^ O.storeInMemory
+    diffAdvancedOpts = O.pauseForGui
+    diffOpts = diffBasicOpts `withStdOpts` diffAdvancedOpts
 
-getDiffOpts :: [DarcsFlag] -> [String]
-getDiffOpts fs = addUnified $ otherDiffOpts fs where
-  addUnified = if parseFlags O.unidiff fs then ("-u":) else id
-  otherDiffOpts = O._diffOpts . parseFlags O.extDiff
+getDiffOpts :: O.ExternalDiff -> [String]
+getDiffOpts O.ExternalDiff {O.diffOpts=os,O.diffUnified=u} = addUnified os where
+  addUnified = if u then ("-u":) else id
 
 -- | Returns the command we should use for diff as a tuple (command, arguments).
 -- This will either be whatever the user specified via --diff-command  or the
@@ -177,20 +143,21 @@ getDiffOpts fs = addUnified $ otherDiffOpts fs where
 -- user's diff-command, hence the possibility for failure with an exception.
 getDiffCmdAndArgs :: String -> [DarcsFlag] -> String -> String
                       -> Either String (String, [String])
-getDiffCmdAndArgs cmd opts f1 f2 = helper opts where
-  helper (DiffCmd c:_) =
-    case parseCmd [ ('1', f1) , ('2', f2) ] c of
-    Left err      -> Left $ show err
-    Right ([],_)  -> bug "parseCmd should never return empty list"
-    Right (h:t,_) -> Right (h,t)
-  helper [] = -- if no command specified, use 'diff'
-    Right (cmd, "-rN":getDiffOpts opts++[f1,f2])
-  helper (_:t) = helper t
+getDiffCmdAndArgs cmd opts f1 f2 = helper (O.extDiff ? opts) where
+  helper extDiff =
+    case O.diffCmd extDiff of
+      Just c ->
+        case parseCmd [ ('1', f1) , ('2', f2) ] c of
+          Left err      -> Left $ show err
+          Right ([],_)  -> bug "parseCmd should never return empty list"
+          Right (h:t,_) -> Right (h,t)
+      Nothing -> -- if no command specified, use 'diff'
+        Right (cmd, "-rN":getDiffOpts extDiff++[f1,f2])
 
 diffCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 diffCmd fps opts args
-  | not (null [i | LastN i <- opts]) &&
-      not (null [p | AfterPatch p <- opts]) =
+  | not (null (O.matchLast ? opts)) &&
+      not (null (O.matchFrom ? opts)) =
         fail $ "using --patch and --last at the same time with the 'diff'" ++
           " command doesn't make sense. Use --from-patch to create a diff" ++
           " from this patch to the present, or use just '--patch' to view" ++
@@ -202,14 +169,16 @@ doDiff :: [DarcsFlag] -> Maybe [SubPath] ->  IO ()
 doDiff opts msubpaths = getDiffDoc opts msubpaths >>= putDoc
 
 getDiffDoc :: [DarcsFlag] -> Maybe [SubPath] ->  IO Doc
-getDiffDoc opts msubpaths = withRepository (useCache opts) $ RepoJob $ \repository -> do
+getDiffDoc opts msubpaths = withRepository (useCache ? opts) $ RepoJob $ \repository -> do
   formerdir <- getCurrentDirectory
 
   let thename = takeFileName formerdir
 
   patchset <- readRepo repository
 
-  unrecorded <- fromPrims `fmap` unrecordedChanges (UseIndex, ScanKnown, MyersDiff) repository msubpaths
+  unrecorded <- fromPrims `fmap` unrecordedChanges (UseIndex, ScanKnown, MyersDiff)
+    O.NoLookForMoves O.NoLookForReplaces
+    repository msubpaths
   unrecorded' <- n2pia `fmap` anonymous unrecorded
 
   let matchFlags = parseFlags O.matchRange opts
@@ -270,7 +239,7 @@ getDiffDoc opts msubpaths = withRepository (useCache opts) $ RepoJob $ \reposito
                 let pausingForGui = (wantGuiPause opts == YesWantGuiPause) in
                   do when pausingForGui $ putStrLn $
                        "Running command '" ++ unwords (d_cmd:d_args) ++ "'"
-                     output <- execPipeIgnoreError Encode d_cmd d_args empty
+                     output <- execPipeIgnoreError d_cmd d_args empty
                      when pausingForGui $
                         askEnter "Hit return to move on..."
                      return output
@@ -278,7 +247,7 @@ getDiffDoc opts msubpaths = withRepository (useCache opts) $ RepoJob $ \reposito
 getDiffInfo :: (IsRepoType rt, RepoPatch p) => [DarcsFlag] -> PatchSet rt p wStart wX -> [PatchInfo]
 getDiffInfo opts ps =
     let matchFlags = parseFlags O.matchRange opts
-        infos = mapRL info . newset2RL
+        infos = mapRL info . patchSet2RL
         handle (match_cond, do_match)
           | match_cond matchFlags = unseal infos (do_match matchFlags ps)
           | otherwise = infos ps
@@ -286,5 +255,5 @@ getDiffInfo opts ps =
          \\ handle (firstMatch, matchFirstPatchset)
 
 changelog :: [PatchInfo] -> Doc
-changelog pis = vcat $ map showPatchInfoUI pis
+changelog pis = vcat $ map displayPatchInfo pis
 
