@@ -34,27 +34,31 @@ import Darcs.UI.Commands
     ( CommandArgs(..)
     , CommandControl(..)
     , normalCommand
-    , DarcsCommand(..), withStdOpts
+    , DarcsCommand(..)
     , WrappedCommand(..)
     , wrappedCommandName
     , disambiguateCommands
     , extractCommands
-    , getCommandHelp
+    , getSubcommands
     , nodefaults
-    , usage
     )
 import Darcs.UI.External ( viewDoc )
+import Darcs.UI.Usage
+    ( getCommandHelp
+    , usage
+    , subusage
+    )
 import Darcs.Util.Lock ( environmentHelpTmpdir, environmentHelpKeepTmpdir
                        , environmentHelpLocks )
 import Darcs.Patch.Match ( helpOnMatchers )
-import Darcs.Repository.Prefs ( boringFileHelp, binariesFileHelp, environmentHelpHome )
+import Darcs.Repository.Prefs ( environmentHelpHome, prefsFilesHelp )
 import Darcs.Util.Ssh ( environmentHelpSsh, environmentHelpScp, environmentHelpSshPort )
 import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Path ( AbsolutePath )
 import Control.Arrow ( (***) )
 import Data.Char ( isAlphaNum, toLower, toUpper )
 import Data.Either ( partitionEithers )
-import Data.List ( groupBy, isPrefixOf, intercalate, nub )
+import Data.List ( groupBy, isPrefixOf, intercalate, nub, lookup )
 import Darcs.Util.English ( andClauses )
 import Darcs.Util.Printer (text, vcat, vsep, ($$), empty)
 import Darcs.Util.Printer.Color ( environmentHelpColor, environmentHelpEscape, environmentHelpEscapeWhite )
@@ -62,8 +66,7 @@ import System.Exit ( exitSuccess )
 import Version ( version )
 import Darcs.Util.Download ( environmentHelpProxy, environmentHelpProxyPassword )
 import Darcs.Util.Workaround ( getCurrentDirectory )
-import Darcs.UI.Options ( DarcsOption, defaultFlags, ocheck, onormalise, oid )
-import qualified Darcs.UI.Options.All as O ( StdCmdAction, Verbosity, UseCache )
+import Darcs.UI.Options ( defaultFlags, ocheck, onormalise, oid )
 import qualified Darcs.UI.TheCommands as TheCommands
 
 helpDescription :: String
@@ -75,22 +78,25 @@ helpHelp =
  "commands and a short description of each one.  With an extra argument,\n" ++
  "`darcs help foo` prints detailed help about the darcs command foo.\n"
 
-argPossibilities :: [String]
-argPossibilities = map wrappedCommandName $ extractCommands commandControlList
+-- | Starting from a list of 'CommandControl's, unwrap one level
+-- to get a list of command names together with their subcommands.
+unwrapTree :: [CommandControl] -> [(String, [CommandControl])]
+unwrapTree cs = [ (wrappedCommandName c, subcmds c) | CommandData c <- cs ]
+  where
+    subcmds (WrappedCommand sc) = getSubcommands sc
 
-helpOpts :: DarcsOption a
-            (Maybe O.StdCmdAction
-             -> Bool
-             -> Bool
-             -> O.Verbosity
-             -> Bool
-             -> O.UseCache
-             -> Maybe String
-             -> Bool
-             -> Maybe String
-             -> Bool
-             -> a)
-helpOpts = withStdOpts oid oid
+-- | Given a list of (normal) arguments to the help command, produce a list
+-- of possible completions for the next (normal) argument.
+completeArgs :: [String] -> [String]
+completeArgs [] = map fst (unwrapTree commandControlList) ++ extraArgs where
+  extraArgs = [ "manpage", "markdown", "patterns", "environment" ]
+completeArgs (arg:args) = exploreTree arg args commandControlList where
+  exploreTree cmd cmds cs =
+    case lookup cmd (unwrapTree cs) of
+      Nothing -> []
+      Just cs' -> case cmds of
+        [] -> map fst (unwrapTree cs')
+        sub:cmds' -> exploreTree sub cmds' cs'
 
 help :: DarcsCommand [DarcsFlag]
 help = DarcsCommand
@@ -102,13 +108,13 @@ help = DarcsCommand
     , commandExtraArgHelp = ["[<DARCS_COMMAND> [DARCS_SUBCOMMAND]]  "]
     , commandCommand = \ x y z -> helpCmd x y z >> exitSuccess
     , commandPrereq = \_ -> return $ Right ()
-    , commandGetArgPossibilities = return argPossibilities
+    , commandCompleteArgs = \_ _ -> return . completeArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = []
     , commandBasicOptions = []
-    , commandDefaults = defaultFlags helpOpts
-    , commandCheckOptions = ocheck helpOpts
-    , commandParseOptions = onormalise helpOpts
+    , commandDefaults = defaultFlags oid
+    , commandCheckOptions = ocheck oid
+    , commandParseOptions = onormalise oid
     }
 
 helpCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
@@ -143,18 +149,21 @@ helpCmd _ _ ("environment":vs_) =
                      [] -> Left v
                      es -> Right es
 
-helpCmd _ _ [] = viewDoc $ text $ usage commandControlList
+helpCmd _ _ [] = viewDoc $ usage commandControlList
 
 helpCmd _ _ (cmd:args) =
-    let disambiguated = disambiguateCommands commandControlList cmd args
-    in case disambiguated of
+    case disambiguateCommands commandControlList cmd args of
          Left err -> fail err
-         Right (cmds,_) ->
+         Right (cmds,as) ->
              let msg = case cmds of
                          CommandOnly c       -> getCommandHelp Nothing  c
-                         SuperCommandOnly c  -> getCommandHelp Nothing  c
+                         SuperCommandOnly c  ->
+                          if null as then
+                            getCommandHelp Nothing  c
+                          else
+                            text $ "Invalid subcommand!\n\n" ++ subusage c
                          SuperCommandSub c s -> getCommandHelp (Just c) s
-             in viewDoc $ text msg
+             in viewDoc $ msg
 
 listAvailableCommands :: IO ()
 listAvailableCommands =
@@ -168,7 +177,6 @@ listAvailableCommands =
        putStrLn "--help"
        putStrLn "--version"
        putStrLn "--exact-version"
-       putStrLn "--overview"
     where isRight (Right _) = True
           isRight _ = False
 
@@ -263,10 +271,7 @@ manpageLines = [
  unlines commands,
  unlines environment,
  ".SH FILES",
- ".SS \"_darcs/prefs/binaries\"",
- escape $ unlines binariesFileHelp,
- ".SS \"_darcs/prefs/boring\"",
- escape $ unlines boringFileHelp,
+ unlines prefFiles,
  ".SH BUGS",
  "At http://bugs.darcs.net/ you can find a list of known",
  "bugs in Darcs.  Unknown bugs can be reported at that",
@@ -357,15 +362,23 @@ manpageLines = [
                   then repl ++ replace find repl (drop (length find) s)
                   else head s : replace find repl (tail s)
 
+      prefFiles = concatMap go prefsFilesHelp
+        where go (f,h) = [".SS \"_darcs/prefs/" ++ f ++ "\"", escape h]
+
 markdownLines :: [String]
 markdownLines =
  [ "# Commands", ""
  , unlines commands
- , "# Environment variables"
- , "", unlines environment
  , "# Patterns"
- , "", unlines helpOnMatchers  ]
+ , "", unlines helpOnMatchers
+ , "# Configuration"
+ , "", unlines prefFiles
+ , "# Environment variables"
+ , "", unlines environment ]
    where
+      prefFiles = concatMap go prefsFilesHelp
+        where go (f,h) = ["## `_darcs/prefs/" ++ f ++ "`", "", h]
+
       environment :: [String]
       environment = intercalate [""]
                      [ renderEnv ks ds | (ks, ds) <- environmentHelp ]
@@ -395,12 +408,12 @@ markdownLines =
        where opts2 = commandAdvancedOptions c
 
 environmentHelpEditor :: ([String], [String])
-environmentHelpEditor = (["DARCS_EDITOR", "DARCSEDITOR", "VISUAL", "EDITOR"],[
+environmentHelpEditor = (["DARCS_EDITOR", "VISUAL", "EDITOR"],[
  "To edit a patch description of email comment, Darcs will invoke an",
  "external editor.  Your preferred editor can be set as any of the",
- "environment variables $DARCS_EDITOR, $DARCSEDITOR, $VISUAL or $EDITOR.",
- "If none of these are set, vi(1) is used.  If vi crashes or is not",
- "found in your PATH, emacs, emacs -nw, nano and (on Windows) edit are",
+ "environment variables $DARCS_EDITOR, $VISUAL or $EDITOR.",
+ "If none of these are set, nano is used.  If nano crashes or is not",
+ "found in your PATH, vi, emacs, emacs -nw and (on Windows) edit are",
  "each tried in turn."])
 
 environmentHelpPager :: ([String], [String])
@@ -412,8 +425,8 @@ environmentHelpPager = (["DARCS_PAGER", "PAGER"],[
 environmentHelpTimeout :: ([String], [String])
 environmentHelpTimeout = (["DARCS_CONNECTION_TIMEOUT"],[
  "Set the maximum time in seconds that darcs allows and connection to",
- "take. If the variable is not specified the default are 30 seconds. This",
- "option only works with curl."])
+ "take. If the variable is not specified the default are 30 seconds.",
+ "This option only works with curl."])
 
 -- | There are two environment variables that we do not document:
 -- - DARCS_USE_ISPRINT: deprecated, use DARCS_DONT_ESCAPE_ISPRINT.
