@@ -15,56 +15,49 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings #-}
 
-module Darcs.UI.Commands.Optimize ( optimize, doOptimizeHTTP ) where
+module Darcs.UI.Commands.Optimize ( optimize ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( finally )
-import Control.Monad ( when, unless, forM, forM_ )
+import Control.Monad ( when, unless, forM_ )
+import Data.List ( nub )
 import Data.Maybe ( isJust, fromJust )
-import Data.List ( sort )
-import Data.Set ( difference )
 import System.Directory
     ( getDirectoryContents
     , doesDirectoryExist
-    , doesFileExist
     , renameFile
-    , getModificationTime
     , createDirectoryIfMissing
     , removeFile
     , getHomeDirectory
     )
-import System.IO.Unsafe ( unsafeInterleaveIO )
-import qualified Data.ByteString.Char8 as BS
-import qualified Data.ByteString.Lazy as BL
-
-
-import Darcs.Patch.PatchInfoAnd ( extractHash )
+import qualified Data.ByteString.Char8 as BC
 import Darcs.UI.Commands ( DarcsCommand(..), nodefaults
                          , amInHashedRepository, amInRepository, putInfo
                          , normalCommand, withStdOpts )
-import Darcs.Repository.Prefs ( getPreflist, getCaches, globalCacheDir, oldGlobalCacheDir )
+import Darcs.UI.Completion ( noArgs )
+import Darcs.Repository.Prefs ( getPreflist, getCaches, globalCacheDir )
 import Darcs.Repository
     ( Repository
+    , repoLocation
     , withRepoLock
     , RepoJob(..)
     , readRepo
     , reorderInventory
     , cleanRepository
     , replacePristine
-    , maybeIdentifyRepository
     )
-import Darcs.Repository.Internal ( IdentifyRepo(..) )
-import Darcs.Repository.HashedRepo ( inventoriesDir, patchesDir, pristineDir,
-                                    hashedInventory, filterDirContents,
-                                    readHashedPristineRoot, listInventoriesRepoDir,
-                                    listPatchesLocalBucketed, set, unset, inv2pris )
+import Darcs.Repository.Job ( withOldRepoLock )
+import Darcs.Repository.Identify ( findAllReposInDir )
+import Darcs.Repository.Hashed ( inventoriesDir, patchesDir, pristineDir,
+                                    hashedInventory,
+                                    listInventoriesRepoDir,
+                                    listPatchesLocalBucketed, diffHashLists, peekPristineHash )
+import Darcs.Repository.Packs ( createPacks )
+import Darcs.Repository.Pending ( pendingName )
 import Darcs.Repository.HashedIO ( getHashedFiles )
-import Darcs.Repository.Old ( oldRepoFailMsg )
-import Darcs.Repository.InternalTypes ( Repository(..), Pristine(..) )
 import Darcs.Patch.Witnesses.Ordered
      ( mapFL
      , bunchFL
@@ -72,8 +65,8 @@ import Darcs.Patch.Witnesses.Ordered
      )
 import Darcs.Patch ( IsRepoType, RepoPatch )
 import Darcs.Patch.Set
-    ( newset2RL
-    , newset2FL
+    ( patchSet2RL
+    , patchSet2FL
     , progressPatchSet
     )
 import Darcs.Patch.Apply( ApplyState )
@@ -85,8 +78,13 @@ import Darcs.Util.Lock
     , writeAtomicFilePS
     , rmRecursive
     , removeFileMayNotExist
+    , writeBinFile
+    )
+import Darcs.Util.File
+    ( withCurrentDirectory
+    , getRecursiveContents
+    , doesDirectoryReallyExist
     )
-import Darcs.Util.File ( withCurrentDirectory, getRecursiveContents )
 import Darcs.UI.External ( catchall )
 import Darcs.Util.Progress
     ( beginTedious
@@ -99,17 +97,13 @@ import Darcs.Util.Global ( darcsdir )
 import System.FilePath.Posix
     ( takeExtension
     , (</>)
-    , (<.>)
-    , takeFileName
     , joinPath
     )
 import Text.Printf ( printf )
-import System.Posix.Files ( getFileStatus, isDirectory )
 import Darcs.UI.Flags
-    (  DarcsFlag(Compress)
-    , compression, verbosity, useCache, umask )
+    (  DarcsFlag, verbosity, useCache, umask )
 import Darcs.UI.Options ( DarcsOption, (^), oid, odesc, ocheck, onormalise
-                        , defaultFlags, parseFlags )
+                        , defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags
     ( UpdateWorking (..), DryRun ( NoDryRun ), UseCache (..), UMask (..)
@@ -125,7 +119,7 @@ import Darcs.Repository.Format
     , RepoProperty ( HashedInventory )
     )
 import Darcs.Repository.PatchIndex
-import qualified Darcs.Repository.HashedRepo as HashedRepo
+import qualified Darcs.Repository.Hashed as HashedRepo
 import Darcs.Repository.State ( readRecorded )
 
 import Darcs.Util.Tree
@@ -141,10 +135,6 @@ import Darcs.Util.Tree.Hashed
     ( writeDarcsHashed
     , decodeDarcsSize
     )
-import Codec.Archive.Tar ( write )
-import Codec.Archive.Tar.Entry ( fileEntry, toTarPath )
-import Codec.Compression.GZip ( compress )
-
 
 optimizeDescription :: String
 optimizeDescription = "Optimize the repository."
@@ -176,24 +166,10 @@ optimize = SuperCommand {
     }
 
 commonBasicOpts :: DarcsOption a (Maybe String -> UMask -> a)
-commonBasicOpts = O.workingRepoDir ^ O.umask
+commonBasicOpts = O.repoDir ^ O.umask
+
 commonAdvancedOpts :: DarcsOption a a
 commonAdvancedOpts = oid
-commonOpts :: DarcsOption a
-              (Maybe String
-               -> UMask
-               -> Maybe O.StdCmdAction
-               -> Bool
-               -> Bool
-               -> O.Verbosity
-               -> Bool
-               -> UseCache
-               -> Maybe String
-               -> Bool
-               -> Maybe String
-               -> Bool
-               -> a)
-commonOpts = commonBasicOpts `withStdOpts` commonAdvancedOpts
 
 common :: DarcsCommand [DarcsFlag]
 common = DarcsCommand
@@ -206,13 +182,15 @@ common = DarcsCommand
     , commandHelp = undefined
     , commandDescription = undefined
     , commandCommand =  undefined
-    , commandGetArgPossibilities = undefined
+    , commandCompleteArgs = noArgs
     , commandAdvancedOptions = odesc commonAdvancedOpts
     , commandBasicOptions = odesc commonBasicOpts
     , commandDefaults = defaultFlags commonOpts
     , commandCheckOptions = ocheck commonOpts
     , commandParseOptions = onormalise commonOpts
     }
+  where
+    commonOpts = commonBasicOpts `withStdOpts` commonAdvancedOpts
 
 
 optimizeClean :: DarcsCommand [DarcsFlag]
@@ -225,7 +203,8 @@ optimizeClean = common
 
 optimizeCleanCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeCleanCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       putInfo opts "Done cleaning repository!"
 
@@ -248,9 +227,10 @@ optimizeHttp = common
 
 optimizeHttpCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeHttpCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
-      doOptimizeHTTP repository
+      createPacks repository
       putInfo opts "Done creating packs!"
 
 optimizePristine :: DarcsCommand [DarcsFlag]
@@ -264,7 +244,8 @@ optimizePristine = common
 
 optimizePristineCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizePristineCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doOptimizePristine opts repository
       putInfo opts "Done optimizing pristine!"
@@ -287,34 +268,39 @@ optimizeUncompress = common
 
 optimizeCompressCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeCompressCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
-      optimizeCompression [Compress]
+      optimizeCompression O.GzipCompression opts
       putInfo opts "Done optimizing by compression!"
 
 optimizeUncompressCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeUncompressCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
-      optimizeCompression []
+      optimizeCompression O.NoCompression opts
       putInfo opts "Done optimizing by uncompression!"
 
-optimizeCompression :: [DarcsFlag] -> IO ()
-optimizeCompression opts = do
+optimizeCompression :: O.Compression -> [DarcsFlag] -> IO ()
+optimizeCompression compression opts = do
     putInfo opts "Optimizing (un)compression of patches..."
-    do_compress (darcsdir++"/patches")
+    do_compress (darcsdir ++ "/patches")
     putInfo opts "Optimizing (un)compression of inventories..."
-    do_compress (darcsdir++"/inventories")
-    where do_compress f =
-              do isd <- doesDirectoryExist f
-                 if isd then withCurrentDirectory f $
-                             do fs <- filter notdot `fmap` getDirectoryContents "."
-                                mapM_ do_compress fs
-                        else if Compress `elem` opts
-                             then gzReadFilePS f >>= gzWriteAtomicFilePS f
-                             else gzReadFilePS f >>= writeAtomicFilePS f
-          notdot ('.':_) = False
-          notdot _ = True
+    do_compress (darcsdir ++ "/inventories")
+    where
+      do_compress f = do
+        isd <- doesDirectoryExist f
+        if isd
+          then withCurrentDirectory f $ do
+                 fs <- filter notdot `fmap` getDirectoryContents "."
+                 mapM_ do_compress fs
+          else gzReadFilePS f >>=
+               case compression of
+                 O.GzipCompression -> gzWriteAtomicFilePS f
+                 O.NoCompression -> writeAtomicFilePS f
+      notdot ('.':_) = False
+      notdot _ = True
 
 optimizeEnablePatchIndex :: DarcsCommand [DarcsFlag]
 optimizeEnablePatchIndex = common
@@ -336,14 +322,17 @@ optimizeDisablePatchIndex = common
 
 optimizeEnablePatchIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeEnablePatchIndexCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
-      createOrUpdatePatchIndexDisk repository
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
+      ps <- readRepo repository
+      createOrUpdatePatchIndexDisk repository ps
       putInfo opts "Done enabling patch index!"
 
 optimizeDisablePatchIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeDisablePatchIndexCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \(Repo repodir _ _ _) -> do
-      deletePatchIndex repodir
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repo -> do
+      deletePatchIndex (repoLocation repo)
       putInfo opts "Done disabling patch index!"
 
 optimizeReorder :: DarcsCommand [DarcsFlag]
@@ -359,30 +348,11 @@ optimizeReorder = common
 
 optimizeReorderCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeReorderCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
-      reorderInventory repository (compression opts) YesUpdateWorking (verbosity opts)
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
+      reorderInventory repository (O.compress ? opts) YesUpdateWorking (verbosity ? opts)
       putInfo opts "Done reordering!"
 
-optimizeRelinkBasicOpts :: DarcsOption a
-                           (Maybe String -> UMask -> [AbsolutePath] -> a)
-optimizeRelinkBasicOpts = commonBasicOpts ^ O.siblings
-optimizeRelinkOpts :: DarcsOption a
-                      (Maybe String
-                       -> UMask
-                       -> [AbsolutePath]
-                       -> Maybe O.StdCmdAction
-                       -> Bool
-                       -> Bool
-                       -> O.Verbosity
-                       -> Bool
-                       -> UseCache
-                       -> Maybe String
-                       -> Bool
-                       -> Maybe String
-                       -> Bool
-                       -> a)
-optimizeRelinkOpts = optimizeRelinkBasicOpts `withStdOpts` commonAdvancedOpts
-
 optimizeRelink :: DarcsCommand [DarcsFlag]
 optimizeRelink = common
     { commandName = "relink"
@@ -395,10 +365,14 @@ optimizeRelink = common
     , commandCheckOptions = ocheck optimizeRelinkOpts
     , commandParseOptions = onormalise optimizeRelinkOpts
     }
+  where
+    optimizeRelinkBasicOpts = commonBasicOpts ^ O.siblings
+    optimizeRelinkOpts = optimizeRelinkBasicOpts `withStdOpts` commonAdvancedOpts
 
 optimizeRelinkCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeRelinkCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doRelink opts
       putInfo opts "Done relinking!"
@@ -434,14 +408,14 @@ optimizeHelpRelink =
  "repository, or if you pulled the same patch from a remote repository\n" ++
  "into multiple local repositories."
 
-doOptimizePristine :: (RepoPatch p, ApplyState p ~ Tree) => [DarcsFlag] -> Repository rt p wR wU wT -> IO ()
+doOptimizePristine :: [DarcsFlag] -> Repository rt p wR wU wT -> IO ()
 doOptimizePristine opts repo = do
-    inv <- BS.readFile (darcsdir </> "hashed_inventory")
-    let linesInv = BS.split '\n' inv
+    inv <- BC.readFile (darcsdir </> "hashed_inventory")
+    let linesInv = BC.split '\n' inv
     case linesInv of
       [] -> return ()
       (pris_line:_) ->
-          let size = decodeDarcsSize $ BS.drop 9 pris_line
+          let size = decodeDarcsSize $ BC.drop 9 pris_line
            in when (isJust size) $ do putInfo opts "Optimizing hashed pristine..."
                                       readRecorded repo >>= replacePristine repo
                                       cleanRepository repo
@@ -473,14 +447,12 @@ maybeRelinkFile (h:t) f =
 -- Only 'optimize' commands that works on old-fashionned repositories
 optimizeUpgradeCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeUpgradeCmd _ opts _ = do
-  debugMessage "Upgrading to hashed..."
   rf <- identifyRepoFormat "."
   debugMessage "Found our format"
   if formatHas HashedInventory rf
      then putInfo opts "No action taken because this repository already is hashed."
-     else do putInfo opts "Checking repository in case of corruption..."
-             withRepoLock NoDryRun YesUseCache YesUpdateWorking NoUMask $ RepoJob $ \repository ->
-               actuallyUpgradeFormat repository
+     else do putInfo opts "Upgrading to hashed..."
+             withOldRepoLock $ RepoJob actuallyUpgradeFormat
 
 actuallyUpgradeFormat
   :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
@@ -490,98 +462,48 @@ actuallyUpgradeFormat repository = do
   patches <- readRepo repository
   let k = "Hashing patch"
   beginTedious k
-  tediousSize k (lengthRL $ newset2RL patches)
+  tediousSize k (lengthRL $ patchSet2RL patches)
   let patches' = progressPatchSet k patches
   cache <- getCaches YesUseCache "."
-  let compr = compression [] -- default compression
-  HashedRepo.writeTentativeInventory cache compr patches'
+  -- TODO Why use the default and not what the user provided?
+  -- Is it because the author couldn't be bothered to add the option?
+  -- Or is there a more profound reason?
+  -- Such things justify a few lines of comment!
+  let compressDefault = O.compress ? []
+  HashedRepo.writeTentativeInventory cache compressDefault patches'
   endTedious k
   -- convert pristine by applying patches
   -- the faster alternative would be to copy pristine, but the apply method is more reliable
-  let patchesToApply = progressFL "Applying patch" $ newset2FL patches'
+  let patchesToApply = progressFL "Applying patch" $ patchSet2FL patches'
   createDirectoryIfMissing False $ darcsdir </> hashedDir HashedPristineDir
   -- We ignore the returned root hash, we don't use it.
-  _ <- writeDarcsHashed emptyTree $ darcsdir </> "pristine.hashed"
-  sequence_ $ mapFL HashedRepo.applyToTentativePristine $ bunchFL 100 patchesToApply
+  _ <- writeDarcsHashed emptyTree $ darcsdir </> hashedDir HashedPristineDir
+  writeBinFile (darcsdir++"/tentative_pristine") ""
+  sequence_ $ mapFL HashedRepo.applyToTentativePristineCwd $ bunchFL 100 patchesToApply
   -- now make it official
-  HashedRepo.finalizeTentativeChanges repository compr
+  HashedRepo.finalizeTentativeChanges repository compressDefault
   writeRepoFormat (createRepoFormat PatchFormat1 WithWorkingDir) (darcsdir </> "format")
   -- clean out old-fashioned junk
   debugMessage "Cleaning out old-fashioned repository files..."
-  removeFile   $ darcsdir </> "inventory"
-  removeFile   $ darcsdir </> "tentative_inventory"
+  removeFileMayNotExist $ darcsdir </> "inventory"
+  removeFileMayNotExist $ darcsdir </> "tentative_inventory"
   rmRecursive (darcsdir </> "pristine") `catchall` rmRecursive (darcsdir </> "current")
   rmGzsIn (darcsdir </> "patches")
   rmGzsIn (darcsdir </> "inventories")
   let checkpointDir = darcsdir </> "checkpoints"
   hasCheckPoints <- doesDirectoryExist checkpointDir
   when hasCheckPoints $ rmRecursive checkpointDir
+  removeFileMayNotExist (pendingName ++ ".tentative")
+  removeFileMayNotExist pendingName
  where
   rmGzsIn dir =
     withCurrentDirectory dir $ do
       gzs <- filter ((== ".gz") . takeExtension) `fmap` getDirectoryContents "."
       mapM_ removeFile gzs
 
-doOptimizeHTTP
-  :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-  => Repository rt p wR wU wT -> IO ()
-doOptimizeHTTP repo = flip finally (mapM_ removeFileIfExists
-  [ darcsdir </> "meta-filelist-inventories"
-  , darcsdir </> "meta-filelist-pristine"
-  , basicTar <.> "part"
-  , patchesTar <.> "part"
-  ]) $ do
-  rf <- identifyRepoFormat "."
-  -- function is exposed in API so could be called on non-hashed repo
-  unless (formatHas HashedInventory rf) $ fail oldRepoFailMsg
-  createDirectoryIfMissing False packsDir
-  -- pristine hash
-  Just hash <- readHashedPristineRoot repo
-  writeFile ( packsDir </> "pristine" ) hash
-  -- pack patchesTar
-  ps <- mapFL hashedPatchFileName . newset2FL <$> readRepo repo
-  is <- map ((darcsdir </> "inventories") </>) <$> HashedRepo.listInventories
-  writeFile (darcsdir </> "meta-filelist-inventories") . unlines $
-    map takeFileName is
-  -- tinkering with zlib's compression parameters does not make
-  -- any noticeable difference in generated archive size.
-  -- switching to bzip2 does OTOH (~25% size gain).
-  BL.writeFile (patchesTar <.> "part") . compress . write =<<
-    mapM fileEntry' ((darcsdir </> "meta-filelist-inventories") : ps ++
-    reverse is)
-  renameFile (patchesTar <.> "part") patchesTar
-  -- pack basicTar
-  pr <- sortByMTime =<< dirContents "pristine.hashed"
-  writeFile (darcsdir </> "meta-filelist-pristine") . unlines $
-    map takeFileName pr
-  BL.writeFile (basicTar <.> "part") . compress . write =<< mapM fileEntry' (
-    [ darcsdir </> "meta-filelist-pristine"
-    , darcsdir </> "hashed_inventory"
-    ] ++ reverse pr)
-  renameFile (basicTar <.> "part") basicTar
- where
-  packsDir = darcsdir </> "packs"
-  basicTar = packsDir </> "basic.tar.gz"
-  patchesTar = packsDir </> "patches.tar.gz"
-  fileEntry' x = unsafeInterleaveIO $ do
-    content <- BL.fromChunks . return <$> gzReadFilePS x
-    tp <- either fail return $ toTarPath False x
-    return $ fileEntry tp content
-  dirContents d = map ((darcsdir </> d) </>) <$>
-    filterDirContents d (const True)
-  hashedPatchFileName x = case extractHash x of
-    Left _ -> fail "unexpected unhashed patch"
-    Right h -> darcsdir </> "patches" </> h
-  sortByMTime xs = map snd . sort <$> mapM (\x -> (\t -> (t, x)) <$>
-    getModificationTime x) xs
-  removeFileIfExists x = do
-    ex <- doesFileExist x
-    when ex $ removeFile x
-
 optimizeBucketed :: [DarcsFlag] -> IO ()
 optimizeBucketed opts = do
   putInfo opts "Migrating global cache to bucketed format."
-  gOldCacheDir <- oldGlobalCacheDir
   gCacheDir <- globalCacheDir
 
   case gCacheDir of
@@ -594,13 +516,6 @@ optimizeBucketed opts = do
       toBucketed gCachePristineDir gCachePristineDir
       toBucketed gCacheInventoriesDir gCacheInventoriesDir
       toBucketed gCachePatchesDir gCachePatchesDir
-      case gOldCacheDir of
-        Nothing -> debugMessage "Old global cache doesn't exist."
-        Just gOldCacheDir' -> do
-          debugMessage "Making bucketed cache from old cache."
-          toBucketed (joinPath [gOldCacheDir', pristineDir]) gCachePristineDir
-          toBucketed (joinPath [gOldCacheDir', inventoriesDir]) gCacheInventoriesDir
-          toBucketed (joinPath [gOldCacheDir', patchesDir]) gCachePatchesDir
       putInfo opts "Done making bucketed cache!"
   where
     toBucketed :: FilePath -> FilePath -> IO ()
@@ -613,8 +528,8 @@ optimizeBucketed opts = do
                   createDirectoryIfMissing True (dest </> subDir)
                 fileNames <- getDirectoryContents src
                 forM_ fileNames $ \file -> do
-                  fileStatus <- getFileStatus (src </> file)
-                  if not $ isDirectory fileStatus
+                  exists <- doesDirectoryReallyExist (src </> file)
+                  if not $ exists
                     then renameFile' src dest file
                     else return ()
         else do
@@ -668,11 +583,11 @@ cleanGlobalCache dirs opts = do
   putInfo opts "\nLooking for repositories in the following directories:"
   putInfo opts $ text $ unlines dirs
   gCacheDir' <- globalCacheDir
-  repoPaths'  <- mapM getRecursiveDarcsRepos dirs
+  repoPaths'  <- mapM findAllReposInDir dirs
 
   putInfo opts "Finished listing repositories."
 
-  let repoPaths         = unset . set $ concat repoPaths'
+  let repoPaths         = nub $ concat repoPaths'
       gCache            = fromJust gCacheDir'
       gCacheInvDir      = gCache </> inventoriesDir
       gCachePatchesDir  = gCache </> patchesDir
@@ -696,33 +611,11 @@ cleanGlobalCache dirs opts = do
   remove' dir s1 s2 =
     mapM_ (removeFileMayNotExist . (\hashedFile ->
       dir </> bucketFolder hashedFile </> hashedFile))
-      (unset $ set s1 `difference` set s2)
+      (diffHashLists s1 s2)
 
   getPristine :: String -> IO [String]
   getPristine darcsDir = do
     i <- gzReadFilePS (darcsDir </> darcsdir </> hashedInventory)
-    getHashedFiles (darcsDir </> darcsdir </> pristineDir) [inv2pris i]
+    getHashedFiles (darcsDir </> darcsdir </> pristineDir) [peekPristineHash i]
 
--- |getRecursiveDarcsRepos returns all paths to repositories under topdir.
-getRecursiveDarcsRepos :: FilePath -> IO [FilePath]
-getRecursiveDarcsRepos topdir = do
-  isDir <- doesDirectoryExist topdir
-  if isDir
-    then do
-      status <- maybeIdentifyRepository NoUseCache topdir
-      case status of
-        GoodRepository (Repo _ _ pris _)  ->
-                                case pris of
-                                  HashedPristine -> return [topdir]
-                                  _ -> return [] -- old fashioned or broken repo
-        _                 -> getRecursiveDarcsRepos' topdir
-    else return []
 
-  where
-    getRecursiveDarcsRepos' d = do
-      names <- getDirectoryContents d
-      let properNames = filter (\x -> head x /= '.') names
-      paths <- forM properNames $ \name -> do
-        let path = d </> name
-        getRecursiveDarcsRepos path
-      return (concat paths)
