@@ -15,7 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 
 module Darcs.UI.Commands.Record
@@ -43,16 +42,13 @@ import Darcs.Repository
     , tentativelyAddPatch
     , finalizeRepositoryChanges
     , invalidateIndex
-    , unrecordedChangesWithPatches
+    , unrecordedChanges
     , readRecorded
-    , listRegisteredFiles
     )
 import Darcs.Patch ( IsRepoType, RepoPatch, PrimOf, fromPrims )
 import Darcs.Patch.Named.Wrapped ( namepatch, adddeps )
-import Darcs.Patch.Witnesses.Sealed
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), (:>)(..), nullFL )
-import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Info ( PatchInfo )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Split ( primSplitter )
@@ -74,6 +70,7 @@ import Darcs.UI.Commands
     )
 import Darcs.UI.Commands.Util ( announceFiles, filterExistingPaths,
                                 testTentativeAndMaybeExit )
+import Darcs.UI.Completion ( modifiedFileArgs )
 import Darcs.UI.Flags
     ( DarcsFlag
     , fileHelpAuthor
@@ -87,7 +84,6 @@ import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, oparse, defaultFlags
 import Darcs.UI.PatchHeader ( getLog )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags ( UpdateWorking (..), DryRun(NoDryRun), ScanKnown(..) )
-import Darcs.Repository.State ( getMovesPs, getReplaces )
 import Darcs.Util.Exception ( clarifyErrors )
 import Darcs.Util.Prompt ( promptYorn )
 import Darcs.Util.Progress ( debugMessage )
@@ -127,10 +123,10 @@ recordBasicOpts
     ^ O.testChanges
     ^ O.interactive
     ^ O.pipe
-    ^ O.askdeps
+    ^ O.askDeps
     ^ O.askLongComment
     ^ O.lookfor
-    ^ O.workingRepoDir
+    ^ O.repoDir
     ^ O.withContext
     ^ O.diffAlgorithm
 
@@ -138,37 +134,6 @@ recordAdvancedOpts :: DarcsOption a
                       (O.Logfile -> O.Compression -> O.UseIndex -> O.UMask -> O.SetScriptsExecutable -> O.IncludeBoring -> a)
 recordAdvancedOpts = O.logfile ^ O.compress ^ O.useIndex ^ O.umask ^ O.setScriptsExecutable ^ O.includeBoring
 
-recordOpts :: DarcsOption a
-              (Maybe String
-               -> Maybe String
-               -> O.TestChanges
-               -> Maybe Bool
-               -> Bool
-               -> Bool
-               -> Maybe O.AskLongComment
-               -> O.LookFor
-               -> Maybe String
-               -> O.WithContext
-               -> O.DiffAlgorithm
-               -> Maybe O.StdCmdAction
-               -> Bool
-               -> Bool
-               -> O.Verbosity
-               -> Bool
-               -> O.Logfile
-               -> O.Compression
-               -> O.UseIndex
-               -> O.UMask
-               -> O.SetScriptsExecutable
-               -> O.IncludeBoring
-               -> O.UseCache
-               -> Maybe String
-               -> Bool
-               -> Maybe String
-               -> Bool
-               -> a)
-recordOpts = recordBasicOpts `withStdOpts` recordAdvancedOpts
-
 data RecordConfig = RecordConfig
     { patchname :: Maybe String
     , author :: Maybe String
@@ -204,14 +169,16 @@ record = DarcsCommand
     , commandExtraArgHelp = ["[FILE or DIRECTORY]..."]
     , commandCommand = recordCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = listRegisteredFiles
+    , commandCompleteArgs = modifiedFileArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc recordAdvancedOpts
     , commandBasicOptions = odesc recordBasicOpts
     , commandDefaults = defaultFlags recordOpts
     , commandCheckOptions = ocheck recordOpts
     , commandParseOptions = recordConfig
-}
+    }
+  where
+    recordOpts = recordBasicOpts `withStdOpts` recordAdvancedOpts
 
 -- | commit is an alias for record
 commit :: DarcsCommand RecordConfig
@@ -232,7 +199,11 @@ recordCmd fps cfg args = do
         files <- if null args then return Nothing
                  else Just . sort <$> fixSubPaths fps args
         when (files == Just []) $ fail "No valid arguments were given."
-        files' <- traverse (filterExistingPaths repository (verbosity cfg) (useIndex cfg) scan) files
+        files' <-
+          traverse
+            (filterExistingPaths
+              repository (verbosity cfg) (useIndex cfg) scan (O.moves (lookfor cfg)))
+            files
         when (verbosity cfg /= O.Quiet) $
             traverse_ (reportNonExisting scan) files'
         let files'' = fmap snd files'
@@ -241,15 +212,9 @@ recordCmd fps cfg args = do
         return files''
       announceFiles (verbosity cfg) existing_files "Recording changes in"
       debugMessage "About to get the unrecorded changes."
-      Sealed replacePs <- if O.replaces (lookfor cfg) == O.YesLookForReplaces
-          then getReplaces (diffingOpts cfg) repository existing_files
-          else return (Sealed NilFL)
-      movesPs <- if O.moves (lookfor cfg) == O.YesLookForMoves
-          then getMovesPs repository existing_files
-          else return NilFL
-      changes <- unrecordedChangesWithPatches
-                   movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
-                   (diffingOpts cfg) repository existing_files
+      changes <- unrecordedChanges (diffingOpts cfg)
+                   (O.moves (lookfor cfg)) (O.replaces (lookfor cfg))
+                   repository existing_files
       debugMessage "I've got unrecorded changes."
       case changes of
           NilFL | not (askDeps cfg) -> do
@@ -270,7 +235,7 @@ checkNameIsNotOption (Just name) True   =
         confirmed <- promptYorn $ "You specified " ++ show name ++ " as the patch name. Is that really what you want?"
         unless confirmed $ putStrLn "Okay, aborting the record." >> exitFailure
 
-doRecord :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+doRecord :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
          => Repository rt p wR wU wR -> RecordConfig -> Maybe [SubPath] -> FL (PrimOf p) wR wX -> IO ()
 doRecord repository cfg files ps = do
     date <- getDate (pipe cfg)
@@ -361,7 +326,7 @@ recordHelp' = unlines
  , "record failed and left a `_darcs/patch_description.txt` file."
  , ""
  , unlines fileHelpAuthor
- , "If you want to manually define any extra dependencies for your patch,"
+ , "If you want to manually define any explicit dependencies for your patch,"
  , "you can use the `--ask-deps` flag. Some dependencies may be automatically"
  , "inferred from the patch's content and cannot be removed. A patch with"
  , "specific dependencies can be empty."
