@@ -15,25 +15,24 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.UI.Commands.Remove ( remove, rm, unadd ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Prelude hiding ( (^) )
-
 import Control.Monad ( when, foldM )
-import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults,
-                        commandAlias, commandStub,
-                        putWarning
-                      , amInHashedRepository
-                      )
-import Darcs.UI.Commands.Add( expandDirs )
+import Darcs.UI.Commands
+    ( DarcsCommand(..)
+    , withStdOpts, nodefaults
+    , commandAlias, commandStub
+    , putWarning, putInfo
+    , amInHashedRepository
+    )
+import Darcs.UI.Commands.Util ( expandDirs )
+import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.Flags
-    ( DarcsFlag, useCache, dryRun, umask, diffAlgorithm, fixSubPaths, verbosity )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+    ( DarcsFlag, useCache, dryRun, umask, diffAlgorithm, fixSubPaths, quiet )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags ( UpdateWorking (..) )
 import Darcs.Repository
@@ -43,7 +42,6 @@ import Darcs.Repository
     , addToPending
     , readRecordedAndPending
     , readUnrecorded
-    , listRegisteredFiles
     )
 import Darcs.Repository.Diff( treeDiff )
 import Darcs.Patch ( RepoPatch, PrimOf, PrimPatch, adddir, rmdir, addfile, rmfile,
@@ -55,8 +53,7 @@ import Darcs.Repository.Prefs ( filetypeFunction, FileType )
 import Darcs.Util.Tree( Tree, TreeItem(..), find, modifyTree, expand, list )
 import Darcs.Util.Path( anchorPath, AnchoredPath, fn2fp, SubPath, sp2fn
                       , AbsolutePath, floatPath )
-import Darcs.Util.Printer ( text )
-
+import Darcs.Util.Printer ( text, vcat )
 
 removeDescription :: String
 removeDescription = "Remove files from version control."
@@ -72,29 +69,6 @@ removeHelp =
  "Note that applying a removal patch to a repository (e.g. by pulling\n" ++
  "the patch) will ALWAYS affect the working tree of that repository.\n"
 
-removeBasicOpts :: DarcsOption a (Maybe String -> Bool -> a)
-removeBasicOpts = O.workingRepoDir ^ O.recursive
-
-removeAdvancedOpts :: DarcsOption a (O.UMask -> a)
-removeAdvancedOpts = O.umask
-
-removeOpts :: DarcsOption a
-              (Maybe String
-               -> Bool
-               -> Maybe O.StdCmdAction
-               -> Bool
-               -> Bool
-               -> O.Verbosity
-               -> Bool
-               -> O.UMask
-               -> O.UseCache
-               -> Maybe String
-               -> Bool
-               -> Maybe String
-               -> Bool
-               -> a)
-removeOpts = removeBasicOpts `withStdOpts` removeAdvancedOpts
-
 remove :: DarcsCommand [DarcsFlag]
 remove = DarcsCommand
     { commandProgramName = "darcs"
@@ -105,7 +79,7 @@ remove = DarcsCommand
     , commandExtraArgHelp = ["<FILE or DIRECTORY> ..."]
     , commandCommand = removeCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = listRegisteredFiles
+    , commandCompleteArgs = knownFileArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc removeAdvancedOpts
     , commandBasicOptions = odesc removeBasicOpts
@@ -113,6 +87,10 @@ remove = DarcsCommand
     , commandCheckOptions = ocheck removeOpts
     , commandParseOptions = onormalise removeOpts
     }
+  where
+    removeBasicOpts = O.repoDir ^ O.recursive
+    removeAdvancedOpts = O.umask
+    removeOpts = removeBasicOpts `withStdOpts` removeAdvancedOpts
 
 removeCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 removeCmd fps opts relargs = do
@@ -121,17 +99,17 @@ removeCmd fps opts relargs = do
     origfiles <- fixSubPaths fps relargs
     when (null origfiles) $
         fail "No valid arguments were given."
-    withRepoLock (dryRun opts) (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+    withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $
+      RepoJob $ \repository -> do
         args <- if parseFlags O.recursive opts
                 then reverse `fmap` expandDirs False origfiles
                 else return origfiles
         Sealed p <- makeRemovePatch opts repository args
-        let notQuiet = verbosity opts /= O.Quiet
-        when (nullFL p && not (null origfiles) && notQuiet) $
+        -- TODO whether command fails depends on verbosity BAD BAD BAD
+        when (nullFL p && not (null origfiles) && not (quiet opts)) $
             fail "No files were removed."
         addToPending repository YesUpdateWorking p
-        when notQuiet $
-            putStr $ unlines $ ["Will stop tracking:"] ++ listTouchedFiles p
+        putInfo opts $ vcat $ map text $ ["Will stop tracking:"] ++ listTouchedFiles p
 
 -- | makeRemovePatch builds a list of patches to remove the given filepaths.
 --   This function does not recursively process directories. The 'Recursive'
@@ -176,7 +154,7 @@ makeRemoveGap opts ftf recorded unrecorded unrecorded' f =
               then skipAndWarn "it is not empty"
               else return $ Just $ freeGap (rmdir f_fp :>: NilFL)
         (Just (File _), Just (File _)) -> do
-            Just `fmap` treeDiff (diffAlgorithm opts) ftf unrecorded unrecorded'
+            Just `fmap` treeDiff (diffAlgorithm ? opts) ftf unrecorded unrecorded'
         (Just (File _), _) ->
             return $ Just $ freeGap (addfile f_fp :>: rmfile f_fp :>: NilFL)
         (Just (SubTree _), _) ->
