@@ -15,8 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.UI.Commands.Replace
     ( replace
     , defaultToks
@@ -27,7 +25,7 @@ import Darcs.Prelude
 
 import qualified Data.ByteString.Char8 as BC
 import qualified Data.ByteString.Lazy as BL
-import qualified Data.ByteString as BS
+import qualified Data.ByteString as B
 import Data.Char ( isSpace )
 import Data.Maybe ( isJust )
 import Control.Exception ( catch, IOException )
@@ -36,14 +34,15 @@ import Darcs.Util.Tree( readBlob, modifyTree, findFile, TreeItem(..), Tree
                       , makeBlobBS )
 import Darcs.Util.Path( SubPath, toFilePath, AbsolutePath )
 import Darcs.UI.Flags
-    ( DarcsFlag( ForceReplace, Toks )
+    ( DarcsFlag
     , verbosity, useCache, dryRun, umask, diffAlgorithm, fixSubPaths )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags ( UpdateWorking(..) )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
+import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.Repository.Diff( treeDiff )
-import Darcs.Patch ( Patchy, PrimPatch, tokreplace, forceTokReplace
+import Darcs.Patch ( PrimPatch, tokreplace, forceTokReplace
                    , maybeApplyToTree )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.RegChars ( regChars )
@@ -53,8 +52,6 @@ import Darcs.Repository
     , addToPending
     , applyToWorking
     , readUnrecorded
-    , readRecordedAndPending
-    , listRegisteredFiles
     )
 import Darcs.Patch.TokenReplace ( defaultToks )
 import Darcs.Repository.Prefs ( FileType(TextFile) )
@@ -62,8 +59,6 @@ import Darcs.Util.Path ( floatSubPath )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+), concatFL, toFL, nullFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal, FreeLeft, Gap(..), unFreeLeft, unseal )
 
-#include "impossible.h"
-
 replaceDescription :: String
 replaceDescription = "Substitute one word for another."
 
@@ -123,31 +118,6 @@ replaceHelp =
  "`[[:alnum:]]`) are NOT supported by `--token-chars`, and will be silently\n" ++
  "treated as a simple set of characters.\n"
 
-replaceBasicOpts :: DarcsOption a (Maybe String -> Bool -> Maybe String -> a)
-replaceBasicOpts = O.tokens ^ O.forceReplace ^ O.workingRepoDir
-
-replaceAdvancedOpts :: DarcsOption a (O.UseIndex -> O.UMask -> a)
-replaceAdvancedOpts = O.useIndex ^ O.umask
-
-replaceOpts :: DarcsOption a
-               (Maybe String
-                -> Bool
-                -> Maybe String
-                -> Maybe O.StdCmdAction
-                -> Bool
-                -> Bool
-                -> O.Verbosity
-                -> Bool
-                -> O.UseIndex
-                -> O.UMask
-                -> O.UseCache
-                -> Maybe String
-                -> Bool
-                -> Maybe String
-                -> Bool
-                -> a)
-replaceOpts = replaceBasicOpts `withStdOpts` replaceAdvancedOpts
-
 replace :: DarcsCommand [DarcsFlag]
 replace = DarcsCommand
     { commandProgramName = "darcs"
@@ -161,7 +131,7 @@ replace = DarcsCommand
                             ]
     , commandCommand = replaceCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = listRegisteredFiles
+    , commandCompleteArgs = replaceArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc replaceAdvancedOpts
     , commandBasicOptions = odesc replaceBasicOpts
@@ -169,27 +139,35 @@ replace = DarcsCommand
     , commandCheckOptions = ocheck replaceOpts
     , commandParseOptions = onormalise replaceOpts
     }
+  where
+    replaceBasicOpts = O.tokens ^ O.forceReplace ^ O.repoDir
+    replaceAdvancedOpts = O.useIndex ^ O.umask
+    replaceOpts = replaceBasicOpts `withStdOpts` replaceAdvancedOpts
+
+replaceArgs  :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO [String]
+replaceArgs fps flags args =
+  if length args < 2
+    then return []
+    else knownFileArgs fps flags args
 
 replaceCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 replaceCmd fps opts (old : new : relfs@(_ : _)) =
-  withRepoLock  (dryRun opts) (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $
+  withRepoLock  (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $
     \repository -> do
         fs <- fixSubPaths fps relfs
-        toks <- chooseToks opts old new
+        toks <- chooseToks (O.tokens ? opts) old new
         let checkToken tok = unless (isTok toks tok) $
                                  fail $ "'" ++ tok ++ "' is not a valid token!"
         mapM_ checkToken [ old, new ]
         working <- readUnrecorded repository Nothing
-        pending <- readRecordedAndPending repository
         files <- filterM (exists working) fs
         Sealed replacePs <- mapSeal concatFL . toFL <$>
-            mapM (doReplace toks pending working) files
+            mapM (doReplace toks working) files
+        -- Note: addToPending takes care of commuting the replace patch and
+        -- everything it depends on past the diff between pending and working
         addToPending repository YesUpdateWorking replacePs
-        void $ applyToWorking repository (verbosity opts) replacePs `catch` \(e :: IOException) ->
-            fail $ "Can't do replace on working!\n"
-                   ++ "Perhaps one of the files already" ++ " contains '"
-                   ++ new ++ "'?\n"
-                   ++ show e
+        void $ applyToWorking repository (verbosity ? opts) replacePs `catch` \(e :: IOException) ->
+            bug $ "Can't do replace on working!\n" ++ show e
   where
     exists tree file = if isJust $ findFile tree (floatSubPath file)
                            then return True
@@ -199,20 +177,19 @@ replaceCmd fps opts (old : new : relfs@(_ : _)) =
     skipmsg f = "Skipping file '" ++ toFilePath f
                 ++ "' which isn't in the repository."
 
-    doReplace :: forall prim . (Patchy prim, PrimPatch prim,
-              ApplyState prim ~ Tree) => String -> Tree IO -> Tree IO
+    doReplace :: forall prim . (PrimPatch prim,
+              ApplyState prim ~ Tree) => String -> Tree IO
               -> SubPath -> IO (FreeLeft (FL prim))
-    doReplace toks pend work f = do
-        let maybeReplace p = isJust <$> maybeApplyToTree replacePatch p
-        workReplaced <- maybeReplace work
-        pendReplaced <- maybeReplace pend
-        if workReplaced && pendReplaced
-            then return $ joinGap (:>:) (freeGap replacePatch) gapNilFL
-            else if ForceReplace `elem` opts
-                then getForceReplace f toks work
-                else putStrLn existsMsg >> return gapNilFL
+    doReplace toks work f = do
+        workReplaced <- maybeApplyToTree replacePatch work
+        case workReplaced of
+          Just _ -> do
+            return $ joinGap (:>:) (freeGap replacePatch) gapNilFL
+          Nothing
+            | O.forceReplace ? opts -> getForceReplace f toks work
+            | otherwise -> putStrLn existsMsg >> return gapNilFL
       where
-        existsMsg = "Skipping file '" ++ fp ++ "'\nPerhaps the recorded"
+        existsMsg = "Skipping file '" ++ fp ++ "'\nPerhaps the working"
                     ++ " version of this file already contains '" ++ new
                     ++ "'?\nUse the --force option to override."
         gapNilFL = emptyGap NilFL
@@ -231,9 +208,9 @@ replaceCmd fps opts (old : new : relfs@(_ : _)) =
         let path = floatSubPath f
         content <- readBlob $ fromJust $ findFile tree path
         let newcontent = forceTokReplace toks (BC.pack new) (BC.pack old)
-                            (BS.concat $ BL.toChunks content)
+                            (B.concat $ BL.toChunks content)
             tree' = modifyTree tree path . Just . File $ makeBlobBS newcontent
-        normaliseNewTokPatch <- treeDiff (diffAlgorithm opts) ftf tree tree'
+        normaliseNewTokPatch <- treeDiff (diffAlgorithm ? opts) ftf tree tree'
         unless (unseal nullFL (unFreeLeft normaliseNewTokPatch)) $
             putStrLn $ "Don't be surprised!\n"
                        ++ "I've changed all instances of '" ++ new ++ "' to '"
@@ -243,7 +220,7 @@ replaceCmd fps opts (old : new : relfs@(_ : _)) =
         return . joinGap (+>+) normaliseNewTokPatch $ freeGap $
             tokreplace (toFilePath f) toks old new :>: NilFL
 replaceCmd _ _ [_, _] = fail "You need to supply a list of files to replace in!"
-replaceCmd _ _ _ = fail "Usage: darcs replace OLD NEW [FILES]"
+replaceCmd _ _ _ = fail "Usage: darcs replace <OLD> <NEW> <FILE>..."
 
 filenameToks :: String
 filenameToks = "A-Za-z_0-9\\-\\."
@@ -262,8 +239,8 @@ isTok toks s = all (regChars toks) s
 --
 -- Note: Limitations in the current replace patch file format prevents tokens
 -- and token-char specifiers from containing any whitespace.
-chooseToks :: [DarcsFlag] -> String -> String -> IO String
-chooseToks (Toks t : _) a b
+chooseToks :: Maybe String -> String -> String -> IO String
+chooseToks (Just t) a b
     | length t <= 2 =
         badTokenSpec $ "It must contain more than 2 characters, because it"
                        ++ " should be enclosed in square brackets"
@@ -284,7 +261,7 @@ chooseToks (Toks t : _) a b
     spaceyToken x = x ++ " must not contain any space"
     notAToken x = x ++ " is not a token, according to your spec"
 
-chooseToks (_ : fs) a b = chooseToks fs a b
-chooseToks [] a b = if isTok defaultToks a && isTok defaultToks b
-                        then return defaultToks
-                        else return filenameToks
+chooseToks Nothing a b =
+    if isTok defaultToks a && isTok defaultToks b
+      then return defaultToks
+      else return filenameToks
