@@ -15,29 +15,27 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.UI.Commands.SetPref ( setpref ) where
 
 import Prelude ()
 import Darcs.Prelude
 
 import System.Exit ( exitWith, ExitCode(..) )
-import Control.Monad (when)
-import Data.Maybe (fromMaybe)
+import Control.Monad ( when )
+import Data.Maybe ( fromMaybe )
 
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
 import Darcs.UI.Flags ( DarcsFlag, useCache, dryRun, umask)
-import Darcs.UI.Options ( DarcsOption, PrimDarcsOption, odesc, ocheck, onormalise, defaultFlags )
+import Darcs.UI.Options
+    ( odesc, ocheck, onormalise, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags ( UpdateWorking (..) )
 import Darcs.Repository ( addToPending, withRepoLock, RepoJob(..) )
 import Darcs.Patch ( changepref )
 import Darcs.Patch.Witnesses.Ordered ( FL(..) )
-import Darcs.Repository.Prefs ( getPrefval, changePrefval, )
+import Darcs.Repository.Prefs ( getPrefval, changePrefval )
 import Darcs.Util.English ( orClauses )
 import Darcs.Util.Path ( AbsolutePath )
-#include "impossible.h"
 
 -- | A list of all valid preferences for @_darcs/prefs/prefs@.
 validPrefData :: [(String, String)] -- ^ (name, one line description)
@@ -77,28 +75,6 @@ setprefHelp =
  "the repository will always take precedence.  This is considered a\n" ++
  "low-priority bug, because preferences are seldom set.\n"
 
-setprefBasicOpts :: PrimDarcsOption (Maybe String)
-setprefBasicOpts = O.workingRepoDir
-
-setprefAdvancedOpts :: PrimDarcsOption O.UMask
-setprefAdvancedOpts = O.umask
-
-setprefOpts :: DarcsOption a
-               (Maybe String
-                -> Maybe O.StdCmdAction
-                -> Bool
-                -> Bool
-                -> O.Verbosity
-                -> Bool
-                -> O.UMask
-                -> O.UseCache
-                -> Maybe String
-                -> Bool
-                -> Maybe String
-                -> Bool
-                -> a)
-setprefOpts = setprefBasicOpts `withStdOpts` setprefAdvancedOpts
-
 setpref :: DarcsCommand [DarcsFlag]
 setpref = DarcsCommand
     { commandProgramName = "darcs"
@@ -109,7 +85,7 @@ setpref = DarcsCommand
     , commandExtraArgHelp = ["<PREF>", "<VALUE>"]
     , commandCommand = setprefCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = return validPrefs
+    , commandCompleteArgs = completeArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc setprefAdvancedOpts
     , commandBasicOptions = odesc setprefBasicOpts
@@ -117,13 +93,19 @@ setpref = DarcsCommand
     , commandCheckOptions = ocheck setprefOpts
     , commandParseOptions = onormalise setprefOpts
     }
+  where
+    setprefBasicOpts = O.repoDir
+    setprefAdvancedOpts = O.umask
+    setprefOpts = setprefBasicOpts `withStdOpts` setprefAdvancedOpts
+    completeArgs _ _ [] = return validPrefs
+    completeArgs _ _ _args = return []
 
 setprefCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 setprefCmd _ opts [pref,val] =
- withRepoLock (dryRun opts) (useCache opts) YesUpdateWorking (umask opts) $ RepoJob $ \repository -> do
+ withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $ \repository -> do
   when (' ' `elem` pref) $ do
     putStrLn $ "'"++pref++
-               "' is not a valid preference name:  no spaces allowed!"
+               "' is not a valid preference name: no spaces allowed!"
     exitWith $ ExitFailure 1
   when (pref `notElem` validPrefs) $ do
     putStrLn $ "'"++pref++"' is not a valid preference name!"
