@@ -27,14 +27,12 @@ import Data.Maybe ( fromMaybe )
 import System.IO ( stderr, hPutStrLn )
 
 import Darcs.Patch.Set ( PatchSet(..) )
-import Darcs.Repository ( readRepo, withRepositoryDirectory, RepoJob(..) )
+import Darcs.Repository ( readRepo, withRepositoryLocation, RepoJob(..) )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, findRepository )
-import Darcs.UI.Commands.Tag ( getTags )
+import Darcs.UI.Commands.Util ( repoTags )
+import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags ( DarcsFlag, useCache, getRepourl )
-import Darcs.UI.Options
-    ( DarcsOption, PrimDarcsOption
-    , oid, odesc, ocheck, onormalise, defaultFlags
-    )
+import Darcs.UI.Options ( oid, odesc, ocheck, onormalise, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Text ( formatText )
 import Darcs.Util.Path ( AbsolutePath )
@@ -51,24 +49,6 @@ showTagsHelp = formatText 80
       ++ "if this happens."
     ]
 
-showTagsBasicOpts :: PrimDarcsOption (Maybe String)
-showTagsBasicOpts = O.possiblyRemoteRepo
-
-showTagsOpts :: DarcsOption a
-                (Maybe String
-                 -> Maybe O.StdCmdAction
-                 -> Bool
-                 -> Bool
-                 -> O.Verbosity
-                 -> Bool
-                 -> O.UseCache
-                 -> Maybe String
-                 -> Bool
-                 -> Maybe String
-                 -> Bool
-                 -> a)
-showTagsOpts = showTagsBasicOpts `withStdOpts` oid
-
 showTags :: DarcsCommand [DarcsFlag]
 showTags = DarcsCommand
     { commandProgramName = "darcs"
@@ -79,7 +59,7 @@ showTags = DarcsCommand
     , commandExtraArgHelp = []
     , commandCommand = tagsCmd
     , commandPrereq = findRepository
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = []
     , commandBasicOptions = odesc showTagsBasicOpts
@@ -87,14 +67,17 @@ showTags = DarcsCommand
     , commandCheckOptions = ocheck showTagsOpts
     , commandParseOptions = onormalise showTagsOpts
     }
+  where
+    showTagsBasicOpts = O.possiblyRemoteRepo
+    showTagsOpts = showTagsBasicOpts `withStdOpts` oid
 
 tagsCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 tagsCmd _ opts _ = let repodir = fromMaybe "." (getRepourl opts) in
-    withRepositoryDirectory (useCache opts) repodir $ RepoJob $ \repo ->
+    withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repo ->
         readRepo repo >>= printTags
   where
     printTags :: PatchSet rt p wW wZ -> IO ()
-    printTags = join . fmap (sequence_ . map process) . getTags
+    printTags = join . fmap (sequence_ . map process) . repoTags
     process :: String -> IO ()
     process t = normalize t t False >>= putStrLn
     normalize :: String -> String -> Bool -> IO String
