@@ -37,14 +37,9 @@ import Darcs.UI.Commands
     , nodefaults
     , putInfo
     , amInHashedRepository )
-import Darcs.UI.Flags ( DarcsFlag( SetScriptsExecutable
-                                  , Linear
-                                  , Backoff
-                                  , Bisect
-                                  , LeaveTestDir
-                                  )
-                      , useCache, verbosity )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags )
+import Darcs.UI.Completion ( noArgs )
+import Darcs.UI.Flags ( DarcsFlag, useCache, verbosity )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Patch.PatchInfoAnd ( hopefully )
 import Darcs.Repository (
@@ -66,24 +61,17 @@ import Darcs.Patch.Witnesses.Ordered
     , mapFL
     , mapRL_RL
     )
-import Darcs.Patch.Conflict ( Conflict )
-import Darcs.Patch.FileHunk ( IsHunk )
 import Darcs.Patch.ApplyMonad ( ApplyMonad )
-import Darcs.Patch.Apply ( ApplyState )
-import Darcs.Patch.Format ( PatchListFormat )
+import Darcs.Patch.Apply ( Apply, ApplyState )
 import Darcs.Patch.Inspect ( PatchInspect )
-import Darcs.Patch.Patchy ( Patchy
-                          , Invert
-                          , Apply
-                          , ShowPatch
-                          )
+import Darcs.Patch.Invert ( Invert )
 import Darcs.Patch ( RepoPatch
                    , apply
                    , description
                    , invert
                    )
 import Darcs.Patch.Named.Wrapped ( WrappedNamed )
-import Darcs.Patch.Set ( newset2RL )
+import Darcs.Patch.Set ( patchSet2RL )
 import Darcs.Util.Printer ( putDocLn
                , text
                )
@@ -131,31 +119,6 @@ testHelp =
   ,"break the test is found at random."
  ]
 
-testBasicOpts :: DarcsOption a
-                 (O.TestStrategy -> O.LeaveTestDir -> Maybe String -> a)
-testBasicOpts = O.testStrategy ^ O.leaveTestDir ^ O.workingRepoDir
-
-testAdvancedOpts :: DarcsOption a (O.SetScriptsExecutable -> a)
-testAdvancedOpts = O.setScriptsExecutable
-
-testOpts :: DarcsOption a
-            (O.TestStrategy
-             -> O.LeaveTestDir
-             -> Maybe String
-             -> Maybe O.StdCmdAction
-             -> Bool
-             -> Bool
-             -> O.Verbosity
-             -> Bool
-             -> O.SetScriptsExecutable
-             -> O.UseCache
-             -> Maybe String
-             -> Bool
-             -> Maybe String
-             -> Bool
-             -> a)
-testOpts = testBasicOpts `withStdOpts` testAdvancedOpts
-
 test :: DarcsCommand [DarcsFlag]
 test = DarcsCommand
     { commandProgramName = "darcs"
@@ -166,7 +129,7 @@ test = DarcsCommand
     , commandExtraArgHelp = ["[[INITIALIZATION]", "COMMAND]"]
     , commandCommand = testCommand
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc testAdvancedOpts
     , commandBasicOptions = odesc testBasicOpts
@@ -174,6 +137,10 @@ test = DarcsCommand
     , commandCheckOptions = ocheck testOpts
     , commandParseOptions = onormalise testOpts
     }
+  where
+    testBasicOpts = O.testStrategy ^ O.leaveTestDir ^ O.repoDir
+    testAdvancedOpts = O.setScriptsExecutable
+    testOpts = testBasicOpts `withStdOpts` testAdvancedOpts
 
 -- | Functions defining a strategy for executing a test
 type Strategy = forall rt p wX wY
@@ -186,11 +153,11 @@ type Strategy = forall rt p wX wY
 
 testCommand :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 testCommand _ opts args =
- withRepository (useCache opts) $ RepoJob $ \repository -> do
+ withRepository (useCache ? opts) $ RepoJob $ \repository -> do
   patches <- readRepo repository
   (init,testCmd) <- case args of
     [] ->
-      do t <- getTest (verbosity opts)
+      do t <- getTest (verbosity ? opts)
          return (return ExitSuccess, t)
     [cmd] ->
       do putStrLn $ "Using test command:\n"++cmd
@@ -200,21 +167,22 @@ testCommand _ opts args =
          putStrLn $ "Using test command:\n"++cmd
          return (system init, system cmd)
     _ -> fail "Test expects zero to two arguments."
-  let wd = if LeaveTestDir `elem` opts then withPermDir else withTempDir
+  let wd = case O.leaveTestDir ? opts of
+            O.YesLeaveTestDir -> withPermDir
+            O.NoLeaveTestDir -> withTempDir
   withRecorded repository (wd "testing") $ \_ -> do
-    when (SetScriptsExecutable `elem` opts) setScriptsExecutable
+    when (O.yes (O.setScriptsExecutable ? opts)) setScriptsExecutable
     _ <- init
     putInfo opts $ text "Running test...\n"
     testResult <- testCmd
-    let track = chooseStrategy opts
-    track opts testCmd testResult (mapRL_RL hopefully . newset2RL $ patches)
+    let track = chooseStrategy (O.testStrategy ? opts)
+    track opts testCmd testResult (mapRL_RL hopefully . patchSet2RL $ patches)
 
-chooseStrategy :: [DarcsFlag] -> Strategy
-chooseStrategy opts
-    | Bisect `elem` opts = trackBisect
-    | Linear `elem` opts = trackLinear
-    | Backoff `elem` opts = trackBackoff
-    | otherwise = oneTest
+chooseStrategy :: O.TestStrategy -> Strategy
+chooseStrategy O.Bisect = trackBisect
+chooseStrategy O.Linear = trackLinear
+chooseStrategy O.Backoff = trackBackoff
+chooseStrategy O.Once = oneTest
 
 -- | test only the last recorded state
 oneTest :: Strategy
@@ -229,7 +197,7 @@ trackLinear _ _ ExitSuccess _ = putStrLn "Success!"
 trackLinear opts testCmd (ExitFailure _) (ps:<:p) = do
     let ip = invert p
     safeApply ip
-    when (SetScriptsExecutable `elem` opts) $ setScriptsExecutablePatches ip
+    when (O.yes (O.setScriptsExecutable ? opts)) $ setScriptsExecutablePatches ip
     putStrLn "Trying without the patch:"
     putDocLn $ description ip
     hFlush stdout
@@ -260,7 +228,7 @@ trackNextBackoff opts testCmd n ahead = do
     case splitAtRL n ahead of
         ( ahead' :> skipped' ) -> do
             unapplyRL skipped'
-            when (SetScriptsExecutable `elem` opts) $ setScriptsExecutablePatches skipped'
+            when (O.yes (O.setScriptsExecutable ? opts)) $ setScriptsExecutablePatches skipped'
             testResult <- testCmd
             case testResult of
                 ExitFailure _ ->
@@ -300,13 +268,13 @@ data BisectDir = BisectLeft | BisectRight deriving Show
 type BisectState = (Int, Int)
 
 -- | Create Bisect PatchTree from the RL
-patchTreeFromRL :: (Patchy p) => RL p wX wY -> PatchTree p wX wY
+patchTreeFromRL :: RL p wX wY -> PatchTree p wX wY
 patchTreeFromRL (NilRL :<: l) = Leaf l
 patchTreeFromRL xs = case splitAtRL (lengthRL xs `div` 2) xs of
                        (r :> l) -> Fork (patchTreeFromRL l) (patchTreeFromRL r)
 
 -- | Convert PatchTree back to RL
-patchTree2RL :: (Patchy p) => PatchTree p wX wY -> RL p wX wY
+patchTree2RL :: PatchTree p wX wY -> RL p wX wY
 patchTree2RL (Leaf p)   = NilRL :<: p
 patchTree2RL (Fork l r) = patchTree2RL r +<+ patchTree2RL l
 
@@ -334,31 +302,29 @@ trackNextBisect _ _ _ _ (Leaf p) = do
   putStrLn "Last recent patch that fails the test (assuming monotony in the given range):"
   putDocLn (description p)
 
-jumpHalfOnRight :: (IsHunk p, Conflict p,
-                    PatchListFormat p, ShowPatch p, PatchInspect p,
-                    Patchy p, ApplyMonad (ApplyState p) DefaultIO)
+jumpHalfOnRight :: (Invert p, Apply p, PatchInspect p,
+                    ApplyMonad (ApplyState p) DefaultIO)
                 => [DarcsFlag] -> PatchTree p wX wY -> IO ()
 jumpHalfOnRight opts l = do unapplyRL ps
-                            when (SetScriptsExecutable `elem` opts) $ setScriptsExecutablePatches ps
+                            when (O.yes (O.setScriptsExecutable ? opts)) $ setScriptsExecutablePatches ps
   where ps = patchTree2RL l
 
-jumpHalfOnLeft :: (IsHunk p, Conflict p,
-                   PatchListFormat p, ShowPatch p, PatchInspect p,
-                   Patchy p, ApplyMonad (ApplyState p) DefaultIO)
+jumpHalfOnLeft :: (Apply p, PatchInspect p,
+                   ApplyMonad (ApplyState p) DefaultIO)
                => [DarcsFlag] -> PatchTree p wX wY -> IO ()
 jumpHalfOnLeft opts r = do applyRL p
-                           when (SetScriptsExecutable `elem` opts) $ setScriptsExecutablePatches p
+                           when (O.yes (O.setScriptsExecutable ? opts)) $ setScriptsExecutablePatches p
 
   where p = patchTree2RL r
 
-applyRL :: (Invert p, ShowPatch p, Apply p, ApplyMonad (ApplyState p) DefaultIO)
+applyRL :: (Apply p, ApplyMonad (ApplyState p) DefaultIO)
         => RL p wX wY -> IO ()
 applyRL   patches = sequence_ (mapFL safeApply (reverseRL patches))
 
-unapplyRL :: (Invert p, ShowPatch p, Apply p, ApplyMonad (ApplyState p) DefaultIO)
+unapplyRL :: (Invert p, Apply p, ApplyMonad (ApplyState p) DefaultIO)
            => RL p wX wY -> IO ()
 unapplyRL patches = sequence_ (mapRL (safeApply . invert) patches)
 
-safeApply :: (Invert p, ShowPatch p, Apply p, ApplyMonad (ApplyState p) DefaultIO)
+safeApply :: (Apply p, ApplyMonad (ApplyState p) DefaultIO)
           => p wX wY -> IO ()
 safeApply p = runDefault (apply p) `catch` \(msg :: IOException) -> fail $ "Bad patch:\n" ++ show msg
