@@ -27,8 +27,9 @@ import System.IO ( stdout, hFlush )
 import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Exception ( prettyException )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInRepository )
+import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags ( DarcsFlag )
-import Darcs.UI.Options ( DarcsOption, oid, odesc, ocheck, onormalise, defaultFlags )
+import Darcs.UI.Options ( oid, odesc, ocheck, onormalise, defaultFlags )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Progress ( setProgressMode )
 import Darcs.Util.Global ( darcsdir )
@@ -52,24 +53,6 @@ transferModeHelp =
  "who do not run ssh-agent will be prompted for the ssh password tens or\n" ++
  "hundreds of times!\n"
 
-transferModeBasicOpts :: DarcsOption a (Maybe String -> a)
-transferModeBasicOpts = O.workingRepoDir
-
-transferModeOpts :: DarcsOption a
-                    (Maybe String
-                     -> Maybe O.StdCmdAction
-                     -> Bool
-                     -> Bool
-                     -> O.Verbosity
-                     -> Bool
-                     -> O.UseCache
-                     -> Maybe String
-                     -> Bool
-                     -> Maybe String
-                     -> Bool
-                     -> a)
-transferModeOpts = transferModeBasicOpts `withStdOpts` oid
-
 transferMode :: DarcsCommand [DarcsFlag]
 transferMode = DarcsCommand
     { commandProgramName = "darcs"
@@ -78,7 +61,7 @@ transferMode = DarcsCommand
     , commandDescription = transferModeDescription
     , commandExtraArgs = 0
     , commandExtraArgHelp = []
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandCommand = transferModeCmd
     , commandPrereq = amInRepository
     , commandArgdefaults = nodefaults
@@ -88,6 +71,9 @@ transferMode = DarcsCommand
     , commandCheckOptions = ocheck transferModeOpts
     , commandParseOptions = onormalise transferModeOpts
     }
+  where
+    transferModeBasicOpts = O.repoDir
+    transferModeOpts = transferModeBasicOpts `withStdOpts` oid
 
 transferModeCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 transferModeCmd _ _ _ = do setProgressMode False
