@@ -15,7 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Darcs.UI.Commands.Unrecord
     ( unrecord
@@ -31,17 +31,16 @@ import Darcs.Prelude
 import Prelude hiding ( (^) )
 
 import Control.Exception ( catch, IOException )
-import Control.Monad ( when, unless )
-import Data.Maybe( isJust, mapMaybe )
-import Data.List ( intercalate )
+import Control.Monad ( when )
+import Data.Maybe( isJust )
 import Darcs.Util.Tree( Tree )
 import System.Exit ( exitSuccess )
 
 import Darcs.Patch ( IsRepoType, RepoPatch, invert, commute, effect )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Bundle ( makeBundleN, contextPatches, minContext )
-import Darcs.Patch.Depends ( findCommonWithThem, newsetUnion )
-import Darcs.Patch.Match ( firstMatch, matchFirstPatchset, matchAPatchread, MatchFlag )
+import Darcs.Patch.Depends ( findCommonWithThem, patchSetUnion )
+import Darcs.Patch.Match ( firstMatch, matchFirstPatchset, matchAPatch, MatchFlag )
 import Darcs.Patch.PatchInfoAnd ( hopefully, patchDesc )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), appendPSFL, Origin,
                          SealedPatchSet )
@@ -59,22 +58,23 @@ import Darcs.Repository.Flags( UseIndex(..), ScanKnown(..), UpdateWorking(..), D
 import Darcs.Util.Lock( writeDocBinFile )
 import Darcs.Repository.Prefs ( getDefaultRepoPath )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias
-                         , putVerbose, printDryRunMessageAndExit
+                         , putVerbose
                          , setEnvDarcsPatches, amInHashedRepository
                          , putInfo )
-import Darcs.UI.Commands.Util ( getUniqueDPatchName )
+import Darcs.UI.Commands.Util ( getUniqueDPatchName, printDryRunMessageAndExit )
+import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags
-    ( doReverse, compression, verbosity, getOutput
-    , useCache, dryRun, umask, DarcsFlag ( NotInRemote ), minimize
-    , diffAlgorithm, hasXmlOutput, hasSummary, isInteractive, selectDeps )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+    ( DarcsFlag, changesReverse, compress, verbosity, getOutput
+    , useCache, dryRun, umask, minimize
+    , diffAlgorithm, xmlOutput, isInteractive, selectDeps )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import Darcs.UI.Options.All ( notInRemoteFlagName )
 import qualified Darcs.UI.Options.All as O
 import Darcs.UI.SelectChanges ( WhichChanges(..),
                                 selectionContext, runSelection )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Util.English ( presentParticiple )
-import Darcs.Util.Printer ( text, putDoc, (<>), (<+>) )
+import Darcs.Util.Printer ( text, putDoc, vcat, (<>), (<+>), ($$) )
 import Darcs.Util.Progress ( debugMessage )
 
 unrecordDescription :: String
@@ -91,41 +91,6 @@ unrecordHelp = unlines
  , "possibility that another user may have already pulled the patch."
  ]
 
-unrecordBasicOpts :: DarcsOption a
-                     ([MatchFlag] -> O.SelectDeps -> Maybe Bool -> Maybe String -> a)
-unrecordBasicOpts
-    = O.matchSeveralOrLast
-    ^ O.selectDeps
-    ^ O.interactive -- True
-    ^ O.workingRepoDir
-
-unrecordAdvancedOpts :: DarcsOption a (O.Compression -> O.UMask -> Bool -> a)
-unrecordAdvancedOpts
-    = O.compress
-    ^ O.umask
-    ^ O.changesReverse
-
-unrecordOpts :: DarcsOption a
-                ([MatchFlag]
-                 -> O.SelectDeps
-                 -> Maybe Bool
-                 -> Maybe String
-                 -> Maybe O.StdCmdAction
-                 -> Bool
-                 -> Bool
-                 -> O.Verbosity
-                 -> Bool
-                 -> O.Compression
-                 -> O.UMask
-                 -> Bool
-                 -> O.UseCache
-                 -> Maybe String
-                 -> Bool
-                 -> Maybe String
-                 -> Bool
-                 -> a)
-unrecordOpts = unrecordBasicOpts `withStdOpts` unrecordAdvancedOpts
-
 unrecord :: DarcsCommand [DarcsFlag]
 unrecord = DarcsCommand
     { commandProgramName = "darcs"
@@ -136,7 +101,7 @@ unrecord = DarcsCommand
     , commandExtraArgHelp = []
     , commandCommand = unrecordCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc unrecordAdvancedOpts
     , commandBasicOptions = odesc unrecordBasicOpts
@@ -144,20 +109,27 @@ unrecord = DarcsCommand
     , commandCheckOptions = ocheck unrecordOpts
     , commandParseOptions = onormalise unrecordOpts
     }
+  where
+    unrecordBasicOpts
+      = O.notInRemote
+      ^ O.matchSeveralOrLast
+      ^ O.selectDeps
+      ^ O.interactive -- True
+      ^ O.repoDir
+    unrecordAdvancedOpts
+      = O.compress
+      ^ O.umask
+      ^ O.changesReverse
+    unrecordOpts = unrecordBasicOpts `withStdOpts` unrecordAdvancedOpts
 
 unrecordCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 unrecordCmd _ opts _ =
-    withRepoLock NoDryRun (useCache opts) YesUpdateWorking (umask opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
         RepoJob $ \repository -> do
-            allpatches <- readRepo repository
-            let matchFlags = parseFlags O.matchSeveralOrLast opts
-            (_ :> patches) <- return $
-                if firstMatch matchFlags
-                    then getLastPatches matchFlags allpatches
-                    else matchingHead matchFlags allpatches
-            let direction = if doReverse opts then Last else LastReversed
+            (_ :> removal_candidates) <- preselectPatches opts repository
+            let direction = if changesReverse ? opts then Last else LastReversed
                 context = selectionContext direction "unrecord" (patchSelOpts opts) Nothing Nothing
-            (_ :> to_unrecord) <- runSelection patches context
+            (_ :> to_unrecord) <- runSelection removal_candidates context
             when (nullFL to_unrecord) $ do
                 putInfo opts "No patches selected!"
                 exitSuccess
@@ -165,9 +137,9 @@ unrecordCmd _ opts _ =
                 text "About to write out (potentially) modified patches..."
             setEnvDarcsPatches to_unrecord
             invalidateIndex repository
-            _ <- tentativelyRemovePatches repository (compression opts)
+            _ <- tentativelyRemovePatches repository (compress ? opts)
                      YesUpdateWorking to_unrecord
-            finalizeRepositoryChanges repository YesUpdateWorking (compression opts)
+            finalizeRepositoryChanges repository YesUpdateWorking (compress ? opts)
             putInfo opts "Finished unrecording."
 
 getLastPatches :: (IsRepoType rt, RepoPatch p) => [MatchFlag] -> PatchSet rt p Origin wR
@@ -218,68 +190,6 @@ obliterateHelp = unlines
  , "later to your repository with `darcs apply`."
  ]
 
-obliterateBasicOpts :: DarcsOption a
-                       ([Maybe String]
-                        -> [MatchFlag]
-                        -> O.SelectDeps
-                        -> Maybe Bool
-                        -> Maybe String
-                        -> Maybe O.Summary
-                        -> Maybe O.Output
-                        -> Bool
-                        -> O.DiffAlgorithm
-                        -> O.DryRun
-                        -> O.XmlOutput
-                        -> a)
-obliterateBasicOpts
-    = O.notInRemote
-    ^ O.matchSeveralOrLast
-    ^ O.selectDeps
-    ^ O.interactive
-    ^ O.workingRepoDir
-    ^ O.summary
-    ^ O.output
-    ^ O.minimize
-    ^ O.diffAlgorithm
-    ^ O.dryRunXml
-
-obliterateAdvancedOpts :: DarcsOption a
-                          (O.Compression -> O.UseIndex -> O.UMask -> Bool -> a)
-obliterateAdvancedOpts
-    = O.compress
-    ^ O.useIndex
-    ^ O.umask
-    ^ O.changesReverse
-
-obliterateOpts :: DarcsOption a
-                  ([Maybe String]
-                   -> [MatchFlag]
-                   -> O.SelectDeps
-                   -> Maybe Bool
-                   -> Maybe String
-                   -> Maybe O.Summary
-                   -> Maybe O.Output
-                   -> Bool
-                   -> O.DiffAlgorithm
-                   -> DryRun
-                   -> O.XmlOutput
-                   -> Maybe O.StdCmdAction
-                   -> Bool
-                   -> Bool
-                   -> O.Verbosity
-                   -> Bool
-                   -> O.Compression
-                   -> UseIndex
-                   -> O.UMask
-                   -> Bool
-                   -> O.UseCache
-                   -> Maybe String
-                   -> Bool
-                   -> Maybe String
-                   -> Bool
-                   -> a)
-obliterateOpts = obliterateBasicOpts `withStdOpts` obliterateAdvancedOpts
-
 obliterate :: DarcsCommand [DarcsFlag]
 obliterate = DarcsCommand
     { commandProgramName = "darcs"
@@ -290,7 +200,7 @@ obliterate = DarcsCommand
     , commandExtraArgHelp = []
     , commandCommand = obliterateCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc obliterateAdvancedOpts
     , commandBasicOptions = odesc obliterateBasicOpts
@@ -298,13 +208,28 @@ obliterate = DarcsCommand
     , commandCheckOptions = ocheck obliterateOpts
     , commandParseOptions = onormalise obliterateOpts
     }
+  where
+    obliterateBasicOpts
+      = O.notInRemote
+      ^ O.matchSeveralOrLast
+      ^ O.selectDeps
+      ^ O.interactive
+      ^ O.repoDir
+      ^ O.summary
+      ^ O.output
+      ^ O.minimize
+      ^ O.diffAlgorithm
+      ^ O.dryRunXml
+    obliterateAdvancedOpts
+      = O.compress
+      ^ O.useIndex
+      ^ O.umask
+      ^ O.changesReverse
+    obliterateOpts = obliterateBasicOpts `withStdOpts` obliterateAdvancedOpts
 
 obliterateCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 obliterateCmd = genericObliterateCmd "obliterate"
 
-data NotInRemoteLocation = NotInDefaultRepo
-                         | NotInRemotePath String
-
 -- | genericObliterateCmd is the function that executes the "obliterate" and
 -- "unpull" commands. The first argument is the name under which the command is
 -- invoked (@unpull@ or @obliterate@).
@@ -314,31 +239,17 @@ genericObliterateCmd :: String
                      -> [String]
                      -> IO ()
 genericObliterateCmd cmdname _ opts _ =
-    let cacheOpt = useCache opts
-        verbOpt = verbosity opts
-    in withRepoLock (dryRun opts) cacheOpt YesUpdateWorking (umask opts) $
+    let cacheOpt = useCache ? opts
+        verbOpt = verbosity ? opts
+    in withRepoLock (dryRun ? opts) cacheOpt YesUpdateWorking (umask ? opts) $
         RepoJob $ \repository -> do
             -- FIXME we may need to honour --ignore-times here, although this
             -- command does not take that option (yet)
-            pend <- unrecordedChanges (UseIndex, ScanKnown, diffAlgorithm opts) repository Nothing
-            allpatches <- readRepo repository
-            let collectNotIns (NotInRemote nir) = case nir of
-                    Just p -> Just $ NotInRemotePath p
-                    Nothing -> Just NotInDefaultRepo
-                collectNotIns _ = Nothing
-                notIns = mapMaybe collectNotIns opts
-            (auto_kept :> removal_candidates) <- case notIns of
-                [] -> do
-                    let matchFlags = parseFlags O.matchSeveralOrLast opts
-                    return $ if firstMatch matchFlags
-                                   then getLastPatches matchFlags allpatches
-                                   else matchingHead matchFlags allpatches
-                nirs -> do
-                    (Sealed thems) <-
-                      getNotInRemotePatches verbOpt cacheOpt repository nirs
-                    return $ findCommonWithThem allpatches thems
+            pend <- unrecordedChanges (UseIndex, ScanKnown, diffAlgorithm ? opts)
+              O.NoLookForMoves O.NoLookForReplaces repository Nothing
+            (auto_kept :> removal_candidates) <- preselectPatches opts repository
 
-            let direction = if doReverse opts then Last else LastReversed
+            let direction = if changesReverse ? opts then Last else LastReversed
                 context = selectionContext direction cmdname (patchSelOpts opts) Nothing Nothing
             (kept :> removed) <-
                 runSelection removal_candidates context
@@ -352,9 +263,9 @@ genericObliterateCmd cmdname _ opts _ =
                 Just (_ :> p_after_pending) -> do
                     printDryRunMessageAndExit "obliterate"
                       verbOpt
-                      (hasSummary O.NoSummary opts)
-                      (dryRun opts)
-                      (hasXmlOutput opts)
+                      (O.summary ? opts)
+                      (dryRun ? opts)
+                      (xmlOutput ? opts)
                       (isInteractive True opts)
                       removed
                     setEnvDarcsPatches removed
@@ -362,11 +273,11 @@ genericObliterateCmd cmdname _ opts _ =
                         savetoBundle opts (auto_kept `appendPSFL` kept) removed
                     invalidateIndex repository
                     _ <- tentativelyRemovePatches repository
-                        (compression opts) YesUpdateWorking removed
+                        (compress ? opts) YesUpdateWorking removed
                     tentativelyAddToPending repository
                         YesUpdateWorking $ invert $ effect removed
                     finalizeRepositoryChanges repository
-                        YesUpdateWorking (compression opts)
+                        YesUpdateWorking (compress ? opts)
                     debugMessage "Applying patches to working directory..."
                     _ <- applyToWorking repository verbOpt
                         (invert p_after_pending)
@@ -376,30 +287,27 @@ genericObliterateCmd cmdname _ opts _ =
                     putInfo opts $ "Finished" <+> text (presentParticiple cmdname) <> "."
 
 -- | Get the union of the set of patches in each specified location
-getNotInRemotePatches :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                      => O.Verbosity -> O.UseCache
-                      -> Repository rt p wX wU wT -> [NotInRemoteLocation]
-                      -> IO (SealedPatchSet rt p Origin)
-getNotInRemotePatches verbOpt cacheOpt repository nirs = do
-    unless (verbOpt == O.Quiet) $
-      putStrLn $ "Determining patches not in" ++ pluralExtra ++ ":\n" ++ names
+remotePatches :: (IsRepoType rt, RepoPatch p)
+              => [DarcsFlag]
+              -> Repository rt p wX wU wT -> [O.NotInRemote]
+              -> IO (SealedPatchSet rt p Origin)
+remotePatches opts repository nirs = do
     nirsPaths <- mapM getNotInRemotePath nirs
-    newsetUnion `fmap` mapM readNir nirsPaths
+    putInfo opts $ "Determining patches not in" <+> pluralExtra nirsPaths $$
+        itemize nirsPaths
+    patchSetUnion `fmap` mapM readNir nirsPaths
   where
-    toName (NotInRemotePath s) = "'" ++ s ++ "'"
-    toName NotInDefaultRepo = "Default push/pull repo"
-
-    pluralExtra = if length names > 1 then " any of" else ""
-    names = intercalate "\n" $ map ((leader ++) . toName) nirs
-    leader = "    - "
+    pluralExtra names = if length names > 1 then "any of" else mempty
+    itemize = vcat . map (text . ("  - " ++))
 
     readNir n = do
-        r <- identifyRepositoryFor repository cacheOpt n
+        r <- identifyRepositoryFor repository (useCache ? opts) n
         rps <- readRepo r
         return $ seal rps
 
-    getNotInRemotePath (NotInRemotePath p) = return p
-    getNotInRemotePath NotInDefaultRepo = do
+    getNotInRemotePath :: O.NotInRemote -> IO String
+    getNotInRemotePath (O.NotInRemotePath p) = return p
+    getNotInRemotePath O.NotInDefaultRepo = do
         defaultRepo <- getDefaultRepoPath
         let err = fail $ "No default push/pull repo configured, please pass a "
                          ++ "repo name to --" ++ notInRemoteFlagName
@@ -418,7 +326,7 @@ matchingHead matchFlags set =
     mh :: forall wX . PatchSet rt p Origin wX
        -> (PatchSet rt p :> RL (PatchInfoAnd rt p)) Origin wX
     mh s@(PatchSet _ x)
-        | or (mapRL (matchAPatchread matchFlags) x) = contextPatches s
+        | or (mapRL (matchAPatch matchFlags) x) = contextPatches s
     mh (PatchSet (ts :<: Tagged t _ ps) x) =
         case mh (PatchSet ts (ps :<: t)) of
             (start :> patches) -> start :> patches +<+ x
@@ -428,7 +336,7 @@ savetoBundle :: (RepoPatch p, ApplyState p ~ Tree) => [DarcsFlag]
              -> PatchSet rt p Origin wZ -> FL (PatchInfoAnd rt p) wZ wT -> IO ()
 savetoBundle opts kept removed@(x :>: _) = do
     let genFullBundle = makeBundleN Nothing kept (mapFL_FL hopefully removed)
-    bundle <- if not (minimize opts)
+    bundle <- if not (minimize ? opts)
                then genFullBundle
                else do putInfo opts "Minimizing context, to generate bundle with full context hit ctrl-C..."
                        ( case minContext kept removed of
@@ -441,12 +349,33 @@ savetoBundle opts kept removed@(x :>: _) = do
     useAbsoluteOrStd writeDocBinFile putDoc outname bundle
 savetoBundle _ _ NilFL = return ()
 
+preselectPatches
+  :: (IsRepoType rt, RepoPatch p)
+  => [DarcsFlag]
+  -> Repository rt p wR wU wT
+  -> IO ((PatchSet rt p :> FL (PatchInfoAnd rt p)) Origin wR)
+preselectPatches opts repo = do
+  allpatches <- readRepo repo
+  let matchFlags = parseFlags O.matchSeveralOrLast opts
+  case O.notInRemote ? opts of
+    [] -> do
+      return $
+        if firstMatch matchFlags
+          then getLastPatches matchFlags allpatches
+          else matchingHead matchFlags allpatches
+    -- FIXME what about match options when we have --not-in-remote?
+    -- It looks like they are simply ignored.
+    nirs -> do
+      (Sealed thems) <-
+        remotePatches opts repo nirs
+      return $ findCommonWithThem allpatches thems
+
 patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
-    { S.verbosity = verbosity flags
+    { S.verbosity = verbosity ? flags
     , S.matchFlags = parseFlags O.matchSeveralOrLast flags
     , S.interactive = isInteractive True flags
-    , S.selectDeps = selectDeps flags
-    , S.summary = hasSummary O.NoSummary flags
+    , S.selectDeps = selectDeps ? flags
+    , S.summary = O.summary ? flags
     , S.withContext = O.NoContext
     }
