@@ -1,8 +1,9 @@
-{-# LANGUAGE CPP #-}
 module Darcs.UI.Email
     ( makeEmail
     , readEmail
     , formatHeader
+    -- just for testing
+    , prop_qp_roundtrip
     ) where
 
 import Prelude ()
@@ -11,14 +12,12 @@ import Darcs.Prelude
 import Data.Char ( digitToInt, isHexDigit, ord, intToDigit, isPrint, toUpper )
 import Data.List ( isInfixOf )
 import Darcs.Util.Printer
-    ( Doc, ($$), (<+>), (<>), text, empty, packedString, renderPS
-    , RenderMode(..)
-    )
+    ( Doc, ($$), (<+>), (<>), text, empty, packedString, renderPS )
 
 import Darcs.Util.ByteString ( packStringToUTF8, dropSpace, linesPS, betweenLinesPS )
 import qualified Data.ByteString          as B  (ByteString, length, null, tail
                                                 ,drop, head, concat, singleton
-                                                ,pack, append, empty, unpack
+                                                ,pack, append, empty, unpack, snoc
                                                 )
 import qualified Data.ByteString.Char8    as BC (index, head, pack)
 import Data.ByteString.Internal as B (c2w, createAndTrim)
@@ -131,7 +130,8 @@ qEncodeChar c
         word8ToUDigit :: Word8 -> Char
         word8ToUDigit = toUpper . intToDigit . fromIntegral
 
--- TODO is this doing mime encoding??
+-- Encode a ByteString according to "Quoted Printable" defined by MIME
+-- (https://tools.ietf.org/html/rfc2045#section-6.7)
 qpencode :: B.ByteString -> B.ByteString
 qpencode s = unsafePerformIO
            -- Really only (3 + 2/75) * length or something in the worst case
@@ -143,7 +143,7 @@ encode ps n buf bufi = case B.head ps of
   c | c == newline ->
         do poke (buf `plusPtr` bufi) newline
            encode ps' qlineMax buf (bufi+1)
-    | n == 0 && B.length ps > 1 ->
+    | n == 0 && B.length ps >= 1 ->
         do poke (buf `plusPtr` bufi) equals
            poke (buf `plusPtr` (bufi+1)) newline
            encode ps qlineMax buf (bufi + 2)
@@ -220,7 +220,7 @@ makeEmail repodir headers mcontents mcharset bundle mfilename =
               fromMaybe "x-unknown" mcharset ++ "\"")
          $$ text "Content-Transfer-Encoding: quoted-printable"
          $$ text ""
-         $$ packedString (qpencode (renderPS Standard contents))
+         $$ packedString (qpencode (renderPS contents))
          $$ text ""
          $$ text "--=_"
        Nothing -> empty)
@@ -229,10 +229,10 @@ makeEmail repodir headers mcontents mcharset bundle mfilename =
  $$ text "Content-Transfer-Encoding: quoted-printable"
  $$ text "Content-Description: Patch preview"
  $$ text ""
- $$ (case betweenLinesPS (BC.pack "New patches:") (BC.pack "Context:") (renderPS Standard bundle) of
+ $$ (case betweenLinesPS (BC.pack "New patches:") (BC.pack "Context:") (renderPS bundle) of
      Just s -> packedString $ qpencode s
      -- this should not happen, but in case it does, keep everything
-     Nothing -> packedString $ qpencode $ renderPS Standard bundle)
+     Nothing -> packedString $ qpencode $ renderPS bundle)
  $$ text "--=_"
  $$ text "Content-Type: application/x-darcs-patch" <>
       (case mfilename of
@@ -242,7 +242,7 @@ makeEmail repodir headers mcontents mcharset bundle mfilename =
  $$ text "Content-Disposition: attachment"
  $$ text "Content-Description: A darcs patch for your repository!"
  $$ text ""
- $$ packedString (qpencode (renderPS Standard bundle))
+ $$ packedString (qpencode (renderPS bundle))
  $$ text "--=_--"
  $$ text ""
  $$ text "."
@@ -257,3 +257,6 @@ readEmail s =
     Nothing -> s -- if it wasn't an email in the first place, just pass along.
     Just s' -> qpdecode s'
 
+-- note: qpdecode appends an extra '\n'
+prop_qp_roundtrip :: B.ByteString -> Bool
+prop_qp_roundtrip s = B.snoc s 10 == (qpdecode . qpencode) s
