@@ -28,6 +28,7 @@ module Darcs.UI.External
 
 import Prelude ()
 import Darcs.Prelude
+import Darcs.Util.Text ( showCommandLine )
 
 import Data.Maybe ( isJust, isNothing, maybeToList )
 import Control.Monad ( unless, when, filterM, liftM2, void )
@@ -35,11 +36,7 @@ import GHC.MVar ( MVar )
 import System.Exit ( ExitCode(..) )
 import System.Environment
     ( getEnv
-#if __GLASGOW_HASKELL__ >= 706
     , getExecutablePath
-#else
-    , getProgName
-#endif
     )
 import System.IO ( hPutStr, hPutStrLn, hClose,
                    hIsTerminalDevice, stdout, stderr, Handle )
@@ -49,7 +46,10 @@ import System.FilePath.Posix ( (</>) )
 import System.Process ( createProcess, proc, CreateProcess(..), runInteractiveProcess, waitForProcess, StdStream(..) )
 import System.Process.Internals ( ProcessHandle )
 
-import GHC.IO.Encoding ( setFileSystemEncoding, setForeignEncoding, char8 )
+import GHC.IO.Encoding
+    ( getFileSystemEncoding
+    , setForeignEncoding
+    , setLocaleEncoding )
 
 import Foreign.C.String ( CString, peekCString )
 
@@ -95,7 +95,7 @@ import Darcs.Util.URL ( SshFilePath, sshUhost )
 import Darcs.Util.Printer ( Doc, Printers, hPutDocLnWith, hPutDoc, hPutDocLn, hPutDocWith, ($$), renderPS,
                  simplePrinters,
                  hPutDocCompr,
-                 text, empty, packedString, vcat, renderString, RenderMode(..) )
+                 text, empty, packedString, vcat, renderString )
 import qualified Darcs.Util.Ratified as Ratified
 import Darcs.UI.Email ( formatHeader )
 
@@ -116,28 +116,24 @@ diffProgram = do
 
 -- |Get the name of the darcs executable (as supplied by @getExecutablePath@)
 darcsProgram :: IO String
-#if __GLASGOW_HASKELL__ >= 706
 darcsProgram = getExecutablePath
-#else
-darcsProgram = getProgName
-#endif
 
-pipeDoc :: RenderMode -> String -> [String] -> Doc -> IO ExitCode
+pipeDoc :: String -> [String] -> Doc -> IO ExitCode
 pipeDoc = pipeDocInternal (PipeToOther simplePrinters)
 
 data WhereToPipe = PipeToSsh Compression -- ^ if pipe to ssh, can choose to compress or not
                  | PipeToOther Printers  -- ^ otherwise, can specify printers
 
-pipeDocInternal :: WhereToPipe -> RenderMode -> String -> [String] -> Doc -> IO ExitCode
-pipeDocInternal whereToPipe target c args inp = withoutNonBlock $ withoutProgress $
-    do debugMessage $ unwords (c:args)
+pipeDocInternal :: WhereToPipe -> String -> [String] -> Doc -> IO ExitCode
+pipeDocInternal whereToPipe c args inp = withoutNonBlock $ withoutProgress $
+    do debugMessage $ "Exec: " ++ showCommandLine (c:args)
        (Just i,_,_,pid) <- createProcess (proc c args){ std_in = CreatePipe
-                                                   {- , delegate_ctlc = True -- requires process 1.2.2.0 -} }
+                                                      , delegate_ctlc = True}
        debugMessage "Start transferring data"
        case whereToPipe of
-          PipeToSsh GzipCompression -> hPutDocCompr target i inp
-          PipeToSsh NoCompression   -> hPutDoc target i inp
-          PipeToOther printers      -> hPutDocWith printers target i inp
+          PipeToSsh GzipCompression -> hPutDocCompr i inp
+          PipeToSsh NoCompression   -> hPutDoc i inp
+          PipeToOther printers      -> hPutDocWith printers i inp
        hClose i
        rval <- waitForProcess pid
        debugMessage "Finished transferring data"
@@ -145,10 +141,10 @@ pipeDocInternal whereToPipe target c args inp = withoutNonBlock $ withoutProgres
             putStrLn $ "Command not found:\n   "++ show (c:args)
        return rval
 
-pipeDocSSH :: Compression -> RenderMode -> SshFilePath -> [String] -> Doc -> IO ExitCode
-pipeDocSSH compress target remoteAddr args input = do
+pipeDocSSH :: Compression -> SshFilePath -> [String] -> Doc -> IO ExitCode
+pipeDocSSH compress remoteAddr args input = do
     (ssh, ssh_args) <- getSSH SSH
-    pipeDocInternal (PipeToSsh compress) target ssh (ssh_args++ (sshUhost remoteAddr:args)) input
+    pipeDocInternal (PipeToSsh compress) ssh (ssh_args ++ ("--":sshUhost remoteAddr:args)) input
 
 sendEmail :: String -> String -> String -> String -> String -> String -> IO ()
 sendEmail f t s cc scmd body =
@@ -169,7 +165,7 @@ generateEmail h f t s cc body = do
      putHeader "Subject" s
      unless (null cc) $ putHeader "Cc" cc
      putHeader "X-Mail-Originator" "Darcs Version Control System"
-     hPutDocLn Standard h body
+     hPutDocLn h body
   where putHeader field value
             = B.hPut h (B.append (formatHeader field value) newline)
         newline = B.singleton 10
@@ -200,10 +196,10 @@ sendEmailDoc f t s cc scmd mbundle body = do
       withOpenTemp $ \(hat,at) -> do
         ftable' <- case mbundle of
                    Just (content,bundle) -> do
-                       hPutDocLn Standard hat bundle
-                       return [ ('b', renderString Standard content) , ('a', at) ]
+                       hPutDocLn hat bundle
+                       return [ ('b', renderString content) , ('a', at) ]
                    Nothing ->
-                       return [ ('b', renderString Standard body) ]
+                       return [ ('b', renderString body) ]
         hClose hat
         let ftable = [ ('t',addressOnly t),('c',cc),('f',f),('s',s) ] ++ ftable'
         r <- execSendmail ftable scmd fn
@@ -217,7 +213,7 @@ sendEmailDoc f t s cc scmd mbundle body = do
             withCString cc $ \ccp ->
              withCString s $ \sp ->
               withOpenTemp $ \(h,fn) -> do
-               hPutDoc Standard h body
+               hPutDoc h body
                hClose h
                writeDocBinFile "mailed_patch" body
                cfn <- canonFilename fn
@@ -245,7 +241,7 @@ resendEmail t scmd body = do
      hPutStrLn h $ "To: "++ t
      hPutStrLn h $ find_from (linesPS body)
      hPutStrLn h $ find_subject (linesPS body)
-     hPutDocLn Standard h $ fixit $ linesPS body
+     hPutDocLn h $ fixit $ linesPS body
      hClose h
      let ftable = [('t',t)]
      r <-  execSendmail ftable scmd fn
@@ -299,15 +295,15 @@ foreign import ccall "win32/send_email.h send_email" c_send_email
 #endif
 
 execPSPipe :: String -> [String] -> B.ByteString -> IO B.ByteString
-execPSPipe c args ps = fmap (renderPS Standard)
-                     $ execDocPipe Standard c args
+execPSPipe c args ps = fmap renderPS
+                     $ execDocPipe c args
                      $ packedString ps
 
-execAndGetOutput :: RenderMode -> FilePath -> [String] -> Doc
+execAndGetOutput :: FilePath -> [String] -> Doc
                  -> IO (ProcessHandle, MVar (), B.ByteString)
-execAndGetOutput target c args instr = do
+execAndGetOutput c args instr = do
        (i,o,e,pid) <- runInteractiveProcess c args Nothing Nothing
-       _ <- forkIO $ hPutDoc target i instr >> hClose i
+       _ <- forkIO $ hPutDoc i instr >> hClose i
        mvare <- newEmptyMVar
        _ <- forkIO ((Ratified.hGetContents e >>= -- ratify: immediately consumed
                 hPutStr stderr)
@@ -315,9 +311,9 @@ execAndGetOutput target c args instr = do
        out <- B.hGetContents o
        return (pid, mvare, out)
 
-execDocPipe :: RenderMode -> String -> [String] -> Doc -> IO Doc
-execDocPipe target c args instr = withoutProgress $ do
-       (pid, mvare, out) <- execAndGetOutput target c args instr
+execDocPipe :: String -> [String] -> Doc -> IO Doc
+execDocPipe c args instr = withoutProgress $ do
+       (pid, mvare, out) <- execAndGetOutput c args instr
        rval <- waitForProcess pid
        takeMVar mvare
        case rval of
@@ -327,9 +323,9 @@ execDocPipe target c args instr = withoutProgress $ do
 
 -- The following is needed for diff, which returns non-zero whenever
 -- the files differ.
-execPipeIgnoreError :: RenderMode -> String -> [String] -> Doc -> IO Doc
-execPipeIgnoreError target c args instr = withoutProgress $ do
-       (pid, mvare, out) <- execAndGetOutput target c args instr
+execPipeIgnoreError :: String -> [String] -> Doc -> IO Doc
+execPipeIgnoreError c args instr = withoutProgress $ do
+       (pid, mvare, out) <- execAndGetOutput c args instr
        _ <- waitForProcess pid
        takeMVar mvare
        return $ if B.null out then empty else packedString out
@@ -341,7 +337,7 @@ signString (SignAs keyid) d = signPGP ["--local-user", keyid] d
 signString (SignSSL idf) d = signSSL idf d
 
 signPGP :: [String] -> Doc -> IO Doc
-signPGP args = execDocPipe Standard "gpg" ("--clearsign":args)
+signPGP args = execDocPipe "gpg" ("--clearsign":args)
 
 signSSL :: String -> Doc -> IO Doc
 signSSL idfile t =
@@ -356,7 +352,7 @@ signSSL idfile t =
                 >>= B.writeFile cert
     opensslDoc ["smime", "-sign", "-signer", cert,
                 "-inkey", idfile, "-noattr", "-text"] t
-    where opensslDoc = execDocPipe Standard "openssl"
+    where opensslDoc = execDocPipe "openssl"
           opensslPS = execPSPipe "openssl"
 
 
@@ -418,27 +414,27 @@ verifySSL goodkeys s = do
     where opensslPS = execPSPipe "openssl"
 
 viewDoc :: Doc -> IO ()
-viewDoc = viewDocWith simplePrinters Encode
+viewDoc = viewDocWith simplePrinters
 
-viewDocWith :: Printers -> RenderMode -> Doc -> IO ()
-viewDocWith pr mode msg = do
+viewDocWith :: Printers -> Doc -> IO ()
+viewDocWith pr msg = do
   isTerminal <- hIsTerminalDevice stdout
-  void $ if isTerminal && lengthGreaterThan (20 :: Int) (lines $ renderString mode msg)
+  void $ if isTerminal && lengthGreaterThan (20 :: Int) (lines $ renderString msg)
      then do mbViewerPlusArgs <- getViewer
              case mbViewerPlusArgs of
                   Just viewerPlusArgs -> do
                     let (viewer : args) = words viewerPlusArgs
-                    pipeDocToPager viewer args pr mode msg
+                    pipeDocToPager viewer args pr msg
                   Nothing -> return $ ExitFailure 127 -- No such command
                -- TEMPORARY passing the -K option should be removed as soon as
                -- we can use the delegate_ctrl_c feature in process
-               `ortryrunning` pipeDocToPager  "less" ["-RK"] pr mode msg
-               `ortryrunning` pipeDocToPager  "more" [] pr mode msg
+               `ortryrunning` pipeDocToPager  "less" ["-RK"] pr msg
+               `ortryrunning` pipeDocToPager  "more" [] pr msg
 #ifdef WIN32
-               `ortryrunning` pipeDocToPager  "more.com" [] pr mode msg
+               `ortryrunning` pipeDocToPager  "more.com" [] pr msg
 #endif
-               `ortryrunning` pipeDocToPager "" [] pr mode msg
-     else pipeDocToPager "" [] pr mode msg
+               `ortryrunning` pipeDocToPager "" [] pr msg
+     else pipeDocToPager "" [] pr msg
               where lengthGreaterThan n _ | n <= 0 = True
                     lengthGreaterThan _ [] = False
                     lengthGreaterThan n (_:xs) = lengthGreaterThan (n-1) xs
@@ -448,13 +444,13 @@ getViewer = Just `fmap` (getEnv "DARCS_PAGER" `catchall` getEnv "PAGER")
             `catchall`
             return Nothing
 
-pipeDocToPager :: String -> [String] -> Printers -> RenderMode -> Doc -> IO ExitCode
+pipeDocToPager :: String -> [String] -> Printers -> Doc -> IO ExitCode
 
-pipeDocToPager "" _ pr mode inp = do
-  hPutDocLnWith pr mode stdout inp
+pipeDocToPager "" _ pr inp = do
+  hPutDocLnWith pr stdout inp
   return ExitSuccess
 
-pipeDocToPager c args pr mode inp = pipeDocInternal (PipeToOther pr) mode c args inp
+pipeDocToPager c args pr inp = pipeDocInternal (PipeToOther pr) c args inp
 
 -- | Given two shell commands as arguments, execute the former.  The
 -- latter is then executed if the former failed because the executable
@@ -522,7 +518,6 @@ runEditor f = do
 
 getEditor :: IO String
 getEditor = getEnv "DARCS_EDITOR" `catchall`
-             getEnv "DARCSEDITOR" `catchall`
              getEnv "VISUAL" `catchall`
              getEnv "EDITOR" `catchall` return "nano"
 
@@ -532,37 +527,45 @@ catchall :: IO a
 a `catchall` b = a `catchNonSignal` (\_ -> b)
 
 
--- | In some environments, darcs requires that certain global GHC library variables that
--- control the encoding used in internal translations are set to specific values.
+-- | On Posix systems, GHC by default uses the user's locale encoding to
+-- determine how to decode/encode the raw byte sequences in the Posix API
+-- to/from 'String'. It also uses certain special variants of this
+-- encoding to determine how to handle encoding errors.
 --
--- @setDarcsEncoding@ enforces those settings, and should be called before the
+-- See "GHC.IO.Encoding" for details.
+--
+-- In particular, the default variant used for command line arguments and
+-- environment variables is //ROUNDTRIP, which means that /any/ byte sequence
+-- can be decoded and re-encoded w/o failure or loss of information. To
+-- enable this, GHC uses code points that are outside the range of the regular
+-- unicode set. This is what you get with 'getFileSystemEncoding'.
+--
+-- We need to preserve the raw bytes e.g. for file names passed in by the
+-- user and also when reading file names from disk; also when re-generating
+-- files from patches, and when we display them to the user.
+--
+-- So we want to use this encoding variant for *all* IO and for (almost) all
+-- conversions between raw bytes and 'String's. The encoding used for IO from
+-- and to handles is controlled by 'setLocaleEncoding' which we use here to
+-- make it equal to the //ROUNDTRIP variant.
+--
+-- @setDarcsEncoding@ should be called before the
 -- first time any darcs operation is run, and again if anything else might have
 -- set those encodings to different values.
 --
 -- Note that it isn't thread-safe and has a global effect on your program.
 --
--- The current behaviour of this function is as follows, though this may
--- change in future:
---
--- Encodings are only set on GHC 7.4 and up, on any non-Windows platform.
---
--- Two encodings are set, both to @GHC.IO.Encoding.char8@:
--- @GHC.IO.Encoding.setFileSystemEncoding@ and @GHC.IO.Encoding.setForeignEncoding@.
---
--- Prevent HLint from warning us about a redundant do if the macro isn't
--- defined:
+-- On Windows, this function does (and should) not do anything.
 setDarcsEncodings :: IO ()
+#ifdef WIN32
+setDarcsEncodings = return ()
+#else
 setDarcsEncodings = do
-
--- This is needed for appropriate behaviour from getArgs and from general
--- filesystem calls (e.g. getDirectoryContents, readFile, ...)
-    setFileSystemEncoding char8
-
--- This ensures that foreign calls made by hashed-storage to stat
--- filenames returned from getDirectoryContents are translated appropriately
-    setForeignEncoding char8
-
-    return ()
+    e <- getFileSystemEncoding
+    -- TODO check if we have to set this, too.
+    setForeignEncoding e
+    setLocaleEncoding e
+#endif
 
 -- The following functions are copied from the encoding package (BSD3
 -- licence, by Henning Günther).
