@@ -16,7 +16,7 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | LeaveTestDir | NoLeaveTestDir
                | Timings | Debug | DebugHTTP
                | Verbose | NormalVerbosity | Quiet
-               | Target String | Cc String
+               | To String | Cc String
                | Output AbsolutePathOrStd | OutputAutoName AbsolutePath | Mail
                | Subject String | InReplyTo String | Charset String
                | SendmailCmd String | Author String | SelectAuthor | PatchName String
@@ -24,7 +24,7 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | OneHash String
                | AfterPatch String | UpToPatch String
                | AfterHash String | UpToHash String
-               | TagName String | LastN Int | MaxCount Int | PatchIndexRange Int Int
+               | TagName String | LastN Int | MaxCount String | PatchIndexRange Int Int
                | NumberPatches
                | OneTag String | AfterTag String | UpToTag String
                | GenContext | Context AbsolutePath | Count
@@ -95,4 +95,5 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | NoAmendUnrecord | AmendUnrecord
                | PatchIndexFlag
                | NoPatchIndexFlag
+               | EnumPatches | NoEnumPatches
                  deriving ( Eq, Show )
