@@ -1,6 +1,9 @@
 -- Support for @darcs help markdown@
 module Darcs.UI.Options.Markdown ( optionsMarkdown ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Data.Functor.Compose ( Compose(..) )
 import System.Console.GetOpt ( OptDescr(..), ArgDescr(..) )
 import Darcs.UI.Options.Util ( DarcsOptDescr )
