@@ -21,14 +21,17 @@ module Darcs.UI.Options.Matching
     , matchSeveralOrLast
     , matchRange
     , matchSeveralOrRange
-    , matchAny -- temporary, for toMatchFlags
+    -- * exported for for checking
+    , context
+    , matchLast
+    , matchFrom
+    , matchAny -- temporary hack
     ) where
 
 import Prelude ()
 import Darcs.Prelude hiding ( last )
 
 import Data.Char ( isDigit )
-import Data.Monoid ( (<>) )
 
 import Darcs.Patch.Match ( MatchFlag(..) )
 import qualified Darcs.UI.Options.Flags as F ( DarcsFlag(..) )
@@ -59,10 +62,13 @@ matchOneContext = mconcat [toMatch, toPatch, toHash, tag, context]
 matchOneNontag :: MatchOption
 matchOneNontag =  match <> patch <> hash
 
--- | Used by: rebase pull, apply, send, push, pull, fetch
+-- | Used by: rebase pull/apply, send, push, pull, apply, fetch
 matchSeveral :: MatchOption
 matchSeveral = matches <> patches <> tags <> hash
 
+matchLast :: MatchOption
+matchLast = last
+
 -- | Used by: rebase unsuspend/reify
 matchSeveralOrFirst :: MatchOption
 matchSeveralOrFirst = mconcat [ matchTo, last, matches, patches, tags, hash ]
@@ -89,7 +95,7 @@ matchFrom = fromMatch <> fromPatch <> fromHash <> fromTag
 matchAny :: MatchOption
 matchAny = mconcat [ toMatch, toPatch, toHash, toTag,
   fromMatch, fromPatch, fromHash, fromTag,
-  tag, tags, patch, patches, match, matches, index, indexes, context, last ]
+  tag, tags, patch, patches, hash, match, matches, index, indexes, context, last ]
 
 -- * Primitive matching options
 
@@ -220,7 +226,7 @@ last = OptSpec {..} where
     "select the last NUMBER patches" ]
   toInt s = if not (null s) && all isDigit s then read s else (-1)
 
--- | TODO: see 'Darcs.UI.Options.matchMaxcount'.
+-- | TODO: see 'Darcs.UI.Options.maxCount'.
 index = OptSpec {..} where
   ounparse k mfs = k [ F.PatchIndexRange n m | PatchIndexRange n m <- mfs ]
   oparse k fs = k [ PatchIndexRange n m | F.PatchIndexRange n m <- fs ]
@@ -230,7 +236,7 @@ index = OptSpec {..} where
                  then F.PatchIndexRange (read s) (read s)
                  else F.PatchIndexRange 0 0
 
--- | TODO: see 'Darcs.UI.Options.matchMaxcount'.
+-- | TODO: see 'Darcs.UI.Options.maxCount'.
 indexes = OptSpec {..} where
   ounparse k mfs = k [ F.PatchIndexRange n m | PatchIndexRange n m <- mfs ]
   oparse k fs = k [ PatchIndexRange n m | F.PatchIndexRange n m <- fs ]
