@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-
 --  Copyright (C) 2002,2008-2009 David Roundy
 --
 --  This program is free software; you can redistribute it and/or modify
@@ -35,8 +33,6 @@ import qualified Data.Map.Strict as M
 import qualified Data.Hashable as H ( hash )
 import Darcs.Util.Diff.Myers (initP, aLen, PArray, getSlice)
 
-#include "impossible.h"
-
 empty :: HunkMap
 empty = HunkMapInfo 0 M.empty
 
