@@ -50,7 +50,7 @@ import System.Process ( runInteractiveProcess, readProcessWithExitCode )
 
 import Darcs.Util.SignalHandler ( catchNonSignal )
 import Darcs.Util.URL ( SshFilePath, sshFilePathOf, sshUhost, sshRepo, sshFile )
-import Darcs.Util.Text ( breakCommand )
+import Darcs.Util.Text ( breakCommand, showCommandLine )
 import Darcs.Util.Exception ( prettyException, catchall )
 import Darcs.Util.Exec ( readInteractiveProcess, ExecException(..), Redirect(AsIs) )
 import Darcs.Util.Progress ( withoutProgress, debugMessage, debugFail )
@@ -177,9 +177,9 @@ newSshConnection :: String -> SshFilePath -> IO (Maybe Connection)
 newSshConnection rdarcs sshfp = do
   (sshcmd,sshargs_) <- getSSH SSH
   debugMessage $ "Starting new ssh connection to " ++ sshUhost sshfp
-  let sshargs = sshargs_ ++ [sshUhost sshfp, rdarcs,
-                             "transfer-mode","--repodir",sshRepo sshfp]
-  debugMessage $ unwords (sshcmd : sshargs)
+  let sshargs = sshargs_ ++ ["--", sshUhost sshfp, rdarcs,
+                             "transfer-mode", "--repodir", sshRepo sshfp]
+  debugMessage $ "Exec: " ++ showCommandLine (sshcmd:sshargs)
   (i,o,e,_) <- runInteractiveProcess sshcmd sshargs Nothing Nothing
   do
     hSetBinaryMode i True
@@ -245,7 +245,8 @@ copySSH rdarcs src dest = do
         -- remote 'darcs transfer-mode' does not work => use scp
         let u = escape_dollar $ sshFilePathOf src
         (scpcmd, args) <- getSSH SCP
-        let scp_args = filter (/="-q") args ++ [u, dest]
+        let scp_args = filter (/="-q") args ++ ["--", u, dest]
+        debugMessage $ "Exec: " ++ showCommandLine (scpcmd:scp_args)
         (r, scp_err) <- readInteractiveProcess scpcmd scp_args
         unless (r == ExitSuccess) $
           throwIO $ ExecException scpcmd scp_args (AsIs,AsIs,AsIs) scp_err
