@@ -13,6 +13,7 @@ module Darcs.Util.Text
     , breakCommand
     , quote
     , pathlist
+    , showCommandLine
     ) where
 
 import Prelude ()
@@ -21,7 +22,7 @@ import Darcs.Prelude
 import Control.Arrow ( first )
 import Data.List ( intercalate )
 
-import Darcs.Util.Printer ( Doc, (<>), renderString, RenderMode(..), quoted, hsep )
+import Darcs.Util.Printer ( Doc, (<>), renderString, quoted, hsep )
 
 sentence :: Doc -> Doc
 sentence = (<> ".")
@@ -56,7 +57,7 @@ chompTrailingNewline s = if last s == '\n' then init s else s
 
 -- | Quote a string for screen output.
 quote :: String -> String
-quote = renderString Encode . quoted
+quote = renderString . quoted
 
 -- | Format a list of 'FilePath's as quoted text. It deliberately refuses to
 -- use English.andClauses but rather separates the quoted strings only with a
@@ -64,3 +65,10 @@ quote = renderString Encode . quoted
 -- another shell command.
 pathlist :: [FilePath] -> Doc
 pathlist paths = hsep (map quoted paths)
+
+-- | Produce a String composed by the elements of [String] each enclosed in
+-- double quotes.
+showCommandLine :: [String] -> String
+showCommandLine strings = showCommandLine' ['"'] strings
+            where showCommandLine' x xs =
+                        x ++ intercalate (x ++ " " ++ x) xs ++ x
