@@ -417,9 +417,11 @@ partiallyUpdateTree :: (Monad m) => (TreeItem m -> m (TreeItem m))
 partiallyUpdateTree fun predi t' = go (AnchoredPath []) t'
   where go path t = do
           items' <- M.fromList <$> mapM (maybeupdate path) (listImmediate t)
-          SubTree t'' <- fun . SubTree $ t { items = items'
-                                          , treeHash = NoHash }
-          return t''
+          subtree <- fun . SubTree $ t { items = items'
+                                       , treeHash = NoHash }
+          case subtree of
+            SubTree t'' -> return t''
+            _ -> bug "function passed to partiallyUpdateTree didn't changed SubTree to something else"
         maybeupdate path (k, item) = if predi (path `appendPath` k) item
           then update (path `appendPath` k) (k, item)
           else return (k, item)
