@@ -26,6 +26,17 @@ cd temp1
 echo | darcs pull -a ../temp2 --external-merge 'cp %2 %o'
 cd ..
 
+grep "Part 2" temp1/child_of_conflict
 diff -u temp1/child_of_conflict temp2/child_of_conflict
 
+cd temp1
+darcs wh
+darcs rev -a
+echo y | darcs unpull --last 1 -a
+echo | darcs pull -a ../temp2 --external-merge 'cp %1 %o'
+cd ..
+
+cat temp1/child_of_conflict
+grep "Part 1" temp1/child_of_conflict
+
 rm -rf temp1 temp2
