@@ -4,7 +4,7 @@
 # Set up a base repo. Our experiment will start from this point
 mkdir base
 cd base
-darcs init --darcs-2
+darcs init
 printf "Line1\nLine2\nLine3\n" > foo
 darcs rec -alm Base
 cd ..
