@@ -31,7 +31,7 @@
 rm -rf R                        # Another script may have left a mess.
 
 ## Use the first UTF-8 locale we can find.
-export LC_ALL=$(locale -a | egrep 'utf8|UTF-8' | head -1)
+export LC_ALL=$(locale -a | egrep --text 'utf8|UTF-8' | head -1)
 export DARCS_DONT_ESCAPE_ANYTHING=True
 
 ## If LC_ALL is the empty string, this system doesn't support UTF-8.
