@@ -43,7 +43,7 @@ bar
 bar
 bar
 EOF
-echo yyy | darcs amend-record --look-for-replaces
+echo y | darcs amend-record --look-for-replaces -a
 cd ..
 rm -rf R
 
