@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 DARCS_EDITOR=echo
 export DARCS_EDITOR
@@ -19,10 +19,10 @@ darcs add foo bar
 darcs record -a -m add_foo_bar -A x
 
 # Test that --cc is also printed as recipient in case of success
-darcs send --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>&1|grep -i foo@example.com
+darcs send --mail --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>&1|grep -i foo@example.com
 
 # Test that --cc is also printed as recipient in case of error
-darcs send --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>&1|grep -i foo@example.com
+darcs send --mail --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>&1|grep -i foo@example.com
 
 cd ..
 rm -rf temp1 temp2
