@@ -1,6 +1,6 @@
 #!/bin/sh
 
-set -ev
+. ./lib
 
 not () { "$@" && exit 1 || :; }
 
