@@ -2,10 +2,8 @@
 
 # http://bugs.darcs.net/issue803: Darcs 2.0 regression on manual renames
 
-set -ev
 
-## I would use the builtin !, but that has the wrong semantics.
-not () { "$@" && exit 1 || :; }
+. lib
 
 rm -rf temp
 mkdir temp
