@@ -36,7 +36,7 @@
 [ -n "$(fgrep  'ssh: Could not resolve hostname invalid: Name or service not known' log)" ]
 
 # HTTP repo
-( darcs clone http://www.bogus.domain.so.it.will.surely.fail.com || true ) 2>&1 | tee log
+( http_proxy= darcs clone http://www.bogus.domain.so.it.will.surely.fail.com || true ) 2>&1 | tee log
 egrep 'CouldNotResolveHost|host lookup failure' log
 
 # local repos are tested by tests/issue1932-colon-breaks-add.sh
