@@ -4,7 +4,6 @@
 
 . lib
 
-rm -rf temp1
 mkdir temp1
 cd temp1
 darcs init
@@ -15,7 +14,7 @@ darcs rec -a -m add
 env
 # clear all output formating environment variables
 for e in DARCS_DONT_ESCAPE_ISPRINT DARCS_USE_ISPRINT\
-         DARCS_DONT_ESCAPE_8BIT\
+         DARCS_ESCAPE_8BIT\
          DARCS_DONT_ESCAPE_EXTRA DARCS_ESCAPE_EXTRA\
          DARCS_DONT_ESCAPE_TRAILING_SPACES\
          DARCS_DONT_COLOR DARCS_ALWAYS_COLOR DARCS_ALTERNATIVE_COLOR\
@@ -30,7 +29,7 @@ export LC_ALL=C
 test_line () {
     rm -f a
     echo $1 > a
-    darcs whatsnew | fgrep $2
+    darcs whatsnew | grep -F $2
 }
 
 
@@ -62,6 +61,11 @@ test_line $(printf '\x1C') '[_^\_]'
 test_line $(printf '\x1D') '[_^]_]'
 test_line $(printf '\x1E') '[_^^_]'
 test_line $(printf '\x1F') '[_^__]'
+
+# everything up to here is escaped by default;
+# for the rest we must explicitly set it:
+export DARCS_ESCAPE_8BIT=1
+
 test_line $(printf '\x7F') '[_^?_]'
 # other chars are escaped with <U+XXXX>
 test_line $(printf '\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8A\x8B\x8C\x8D\x8E\x8F')\
@@ -80,4 +84,3 @@ test_line $(printf '\xE0\xE1\xE2\xE3\xE4\xE5\xE6\xE7\xE8\xE9\xEA\xEB\xEC\xED\xEE
           '[_<U+00E0>_][_<U+00E1>_][_<U+00E2>_][_<U+00E3>_][_<U+00E4>_][_<U+00E5>_][_<U+00E6>_][_<U+00E7>_][_<U+00E8>_][_<U+00E9>_][_<U+00EA>_][_<U+00EB>_][_<U+00EC>_][_<U+00ED>_][_<U+00EE>_][_<U+00EF>_]'
 test_line $(printf '\xF0\xF1\xF2\xF3\xF4\xF5\xF6\xF7\xF8\xF9\xFA\xFB\xFC\xFD\xFE\xFF')\
           '[_<U+00F0>_][_<U+00F1>_][_<U+00F2>_][_<U+00F3>_][_<U+00F4>_][_<U+00F5>_][_<U+00F6>_][_<U+00F7>_][_<U+00F8>_][_<U+00F9>_][_<U+00FA>_][_<U+00FB>_][_<U+00FC>_][_<U+00FD>_][_<U+00FE>_][_<U+00FF>_]'
-rm -rf temp1
