@@ -1,7 +1,5 @@
 #!/usr/bin/env bash
 
-#pragma repo-format darcs-1,darcs-2
-
 . lib
 
 rm -rf garbage future
@@ -13,7 +11,7 @@ cd ..
 
 mkdir future
 cd future
-darcs  init
+darcs init
 touch titi
 darcs add titi
 darcs record -am titi
@@ -68,9 +66,7 @@ rm -rf garbage
 #  alternative of a line of format
 #  only look at future vs darcs2
 
-if grep darcs-1 .darcs/defaults; then
-    exit 200
-fi
+skip-formats darcs-1
 
 # get future repo: ok
 # --to-match is needed because of bug###
