@@ -63,3 +63,18 @@ cat pullresult
 test ! -s pullresult
 
 rm -rf temp1 temp2 temp3
+
+
+# issue1749 - darcs remove <dir> corrupts the patch sequence
+darcs init temp1
+cd temp1
+mkdir dir
+touch dir/file
+darcs rec -lam "add dir and file"
+not darcs remove dir
+# removed dir but not file - should be nothing to add
+not darcs rec -a -m"remove dir"
+darcs obliterate -a --patch "remove dir" | grep 'No patches selected!'
+darcs check
+cd ..
+rm -rf temp1
