@@ -24,11 +24,9 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-#pragma repo-format darcs-1
-
 . lib                  # Load some portability helpers.
 
-grep darcs-1 $HOME/.darcs/defaults || exit 200
+skip-formats darcs-2
 
 rm -rf empty
 mkdir empty
