@@ -17,3 +17,12 @@ darcs changes --last 1 > log
 grep tagged log
 cd ..
 rm -rf temp1 log
+
+# issue2244: warn about duplicate tags
+
+darcs init temp1
+cd temp1
+darcs tag shouldbeunique
+darcs tag shouldbeunique | grep 'WARNING'
+cd ..
+rm -rf temp1
